@@ -21,6 +21,7 @@
 #include "config.h"
 
 #include <QtCore/QString>
+#include <QtGui/QScrollArea>
 #include <QtGui/QVBoxLayout>
 #include <QtGui/QWidget>
 
@@ -53,7 +54,7 @@ namespace Kwave
      * | m_upper_dock                                                          |
      * |-----------------------------------------------------------------------|
      * | /- hbox ------------------------------------------------------------\ |
-     * | +--- m_view_port--------------------------------------------------+-| |
+     * | +--- m_scroll_area------------------------------------------------+-| |
      * | |/---- m_signal_widget-------------------------------------------\|^| |
      * | ||          |                                                    ||#| |
      * | || controls |  SignalView                                        ||#| |
@@ -97,7 +98,7 @@ namespace Kwave
 	double zoom() const;
 
 	/** Returns the width of the current view in pixels */
-	int displayWidth() const;
+	int viewPortWidth() const;
 
 	/** Returns the width of the current view in samples */
 	sample_index_t displaySamples() const;
@@ -117,16 +118,13 @@ namespace Kwave
 	 * Called if the main widget has been resized and resizes/moves
 	 * the signal widget and the channel controls
 	 */
-	virtual void resizeEvent(QResizeEvent *);
+	virtual void resizeEvent(QResizeEvent *event);
 
 	/** slot for mouse wheel events, for scrolling/zooming */
 	virtual void wheelEvent(QWheelEvent *event);
 
     protected slots:
 
-	/** resize the viewport and its contents */
-	void resizeViewPort();
-
 	/** updates all widgets that depend on the current view range */
 	void updateViewRange();
 
@@ -181,16 +179,18 @@ namespace Kwave
 	void zoomNormal();
 
 	/**
-	 * Zooms into the signal, the new display will show the middle
-	 * 33% of the current display.
+	 * Zooms into the signal, the new display will show the signal
+	 * zoomed at the position given as parameter (if >= 0) or centered
+	 * if a position < 0 is given.
 	 */
-	void zoomIn();
+	void zoomIn(int pos = -1);
 
 	/**
-	 * Zooms the signal out, the current display will become the
-	 * middle 30% of the new display.
+	 * Zooms out the signal, the new display will show the signal
+	 * zoomed at the position given as parameter (if >= 0) or centered
+	 * if a position < 0 is given.
 	 */
-	void zoomOut();
+	void zoomOut(int pos = -1);
 
     private slots:
 
@@ -208,17 +208,11 @@ namespace Kwave
 	 * Called if a track has been deleted. Updates the display by
 	 * resizing/re-positioning the signal views.
 	 * @param index the index of the inserted track [0...tracks-1]
+	 * @param track pointer to the track object (ignored here)
 	 * @see SignalManager::sigTrackDeleted
 	 * @internal
 	 */
-	void slotTrackDeleted(unsigned int index);
-
-	/**
-	 * Connected to the vertical scrollbar and called if the value
-	 * has changed so that the signal display and the channel
-	 * controls have to be moved.
-	 */
-	void verticalScrollBarMoved(int newval);
+	void slotTrackDeleted(unsigned int index, Kwave::Track *track);
 
 	/** refresh the scale and position of the horizontal scrollbar */
 	void refreshHorizontalScrollBar();
@@ -312,7 +306,10 @@ namespace Kwave
 	QVBoxLayout m_lower_dock;
 
 	/** container widget that contains the signal widget. */
-	QWidget m_view_port;
+	QScrollArea m_scroll_area;
+
+	/** horizontal scrollbar */
+	QScrollBar *m_horizontal_scrollbar;
 
 	/** the widget that shows the signal, scrolled within the view port */
 	Kwave::SignalWidget m_signal_widget;
@@ -320,21 +317,12 @@ namespace Kwave
 	/** overview widget */
 	Kwave::OverViewWidget *m_overview;
 
-	/** vertical scrollbar, only visible if tracks do not fit vertically */
-	QScrollBar *m_vertical_scrollbar;
-
-	/** horizontal scrollbar, always visible */
-	QScrollBar *m_horizontal_scrollbar;
-
 	/**
-	* Offset from which signal is beeing displayed. This is equal to
+	* Offset from which signal is being displayed. This is equal to
 	* the index of the first visible sample.
 	*/
 	sample_index_t m_offset;
 
-	/** width of the widget in pixels, cached value */
-	int m_width;
-
 	/** number of samples per pixel */
 	double m_zoom;
 
