@@ -58,6 +58,7 @@
 #include "libkwave/Encoder.h"
 #include "libkwave/Plugin.h" // for some helper functions
 #include "libkwave/LabelList.h"
+#include "libkwave/Logger.h"
 #include "libkwave/MessageBox.h"
 #include "libkwave/MetaDataList.h"
 #include "libkwave/Parser.h"
@@ -101,6 +102,19 @@
 /** returns the number of elements of an array */
 #define ELEMENTS_OF(__x__) (sizeof(__x__) / sizeof(__x__[0]))
 
+//***************************************************************************
+/**
+ * struct for info about a label within a Kwave script
+ * @internal
+ */
+namespace Kwave {
+    typedef struct {
+	qint64       pos;  /**< position within the stream      */
+	unsigned int hits; /**< number of "goto"s to this label */
+    } label_t;
+}
+
+
 //***************************************************************************
 //***************************************************************************
 Kwave::TopWidget::TopWidget(Kwave::App &app)
@@ -520,6 +534,9 @@ int Kwave::TopWidget::executeCommand(const QString &line)
 
     Kwave::PluginManager *plugin_manager = m_context.pluginManager();
 
+    // log all commands to the log file if enabled
+    Kwave::Logger::log(this, Kwave::Logger::Info, _("CMD: ") + line);
+
     // special case: if the command contains ";" it is a list of
     // commands -> macro !
     Kwave::Parser parse_list(command);
@@ -630,7 +647,7 @@ int Kwave::TopWidget::executeCommand(const QString &line)
 	Q_ASSERT(m_menu_manager);
 	if (m_menu_manager) /*result = */m_menu_manager->executeCommand(command);
     CASE_COMMAND("newsignal")
-	sample_index_t samples = parser.toUInt();
+	sample_index_t samples = parser.toSampleIndex();
 	double         rate    = parser.toDouble();
 	unsigned int   bits    = parser.toUInt();
 	unsigned int   tracks  = parser.toUInt();
@@ -681,7 +698,7 @@ int Kwave::TopWidget::executeCommand(const QString &line)
     } else {
 	// try to forward the command to the main widget
 	Q_ASSERT(m_main_widget);
-	return (m_main_widget) ? m_main_widget->executeCommand(command) : -1;
+	result = (m_main_widget) ? m_main_widget->executeCommand(command) : -1;
     }
 
     return result;
@@ -715,7 +732,7 @@ int Kwave::TopWidget::loadBatch(const KUrl &url)
 int Kwave::TopWidget::parseCommands(QTextStream &stream)
 {
     int result = 0;
-    QMap<QString, qint64> labels;
+    QMap<QString, label_t> labels;
 
     // set hourglass cursor
     QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
@@ -733,9 +750,11 @@ int Kwave::TopWidget::parseCommands(QTextStream &stream)
 	    // this line seems to be a "label"
 	    line = line.left(line.length() - 1).simplified();
 	    if (!labels.contains(line)) {
-		qDebug("new label '%s' at %u", DBG(line),
-		       static_cast<unsigned int>(stream.pos()));
-		labels[line] = stream.pos();
+		qDebug("new label '%s' at %llu", DBG(line), stream.pos());
+		label_t label;
+		label.pos  = stream.pos();
+		label.hits = 0;
+		labels[line] = label;
 	    }
 	    continue;
 	}
@@ -747,9 +766,12 @@ int Kwave::TopWidget::parseCommands(QTextStream &stream)
 	    qDebug(">>> detected 'goto'");
 	    QString label = line.split(QLatin1Char(' ')).at(1).simplified();
 	    if (labels.contains(label)) {
-		qDebug(">>> goto '%s' @ offset %u", DBG(label),
-		       static_cast<unsigned int>(labels[label]));
-		stream.seek(labels[label]);
+		labels[label].hits++;
+		qDebug(">>> goto '%s' @ offset %llu (pass #%d)", DBG(label),
+		       labels[label].pos,
+		       labels[label].hits
+  		    );
+		stream.seek(labels[label].pos);
 	    } else {
 		qWarning("label '%s' not found", DBG(label));
 		break;
@@ -998,9 +1020,9 @@ int Kwave::TopWidget::saveFileAs(const QString &filename, bool selection)
 		    QString ext = extensions.first().split(_(" ")).first();
 		    if (ext.length()) {
 			extension = ext;
-			QString filename = current_url.fileName();
-			filename += extension.mid(1); // remove the "*"
-			current_url.setFileName(filename);
+			QString new_filename = current_url.fileName();
+			new_filename += extension.mid(1); // remove the "*"
+			current_url.setFileName(new_filename);
 		    }
 		}
 	    }
@@ -1016,23 +1038,26 @@ int Kwave::TopWidget::saveFileAs(const QString &filename, bool selection)
 	url = dlg.selectedUrl();
 	if (url.isEmpty()) return 0;
 
-	QString name = url.path();
-	QFileInfo path(name);
+	QString new_name = url.path();
+	QFileInfo path(new_name);
 
 	// add the correct extension if necessary
 	if (!path.suffix().length()) {
 	    QString ext = dlg.selectedExtension();
 	    QStringList extensions = ext.split(_(" "));
 	    ext = extensions.first();
-	    name += ext.mid(1);
-	    path = name;
-	    url.setPath(name);
+	    new_name += ext.mid(1);
+	    path = new_name;
+	    url.setPath(new_name);
 	}
     }
 
     // check if the file exists and ask before overwriting it
     // if it is not the old filename
-    if ((name != signalName()) && (QFileInfo(name).exists())) {
+    name = url.path();
+    if ((url.prettyUrl() != KUrl(signalName()).prettyUrl()) &&
+	(QFileInfo(name).exists()))
+    {
 	if (Kwave::MessageBox::warningYesNo(this,
 	    i18n("The file '%1' already exists.\n"
 	         "Do you really want to overwrite it?", name)) !=
@@ -1135,7 +1160,7 @@ void Kwave::TopWidget::selectZoom(int index)
     if (!ok) ms = 1;
 
     const double rate = signal_manager->rate();
-    unsigned int width = m_main_widget->displayWidth();
+    unsigned int width = m_main_widget->viewPortWidth();
     Q_ASSERT(width > 1);
     if (width <= 1) width = 2;
     const double new_zoom = rint(((rate * ms) / 1.0E3) -1 ) /
@@ -1166,7 +1191,7 @@ void Kwave::TopWidget::setZoomInfo(double zoom)
     if ((signal_manager) && (signal_manager->tracks())) {
 	if (rate > 0) {
 	    // time display mode
-	    int s = static_cast<int>(ms) / 1000;
+	    int s = Kwave::toInt(ms) / 1000;
 	    int m = s / 60;
 
 	    if (ms >= 60*1000) {
@@ -1175,7 +1200,7 @@ void Kwave::TopWidget::setZoomInfo(double zoom)
 		strZoom = strZoom.sprintf("%d sec", s);
 	    } else if (ms >= 1) {
 		strZoom = strZoom.sprintf("%d ms",
-		    static_cast<int>(round(ms)));
+		    Kwave::toInt(round(ms)));
 	    } else if (ms >= 0.01) {
 		strZoom = strZoom.sprintf("%0.3g ms", ms);
 	    }
@@ -1201,9 +1226,9 @@ void Kwave::TopWidget::setZoomInfo(double zoom)
 	    if (!v.isValid() || !v.toBool()) {
 		m_zoomselect->removeItem(i);
 	    } else {
-		QVariant v = m_zoomselect->itemData(i, ZOOM_DATA_TIME);
+		QVariant vz = m_zoomselect->itemData(i, ZOOM_DATA_TIME);
 		bool ok = false;
-		double t = v.toDouble(&ok);
+		double t = vz.toDouble(&ok);
 		if (ok && (t > 0) && fabs(1 - (t / ms)) < (1.0 / 60.0)) {
 		    match = i;
 		}
@@ -1253,13 +1278,13 @@ void Kwave::TopWidget::metaDataChanged(Kwave::MetaDataList meta_data)
 
     // length in milliseconds
     if (length) {
-	ms = (rate) ? (static_cast<double>(length) /
+	ms = (rate > 0) ? (static_cast<double>(length) /
 	    static_cast<double>(rate) * 1E3) : 0;
 	txt = _(" ") + i18nc(
 	    "Length, as in total duration of loaded song",
 	    "Length: %1 (%2 samples)",
 	    Kwave::ms2string(ms),
-	    KGlobal::locale()->formatLong(length)
+	    KGlobal::locale()->formatLong(static_cast<long int>(length))
 	) + _(" ");
     } else txt = _("");
     m_lbl_status_size->setText(txt);
@@ -1318,16 +1343,19 @@ void Kwave::TopWidget::selectionChanged(sample_index_t offset,
 	bool sample_mode = false;
 
 	sample_index_t last = offset + ((length) ? length-1 : 0);
-	if (rate == 0) sample_mode = true; // force sample mode if rate==0
+
+	if (qFuzzyIsNull(rate))
+	    sample_mode = true; // force sample mode if rate==0
+
 	QString txt = _(" ");
 	if (sample_mode) {
 	    txt += i18nc(
 	        "%1=first sample, %2=last sample, %3=number of samples, "\
 	        "example: 'Selected: 2000...3000 (1000 samples)'",
 	        "Selected: %1...%2 (%3 samples)",
-	        KGlobal::locale()->formatLong(offset),
-	        KGlobal::locale()->formatLong(last),
-	        KGlobal::locale()->formatLong(length)
+	        KGlobal::locale()->formatLong(static_cast<long int>(offset)),
+	        KGlobal::locale()->formatLong(static_cast<long int>(last)),
+	        KGlobal::locale()->formatLong(static_cast<long int>(length))
 	    );
 	} else {
 	    double ms_first = static_cast<double>(offset)   * 1E3 / rate;
@@ -1351,7 +1379,9 @@ void Kwave::TopWidget::selectionChanged(sample_index_t offset,
 	// Position: 02:00
 	bool sample_mode = false;
 
-	if (rate == 0) sample_mode = true; // force sample mode if rate==0
+	if (qFuzzyIsNull(rate))
+	    sample_mode = true; // force sample mode if rate==0
+
 	if (sample_mode || !signal_manager->tracks()) {
 	    m_lbl_status_cursor->setText(_(""));
 	} else {
@@ -1379,8 +1409,8 @@ void Kwave::TopWidget::updatePlaybackPos(sample_index_t offset)
 	double ms = static_cast<double>(offset) * 1E3 / rate;
 	txt = i18n("Playback: %1", Kwave::ms2string(ms));
     } else {
-	txt = i18n("Playback: %1 samples",
-	            KGlobal::locale()->formatLong(offset));
+	txt = i18n("Playback: %1 samples", KGlobal::locale()->formatLong(
+	           static_cast<long int>(offset)));
     }
     statusBar()->showMessage(txt, 2000);
 
