@@ -112,7 +112,7 @@ void Kwave::SignalView::setTrack(int track)
 //***************************************************************************
 void Kwave::SignalView::setZoomAndOffset(double zoom, sample_index_t offset)
 {
-    if ((zoom == m_zoom) && (offset == m_offset)) return;
+    if (qFuzzyCompare(zoom, m_zoom) && (offset == m_offset)) return;
     m_zoom   = zoom;
     m_offset = offset;
 
@@ -136,7 +136,7 @@ void Kwave::SignalView::setVerticalZoom(double zoom)
 //***************************************************************************
 int Kwave::SignalView::samples2pixels(sample_index_t samples) const
 {
-    return (m_zoom > 0.0) ? (samples / m_zoom) : 0;
+    return Kwave::toInt((m_zoom > 0.0) ? (samples / m_zoom) : 0);
 }
 
 //***************************************************************************
@@ -307,7 +307,7 @@ void Kwave::SignalView::mouseMoveEvent(QMouseEvent *e)
     if (mouse_y < 0) mouse_y = 0;
     if (mouse_x >= width())  mouse_x = width()  - 1;
     if (mouse_y >= height()) mouse_y = height() - 1;
-    QPoint mouse_pos(mouse_x, mouse_y);
+    const QPoint mouse_pos(mouse_x, mouse_y);
 
     // bail out if the position did not change
     static int last_x = -1;
@@ -347,7 +347,6 @@ void Kwave::SignalView::mouseMoveEvent(QMouseEvent *e)
 	const bool     selection_is_empty = (selection_first == selection_last);
 
 	// check whether there is some object near this position
-	const QPoint mouse_pos = QPoint(mouse_x, mouse_y);
 	QSharedPointer<Kwave::ViewItem> item       = findItem(mouse_pos);
 	bool                            item_found = !item.isNull();
 
@@ -657,10 +656,10 @@ void Kwave::SignalView::startDragging()
 	// special case: when dropping into the same widget, before
 	// the previous selection, the previous range has already
 	// been moved to the right !
-	sample_index_t src = first;
-	if ((d->target() == this) && (f < src)) src += len;
+	sample_index_t src_pos = first;
+	if ((d->target() == this) && (f < src_pos)) src_pos += len;
 
-	m_signal_manager->deleteRange(src, len,
+	m_signal_manager->deleteRange(src_pos, len,
 	    m_signal_manager->selectedTracks());
 
 	// restore the new selection
@@ -757,23 +756,18 @@ void Kwave::SignalView::dragMoveEvent(QDragMoveEvent* event)
 	sample_index_t left  = m_signal_manager->selection().first();
 	sample_index_t right = m_signal_manager->selection().last();
 	const sample_index_t w = pixels2samples(width());
-	QRect r(this->rect());
+	QRect rect(this->rect());
 
 	// crop selection to widget borders
 	if (left < m_offset) left = m_offset;
 	if (right > m_offset + w) right = m_offset + w - 1;
 
 	// transform to pixel coordinates
-	left  = samples2pixels(left  - m_offset);
-	right = samples2pixels(right - m_offset);
-	if (right >= static_cast<unsigned int>(width()))
-	    right = width() - 1;
-	if (left > right)
-	    left = right;
-
-	r.setLeft(left);
-	r.setRight(right);
-	event->ignore(r);
+	int l = qMin(samples2pixels(left  - m_offset), width() - 1);
+	int r = qMin(samples2pixels(right - m_offset), l);
+	rect.setLeft(l);
+	rect.setRight(r);
+	event->ignore(rect);
     } else if (Kwave::Drag::canDecode(event->mimeData())) {
 	// accept if it is decodeable within the
 	// current range (if it's outside our own selection)
@@ -909,7 +903,7 @@ void Kwave::SignalView::PositionWidget::updateMask()
     clearMask();
     setMask(bmp);
 
-    // remember size/alignment for detecing changes
+    // remember size/alignment for detecting changes
     m_last_alignment = m_alignment;
     m_last_size      = size();
 }
