@@ -98,25 +98,36 @@ namespace Kwave
 	}
 
 	/** returns the index of the associated track (or -1) */
-	int track() const {
+	inline int track() const {
 	    return m_track_index;
 	}
 
 	/** returns the current start position */
-	sample_index_t offset() const {
+	inline sample_index_t offset() const {
 	    return m_offset;
 	}
 
 	/** returns the current zoom [pixels/sample] */
-	double zoom() const {
+	inline double zoom() const {
 	    return m_zoom;
 	}
 
 	/** returns the current vertical zoom factor */
-	double verticalZoom() const {
+	inline double verticalZoom() const {
 	    return m_vertical_zoom;
 	}
 
+	/** returns the index of the first visible sample */
+	inline sample_index_t firstVisible() const {
+	    return m_offset;
+	}
+
+	/** returns the index of the last visible sample */
+	inline sample_index_t lastVisible() const {
+	    const sample_index_t w = pixels2samples(width());
+	    return m_offset + ((w) ? (w - 1) : 0);
+	}
+
 	/**
 	 * converts a number of samples into a number of pixels,
 	 * based on the current zoom factor
@@ -197,6 +208,9 @@ namespace Kwave
 
     signals:
 
+	/** emitted whenever the size of the content has changed */
+	void contentSizeChanged();
+
 	/**
 	 * Can be emitted to signal that this view needs to be repainted,
 	 * probably after synchronizing with other views and additionally
@@ -327,7 +341,7 @@ namespace Kwave
 	int m_track_index;
 
 	/**
-	 * Offset from which signal is beeing displayed. This is equal to
+	 * Offset from which signal is being displayed. This is equal to
 	 * the index of the first visible sample.
 	 */
 	sample_index_t m_offset;
