@@ -20,6 +20,7 @@
 #include <math.h>
 #include <stdlib.h>
 
+#include <QtGui/QApplication>
 #include <QtGui/QBitmap>
 #include <QtGui/QContextMenuEvent>
 #include <QtGui/QDragLeaveEvent>
@@ -48,6 +49,7 @@
 #include "libkwave/SignalManager.h"
 #include "libkwave/String.h"
 #include "libkwave/Track.h"
+#include "libkwave/Utils.h"
 
 // #include "libgui/ShortcutWrapper.h"
 #include "libgui/SignalView.h"
@@ -69,11 +71,6 @@
 //     Qt::Key_0
 // };
 
-/**
- * useful macro for command parsing
- */
-#define CASE_COMMAND(x) } else if (parser.command() == x) {
-
 /** vertical zoom factor: minimum value */
 #define VERTICAL_ZOOM_MIN 1.0
 
@@ -109,8 +106,8 @@ Kwave::SignalWidget::SignalWidget(QWidget *parent,
 
     connect(sig,  SIGNAL(sigTrackInserted(unsigned int, Kwave::Track *)),
             this, SLOT( slotTrackInserted(unsigned int, Kwave::Track *)));
-    connect(sig,  SIGNAL(sigTrackDeleted(unsigned int)),
-            this, SLOT( slotTrackDeleted(unsigned int)));
+    connect(sig,  SIGNAL(sigTrackDeleted(unsigned int, Kwave::Track *)),
+            this, SLOT( slotTrackDeleted(unsigned int, Kwave::Track *)));
 
     // use a timer for limiting the repaint rate
     m_repaint_timer.setSingleShot(true);
@@ -125,21 +122,14 @@ Kwave::SignalWidget::SignalWidget(QWidget *parent,
 // 	        this, SLOT(parseKey(int)));
 //     }
 
+    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+
     m_layout.setColumnStretch(0,   0);
     m_layout.setColumnStretch(1, 100);
-    m_layout.setMargin(3);
-    m_layout.setSpacing(3);
+    m_layout.setMargin(0);
+    m_layout.setVerticalSpacing(3);
     setLayout(&m_layout);
-
-    setMinimumHeight(200);
-
-//    qDebug("SignalWidget::SignalWidget(): done.");
-}
-
-//***************************************************************************
-bool Kwave::SignalWidget::isOK()
-{
-    return true;
+    m_layout.activate();
 }
 
 //***************************************************************************
@@ -150,6 +140,7 @@ Kwave::SignalWidget::~SignalWidget()
 //***************************************************************************
 void Kwave::SignalWidget::setZoomAndOffset(double zoom, sample_index_t offset)
 {
+    Q_ASSERT(zoom >= 0.0);
     foreach (QPointer<Kwave::SignalView> view, m_views)
 	view->setZoomAndOffset(zoom, offset);
 }
@@ -340,7 +331,7 @@ void Kwave::SignalWidget::wheelEvent(QWheelEvent *event)
     }
 
     if (event->delta() > 0) {
-	// zom in
+	// zoom in
 	setVerticalZoom(m_vertical_zoom  * VERTICAL_ZOOM_STEP_FACTOR);
 	event->accept();
     } else if (event->delta() < 0) {
@@ -358,7 +349,7 @@ void Kwave::SignalWidget::setVerticalZoom(double zoom)
 {
     if (zoom > VERTICAL_ZOOM_MAX) zoom = VERTICAL_ZOOM_MAX;
     if (zoom < VERTICAL_ZOOM_MIN) zoom = VERTICAL_ZOOM_MIN;
-    if (zoom == m_vertical_zoom) return; // no change
+    if (qFuzzyCompare(zoom, m_vertical_zoom)) return; // no change
 
     // take over the zoom factor
     m_vertical_zoom = zoom;
@@ -368,17 +359,20 @@ void Kwave::SignalWidget::setVerticalZoom(double zoom)
 	if (view) view->setVerticalZoom(m_vertical_zoom);
 
     // get back the maximum zoom set by the views
-    double max_zoom = VERTICAL_ZOOM_MIN;
     foreach (QPointer<Kwave::SignalView> view, m_views)
-	if (view && view->verticalZoom() > max_zoom)
-	    max_zoom = view->verticalZoom();
-    if (max_zoom > m_vertical_zoom) m_vertical_zoom = max_zoom;
-
-    emit contentSizeChanged();
+	if (view && view->verticalZoom() > m_vertical_zoom)
+	    m_vertical_zoom = view->verticalZoom();
 }
 
 //***************************************************************************
-int Kwave::SignalWidget::viewPortWidth()
+int Kwave::SignalWidget::mapToViewPort(const QPoint &pos) const
+{
+    if (m_views.isEmpty()) return mapFromGlobal(pos).x(); // if empty
+    return mapFromGlobal(pos).x() - m_layout.cellRect(0, 1).left();
+}
+
+//***************************************************************************
+int Kwave::SignalWidget::viewPortWidth() const
 {
     if (m_views.isEmpty()) return width(); // if empty
     return m_layout.cellRect(0, 1).width();
@@ -412,6 +406,16 @@ void Kwave::SignalWidget::insertRow(int index, Kwave::SignalView *view,
 	// associate the controls to the view, so that when the view
 	// gets removed/deleted, the controls get removed as well
 	view->addSibling(controls);
+
+	m_layout.activate();
+	// NOTE: QLayout::activate() does not really activate the layout
+	//       synchronously as expected. Instead it posts an event to
+	//       invalidate the layout of it's parent widget.
+	//       Unfortunately we need reliable geometry information
+	//       already *now*, which forces us to do the event loop
+	//       right here.
+	qApp->processEvents(QEventLoop::ExcludeUserInputEvents |
+	                    QEventLoop::ExcludeSocketNotifiers);
     }
 }
 
@@ -434,7 +438,6 @@ void Kwave::SignalWidget::deleteRow(int index)
 	    }
 	}
     }
-
 }
 
 //***************************************************************************
@@ -560,7 +563,6 @@ void Kwave::SignalWidget::insertView(Kwave::SignalView *view,
     view->setZoomAndOffset(m_zoom, m_offset);
 
     // connect all signals
-
     QWidget *top_widget = reinterpret_cast<QWidget *>(m_context.topWidget());
     connect(view,       SIGNAL(sigMouseChanged(Kwave::MouseMark::Mode)),
 	    top_widget, SLOT(mouseChanged(Kwave::MouseMark::Mode)));
@@ -572,6 +574,12 @@ void Kwave::SignalWidget::insertView(Kwave::SignalView *view,
     connect(view,       SIGNAL(sigNeedRepaint(Kwave::SignalView *)),
 	    this,       SLOT(requestRepaint(Kwave::SignalView *)));
 
+    connect(view,       SIGNAL(contentSizeChanged()),
+	    this,       SLOT(updateMinimumHeight()));
+    connect(view,       SIGNAL(destroyed(QObject*)),
+	    this,       SLOT(updateMinimumHeight()));
+
+    updateMinimumHeight();
 }
 
 //***************************************************************************
@@ -587,39 +595,42 @@ void Kwave::SignalWidget::slotTrackInserted(unsigned int index,
     if (!controls) return;
 
     // create a new view for the track's signal
-    Kwave::SignalView *view = new Kwave::TrackView(
+    Kwave::SignalView *new_view = new Kwave::TrackView(
 	this, controls, m_context.signalManager(), track);
-    Q_ASSERT(view);
-    if (!view) {
-	if (controls) delete controls;
+    Q_ASSERT(new_view);
+    if (!new_view) {
+	delete controls;
 	return;
     }
 
     // loop over all views and adjust the track index of the following ones
     foreach (QPointer<Kwave::SignalView> view, m_views) {
-	if (view->track() >= static_cast<int>(index))
+	if (view->track() >= Kwave::toInt(index))
 	    view->setTrack(view->track() + 1);
     }
 
     // assign the view to the new track
-    view->setTrack(index);
+    new_view->setTrack(index);
 
-    insertView(view, controls);
+    insertView(new_view, controls);
 }
 
 //***************************************************************************
-void Kwave::SignalWidget::slotTrackDeleted(unsigned int index)
+void Kwave::SignalWidget::slotTrackDeleted(unsigned int index,
+                                           Kwave::Track *track)
 {
+    Q_UNUSED(track);
+
     // loop over all views, delete those that are bound to this track
     // and adjust the index of the following ones
     bool empty = true;
     QMutableListIterator<QPointer<Kwave::SignalView> > it(m_views);
     while (it.hasNext()) {
 	Kwave::SignalView *view = it.next();
-	if (view->track() == static_cast<int>(index)) {
+	if (view->track() == Kwave::toInt(index)) {
 	    it.remove();
 	    delete view;
-	} else if (view->track() > static_cast<int>(index)) {
+	} else if (view->track() > Kwave::toInt(index)) {
 	    view->setTrack(view->track() - 1);
 	    empty = false;
 	} else if (view->track() != -1) {
@@ -631,15 +642,15 @@ void Kwave::SignalWidget::slotTrackDeleted(unsigned int index)
     const int rows = m_layout.rowCount();
     const int cols = m_layout.columnCount();
     for (int row = 0; row < rows; row++) {
-	bool empty = true;
+	bool row_is_empty = true;
 	for (int col = 0; col < cols; col++) {
 	    QLayoutItem *item = m_layout.itemAtPosition(row, col);
 	    if (item) {
-		empty = false;
+		row_is_empty = false;
 		break;
 	    }
 	}
-	if (empty) deleteRow(row);
+	if (row_is_empty) deleteRow(row);
     }
 
     // if there are only views with track() == -1, we are empty,
@@ -650,6 +661,30 @@ void Kwave::SignalWidget::slotTrackDeleted(unsigned int index)
     }
 }
 
+//***************************************************************************
+void Kwave::SignalWidget::updateMinimumHeight()
+{
+    int height = 0;
+
+    const int rows = m_layout.rowCount();
+    const int cols = m_layout.columnCount();
+    for (int row = 0; row < rows; row++) {
+	int h = 0;
+	for (int col = 0; col < cols; col++) {
+	    QLayoutItem *item = m_layout.itemAtPosition(row, col);
+	    QWidget *widget = (item) ? item->widget() : 0;
+	    if (widget) {
+		int min_height = widget->minimumSize().height();
+		if (min_height > h) h = min_height;
+	    }
+	}
+	height += h;
+    }
+    if (rows > 1) height += (rows - 1) * m_layout.verticalSpacing();
+
+    setMinimumHeight(height);
+}
+
 //***************************************************************************
 // void Kwave::SignalWidget::parseKey(int key)
 // {
