@@ -89,12 +89,6 @@ namespace Kwave
 	SignalWidget(QWidget *parent, Kwave::ApplicationContext &context,
 	             QVBoxLayout *upper_dock, QVBoxLayout *lower_dock);
 
-	/**
-	 * Returns true if this instance was successfully initialized, or
-	 * false if something went wrong during initialization.
-	 */
-	bool isOK();
-
 	/** Destructor */
 	virtual ~SignalWidget();
 
@@ -105,12 +99,18 @@ namespace Kwave
 	 */
 	void setZoomAndOffset(double zoom, sample_index_t offset);
 
+	/**
+	 * Returns the x coordinate of an absolute pixel position to a relative
+	 * coordinate within the viewport area (without controls)
+	 */
+	int mapToViewPort(const QPoint &pos) const;
+
 	/**
 	 * Returns the width of the viewport with the signal (without
 	 * controls)
 	 * @return width of the signal area [pixels]
 	 */
-	int viewPortWidth();
+	int viewPortWidth() const;
 
 	/**
 	 * Insert a new signal view into this widget (or the upper/lower
@@ -161,10 +161,11 @@ namespace Kwave
 	/**
 	 * Connected to the signal's sigTrackDeleted.
 	 * @param index numeric index of the deleted track
+	 * @param track reference to the deleted track
 	 * @see Signal::sigTrackInserted
 	 * @internal
 	 */
-	void slotTrackDeleted(unsigned int index);
+	void slotTrackDeleted(unsigned int index, Kwave::Track *track);
 
 	/** context menu: "edit/undo" */
 	void contextMenuEditUndo() { forwardCommand(_("undo()")); }
@@ -201,6 +202,12 @@ namespace Kwave
 	    forwardCommand(_("selectprevlabels()"));
 	}
 
+	/**
+	 * updates the minimum height of the widget
+	 * according to the number of rows
+	 */
+	void updateMinimumHeight();
+
     signals:
 
 	/**
@@ -218,9 +225,6 @@ namespace Kwave
 	 */
 	void sigCommand(const QString &command);
 
-	/** emitted whenever the size of the content has changed */
-	void contentSizeChanged();
-
     protected:
 
 	/** slot for mouse wheel events, used for vertical zoom */
@@ -270,17 +274,11 @@ namespace Kwave
 	QPointer<QVBoxLayout> m_lower_dock;
 
 	/**
-	 * Offset from which signal is beeing displayed. This is equal to
+	 * Offset from which signal is being displayed. This is equal to
 	 * the index of the first visible sample.
 	 */
 	sample_index_t m_offset;
 
-	/** width of the widget in pixels, cached value */
-	int m_width;
-
-	/** height of the widget in pixels, cached value */
-	int m_height;
-
 	/** number of samples per pixel */
 	double m_zoom;
 
@@ -290,12 +288,7 @@ namespace Kwave
 	/** timer for limiting the number of repaints per second */
 	QTimer m_repaint_timer;
 
-	/**
-	 * list of signal views. Contains one entry for each signal view, starting
-	 * with the ones in m_upper_dock, then the ones in m_layout, and at
-	 * the end the ones from m_lower_dock.
-	 * The list is sorted in the order of the appearance in the GUI.
-	 */
+	/** queue with pointers to all signal views that need a repaint */
 	QQueue< QPointer<Kwave::SignalView> > m_repaint_queue;
     };
 }
