@@ -37,7 +37,7 @@ namespace Kwave
     /** handle for memory manager */
     typedef int Handle;
 
-    class MemoryManager
+    class KDE_EXPORT MemoryManager
     {
     public:
 	/** Constructor */
@@ -71,6 +71,12 @@ namespace Kwave
 	 */
 	bool resize(Kwave::Handle handle, size_t size) KDE_EXPORT;
 
+	/**
+	 * Returns the allocated size of the block
+	 * @note this may be more than allocated, can be rounded up
+	 */
+	size_t sizeOf(Kwave::Handle handle) KDE_EXPORT;
+
 	/**
 	 * Frees a block of memory that has been previously allocated with the
 	 * allocate() function.
@@ -84,13 +90,13 @@ namespace Kwave
 	 * Sets the limit of physical memory that can be used.
 	 * @param mb number of whole megabytes of the limit
 	 */
-	void setPhysicalLimit(unsigned int mb) KDE_EXPORT;
+	void setPhysicalLimit(quint64 mb) KDE_EXPORT;
 
 	/**
 	 * Sets the limit of virtual memory that can be used.
 	 * @param mb number of whole megabytes of the limit
 	 */
-	void setVirtualLimit(unsigned int mb) KDE_EXPORT;
+	void setVirtualLimit(quint64 mb) KDE_EXPORT;
 
 	/**
 	 * Sets the directory where swap files should be stored
@@ -102,20 +108,20 @@ namespace Kwave
 	 * Sets the limit of memory that can be used for undo/redo.
 	 * @param mb number of whole megabytes of the limit
 	 */
-	void setUndoLimit(unsigned int mb) KDE_EXPORT;
+	void setUndoLimit(quint64 mb) KDE_EXPORT;
 
 	/**
 	 * Returns the limit of memory that can be used for undo/redo
 	 * in units of whole megabytes
 	 */
-	unsigned int undoLimit() const KDE_EXPORT;
+	quint64 undoLimit() const KDE_EXPORT;
 
 	/**
 	 * Returns the total amount of theoretically available physical
 	 * memory, as the minimum of the totally installed memory and
 	 * ulimit settings.
 	 */
-	unsigned int totalPhysical() KDE_EXPORT;
+	quint64 totalPhysical() KDE_EXPORT;
 
 	/**
 	 * Returns the global instance of the memory manager from the
@@ -165,13 +171,49 @@ namespace Kwave
 	int writeTo(Kwave::Handle handle, unsigned int offset,
 		    const void *buffer, unsigned int length) KDE_EXPORT;
 
+#ifdef DEBUG_MEMORY
+	/** structure used for statistics */
+	typedef struct {
+	    /** physical memory */
+	    struct {
+		quint64 handles; /**< used handles     */
+		quint64 bytes;   /**< allocated bytes  */
+		quint64 limit;   /**< maximum allowed  */
+		quint64 allocs;  /**< number of allocs */
+		quint64 frees;   /**< number of frees  */
+	    } physical;
+
+	    /** swap files */
+	    struct {
+		/** mapped swap files */
+		struct {
+		    quint64 handles; /**< used handles    */
+		    quint64 bytes;   /**< allocated bytes */
+		} mapped;
+		/** swap files mapped (cached) */
+		struct {
+		    quint64 handles; /**< used handles    */
+		    quint64 bytes;   /**< mapped bytes    */
+		} cached;
+		/** unmapped swap files */
+		struct {
+		    quint64 handles; /**< used handles    */
+		    quint64 bytes;   /**< allocated bytes */
+		} unmapped;
+		quint64 limit;  /**< maximum allowed  */
+		quint64 allocs; /**< number of allocs */
+		quint64 frees;  /**< number of frees  */
+	    } swap;
+	} statistic_t;
+#endif
+
     protected:
 
 	/** returns the currently allocated physical memory */
-	size_t physicalUsed() KDE_NO_EXPORT;
+	quint64 physicalUsed() KDE_NO_EXPORT;
 
 	/** returns the currently allocated virtual memory */
-	size_t virtualUsed() KDE_NO_EXPORT;
+	quint64 virtualUsed() KDE_NO_EXPORT;
 
 	/** returns a new swap file name */
 	QString nextSwapFileName(Kwave::Handle handle) KDE_NO_EXPORT;
@@ -235,23 +277,20 @@ namespace Kwave
 
     private:
 
-	/** currently allocated amount of physical memory */
-	unsigned int m_physical_allocated;
-
 	/** limit of the physical memory */
-	unsigned int m_physical_limit;
+	quint64 m_physical_limit;
 
-	/** currently allocated amount of physical memory */
-	unsigned int m_virtual_allocated;
+	/** total maximum physical memory (system dependent) */
+	quint64 m_physical_max;
 
 	/** limit of the virtual memory, 0 = disabled */
-	unsigned int m_virtual_limit;
+	quint64 m_virtual_limit;
 
 	/** path where to store swap files */
 	QDir m_swap_dir;
 
 	/** limit of memory available for undo/redo */
-	unsigned int m_undo_limit;
+	quint64 m_undo_limit;
 
 	/** map of objects in physical memory */
 	Kwave::LRU_Cache<Kwave::Handle, physical_memory_t> m_physical;
@@ -274,6 +313,11 @@ namespace Kwave
 
 	/** last used handle */
 	static Kwave::Handle m_last_handle;
+
+#ifdef DEBUG_MEMORY
+	/** statistics, for debugging */
+	statistic_t m_stats;
+#endif /* DEBUG_MEMORY */
     };
 
 }
