@@ -16,6 +16,8 @@
  ***************************************************************************/
 
 #include "config.h"
+
+#include <new>
 #include <errno.h>
 #include <math.h>
 #include <string.h>
@@ -36,6 +38,7 @@
 #include "libkwave/PluginManager.h"
 #include "libkwave/SignalManager.h"
 #include "libkwave/String.h"
+#include "libkwave/Utils.h"
 #include "libkwave/WorkerThread.h"
 
 #ifdef DEBUG
@@ -92,10 +95,9 @@ Kwave::Plugin::~Plugin()
     close();
 
     // lock usage
-    QMutexLocker lock(&m_usage_lock);
-
+    QMutexLocker lock_usage(&m_usage_lock);
     {
-	QMutexLocker lock(&m_thread_lock);
+	QMutexLocker lock_thread(&m_thread_lock);
 	if (m_thread) {
 	    if (m_thread->isRunning()) m_thread->wait(5000);
 	    if (m_thread->isRunning()) m_thread->stop();
@@ -266,7 +268,8 @@ void Kwave::Plugin::updateProgressTick()
     Q_ASSERT(this->thread() == qApp->thread());
 
     QMutexLocker lock(&m_progress_lock);
-    if (m_progress) m_progress->setValue(m_current_progress);
+    if (m_progress) m_progress->setValue(
+	Kwave::toInt(qBound<double>(0.0, m_current_progress, PROGRESS_MAXIMUM)));
 }
 
 //***************************************************************************
@@ -313,7 +316,7 @@ int Kwave::Plugin::execute(QStringList &params)
     QMutexLocker lock(&m_thread_lock);
     m_stop = false;
 
-    m_thread = new Kwave::WorkerThread(this, QVariant(params));
+    m_thread = new(std::nothrow) Kwave::WorkerThread(this, QVariant(params));
     Q_ASSERT(m_thread);
     if (!m_thread) return -ENOMEM;
 
@@ -452,7 +455,7 @@ double Kwave::Plugin::signalRate()
 //***************************************************************************
 const QList<unsigned int> Kwave::Plugin::selectedTracks()
 {
-    return manager().selectedTracks();
+    return signalManager().selectedTracks();
 }
 
 //***************************************************************************
@@ -472,7 +475,7 @@ sample_index_t Kwave::Plugin::selection(QList<unsigned int> *tracks,
     }
 
     // get the list of selected tracks
-    if (tracks) *tracks = manager().selectedTracks();
+    if (tracks) *tracks = signalManager().selectedTracks();
 
     if (left)  *left  = l;
     if (right) *right = r;
