@@ -282,7 +282,6 @@ bool Kwave::PluginManager::canClose()
     return true;
 }
 
-
 //***************************************************************************
 bool Kwave::PluginManager::onePluginRunning()
 {
@@ -416,12 +415,6 @@ double Kwave::PluginManager::signalRate()
     return m_signal_manager.rate();
 }
 
-//***************************************************************************
-const QList<unsigned int> Kwave::PluginManager::selectedTracks()
-{
-    return m_signal_manager.selectedTracks();
-}
-
 //***************************************************************************
 sample_index_t Kwave::PluginManager::selectionStart()
 {
