@@ -17,6 +17,7 @@
 
 #include "config.h"
 
+#include <new>
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,6 +33,7 @@
 #include "libkwave/MessageBox.h"
 #include "libkwave/Signal.h"
 #include "libkwave/Track.h"
+#include "libkwave/Utils.h"
 #include "libkwave/WindowFunction.h"
 
 //***************************************************************************
@@ -45,7 +47,7 @@ Kwave::Signal::Signal(unsigned int tracks, sample_index_t length)
     :m_tracks(), m_lock_tracks()
 {
     while (tracks--) {
-	appendTrack(length);
+	appendTrack(length, 0);
     }
 }
 
@@ -67,18 +69,19 @@ void Kwave::Signal::close()
 
 //***************************************************************************
 Kwave::Track *Kwave::Signal::insertTrack(unsigned int index,
-                                         sample_index_t length)
+                                         sample_index_t length,
+                                         QUuid *uuid)
 {
     Kwave::Track *t = 0;
     {
 	QWriteLocker lock(&m_lock_tracks);
 
-	t = new Kwave::Track(length);
+	t = new(std::nothrow) Kwave::Track(length, uuid);
 	Q_ASSERT(t);
 	if (!t) return 0;
 
 	// clip the track index
-	if (static_cast<int>(index) > m_tracks.count())
+	if (Kwave::toInt(index) > m_tracks.count())
 	    index = m_tracks.count();
 
 	// insert / append to the list
@@ -109,9 +112,9 @@ Kwave::Track *Kwave::Signal::insertTrack(unsigned int index,
 
 
 //***************************************************************************
-Kwave::Track *Kwave::Signal::appendTrack(sample_index_t length)
+Kwave::Track *Kwave::Signal::appendTrack(sample_index_t length, QUuid *uuid)
 {
-    return insertTrack(tracks(), length);
+    return insertTrack(tracks(), length, uuid);
 }
 
 //***************************************************************************
@@ -121,7 +124,7 @@ void Kwave::Signal::deleteTrack(unsigned int index)
     Kwave::Track *t = 0;
     {
 	QWriteLocker lock(&m_lock_tracks);
-	if (static_cast<int>(index) > m_tracks.count())
+	if (Kwave::toInt(index) > m_tracks.count())
 	    return; // bail out if not in range
 
 	t = m_tracks.at(index);
@@ -131,7 +134,7 @@ void Kwave::Signal::deleteTrack(unsigned int index)
     // now emit a signal that the track has been deleted. Maybe
     // someone is still using a reference to it, so we have not
     // deleted it yet - still only unlinked from the track list!
-    emit sigTrackDeleted(index);
+    emit sigTrackDeleted(index, t);
 
     // as everybody now knows that the track is gone, we can safely
     // delete it now.
@@ -139,13 +142,13 @@ void Kwave::Signal::deleteTrack(unsigned int index)
 }
 
 //***************************************************************************
-Kwave::Writer *Kwave::Signal::openWriter(unsigned int track,
-	Kwave::InsertMode mode, sample_index_t left, sample_index_t right)
+Kwave::Writer *Kwave::Signal::openWriter(Kwave::InsertMode mode,
+	unsigned int track, sample_index_t left, sample_index_t right)
 {
     QReadLocker lock(&m_lock_tracks);
 
-    Q_ASSERT(static_cast<int>(track) < m_tracks.count());
-    if (static_cast<int>(track) >= m_tracks.count()) {
+    Q_ASSERT(Kwave::toInt(track) < m_tracks.count());
+    if (Kwave::toInt(track) >= m_tracks.count()) {
 	return 0; // track does not exist !
     }
 
@@ -155,17 +158,50 @@ Kwave::Writer *Kwave::Signal::openWriter(unsigned int track,
 }
 
 //***************************************************************************
-Kwave::SampleReader *Kwave::Signal::openSampleReader(Kwave::ReaderMode mode,
-	unsigned int track, sample_index_t left, sample_index_t right)
+Kwave::SampleReader *Kwave::Signal::openReader(Kwave::ReaderMode mode,
+                                               unsigned int track,
+                                               sample_index_t left,
+                                               sample_index_t right)
 {
     QReadLocker lock(&m_lock_tracks);
 
-    if (static_cast<int>(track) >= m_tracks.count())
+    if (Kwave::toInt(track) >= m_tracks.count())
 	return 0; // track does not exist !
 
     Kwave::Track *t = m_tracks.at(track);
     Q_ASSERT(t);
-    return (t) ? t->openSampleReader(mode, left, right) : 0;
+    return (t) ? t->openReader(mode, left, right) : 0;
+}
+
+//***************************************************************************
+Kwave::Stripe::List Kwave::Signal::stripes(unsigned int track,
+                                           sample_index_t left,
+                                           sample_index_t right)
+{
+    QReadLocker lock(&m_lock_tracks);
+
+    if (Kwave::toInt(track) < m_tracks.count()) {
+	Kwave::Track *t = m_tracks.at(track);
+	Q_ASSERT(t);
+	if (t) return t->stripes(left, right);
+    }
+    return Kwave::Stripe::List(); // track does not exist !
+}
+
+//***************************************************************************
+bool Kwave::Signal::mergeStripes(const Kwave::Stripe::List &stripes,
+                                 unsigned int track)
+{
+    QReadLocker lock(&m_lock_tracks);
+
+    if (Kwave::toInt(track) >= m_tracks.count())
+	return false;
+
+    Kwave::Track *t = m_tracks.at(track);
+    Q_ASSERT(t);
+    if (!t) return false;
+
+    return t->mergeStripes(stripes);
 }
 
 //***************************************************************************
@@ -183,13 +219,14 @@ QList<unsigned int> Kwave::Signal::allTracks()
 }
 
 //***************************************************************************
-void Kwave::Signal::deleteRange(unsigned int track, sample_index_t offset,
+void Kwave::Signal::deleteRange(unsigned int track,
+                                sample_index_t offset,
                                 sample_index_t length)
 {
     QReadLocker lock(&m_lock_tracks);
 
-    Q_ASSERT(static_cast<int>(track) < m_tracks.count());
-    if (static_cast<int>(track) >= m_tracks.count())
+    Q_ASSERT(Kwave::toInt(track) < m_tracks.count());
+    if (Kwave::toInt(track) >= m_tracks.count())
 	return; // track does not exist !
 
     Kwave::Track *t = m_tracks.at(track);
@@ -203,8 +240,8 @@ void Kwave::Signal::insertSpace(unsigned int track, sample_index_t offset,
 {
     QReadLocker lock(&m_lock_tracks);
 
-    Q_ASSERT(static_cast<int>(track) < m_tracks.count());
-    if (static_cast<int>(track) >= m_tracks.count())
+    Q_ASSERT(Kwave::toInt(track) < m_tracks.count());
+    if (Kwave::toInt(track) >= m_tracks.count())
 	return; // track does not exist !
 
     Kwave::Track *t = m_tracks.at(track);
@@ -238,7 +275,7 @@ bool Kwave::Signal::trackSelected(unsigned int track)
 {
     QReadLocker lock(&m_lock_tracks);
 
-    if (static_cast<int>(track) >= m_tracks.count()) return false;
+    if (Kwave::toInt(track) >= m_tracks.count()) return false;
     if (!m_tracks.at(track)) return false;
 
     return m_tracks.at(track)->selected();
@@ -249,14 +286,26 @@ void Kwave::Signal::selectTrack(unsigned int track, bool select)
 {
     QReadLocker lock(&m_lock_tracks);
 
-    Q_ASSERT(static_cast<int>(track) < m_tracks.count());
-    if (static_cast<int>(track) >= m_tracks.count()) return;
+    Q_ASSERT(Kwave::toInt(track) < m_tracks.count());
+    if (Kwave::toInt(track) >= m_tracks.count()) return;
     Q_ASSERT(m_tracks.at(track));
     if (!m_tracks.at(track)) return;
 
     m_tracks.at(track)->select(select);
 }
 
+//***************************************************************************
+QUuid Kwave::Signal::uuidOfTrack(unsigned int track)
+{
+    QReadLocker lock(&m_lock_tracks);
+
+    if (Kwave::toInt(track) >= m_tracks.count()) return QUuid();
+    Q_ASSERT(m_tracks.at(track));
+    if (!m_tracks.at(track)) return QUuid();
+
+    return m_tracks.at(track)->uuid();
+}
+
 //// now follow the various editing and effects functions
 ////**********************************************************
 //#define MAXPRIME 512
