@@ -25,15 +25,16 @@
 
 #include <QtCore/QReadWriteLock>
 #include <QtCore/QList>
+#include <QtCore/QUuid>
 
 #include <kdemacros.h>
 
 #include "libkwave/InsertMode.h"
 #include "libkwave/ReaderMode.h"
 #include "libkwave/Sample.h"
+#include "libkwave/Stripe.h"
 #include "libkwave/WindowFunction.h"
 
-
 //**********************************************************************
 namespace Kwave
 {
@@ -77,19 +78,24 @@ namespace Kwave
 	 * @param index the position where to insert [0...tracks()]. If the
 	 *        position is at or after the last track, the new track will
 	 *        be appended to the end.
-	 * @param length number of samples of the new track. Optional, if omitted
-	 *        the track will be zero-length.
+	 * @param length number of samples of the new track (zero is allowed)
+	 * @param uuid pointer to a unique ID (optional, can be null)
 	 * @return pointer to the created track. If the length is
 	 *         omitted or zero, the track will have zero length.
 	 */
-	Kwave::Track *insertTrack(unsigned int index, sample_index_t length = 0);
+	Kwave::Track *insertTrack(unsigned int index,
+	                          sample_index_t length,
+	                          QUuid *uuid);
 
 	/**
 	 * Appends a new track to the end of the tracks list, shortcut for
 	 * insertTrack(tracks()-1, length)
 	 * @see insertTrack
+	 * @param length number of samples of the new track (zero is allowed)
+	 * @param uuid pointer to a unique ID (optional, can be null)
 	 */
-	Kwave::Track *appendTrack(sample_index_t length);
+	Kwave::Track *appendTrack(sample_index_t length,
+	                          QUuid *uuid);
 
 	/**
 	 * Deletes a track.
@@ -103,31 +109,55 @@ namespace Kwave
 	QList<unsigned int> allTracks();
 
 	/**
-	 * Opens an input stream for a track, starting at a specified sample
+	 * Opens an output stream for a track, starting at a specified sample
 	 * position.
-	 * @param track index of the track. If the track does not exist, this
-	 *        function will fail and return 0
 	 * @param mode specifies where and how to insert
+	 * @param track index of the track
 	 *  @param left start of the input (only useful in insert and
 	 *             overwrite mode)
 	 * @param right end of the input (only useful with overwrite mode)
+	 * @return a Writer or null pointer if the track does not exist
 	 * @see InsertMode
 	 */
-	Kwave::Writer *openWriter(unsigned int track, Kwave::InsertMode mode,
-	    sample_index_t left = 0, sample_index_t right = 0);
+	Kwave::Writer *openWriter(Kwave::InsertMode mode,
+	                          unsigned int track,
+	                          sample_index_t left = 0,
+	                          sample_index_t right = 0);
 
 	/**
 	 * Opens a stream for reading samples. If the the last position
 	 * is omitted, the value UINT_MAX will be used.
 	 * @param mode a reader mode, see Kwave::ReaderMode
-	 * @param track index of the track. If the track does not exist, this
-	 *        function will fail and return 0
+	 * @param track index of the track
 	 * @param left first offset to be read (default = 0)
 	 * @param right last position to read (default = UINT_MAX)
+	 * @return a SampleReader or null if the track does not exist
 	 */
-	Kwave::SampleReader *openSampleReader(Kwave::ReaderMode mode,
-	    unsigned int track, sample_index_t left = 0,
-	    sample_index_t right = SAMPLE_INDEX_MAX);
+	Kwave::SampleReader *openReader(Kwave::ReaderMode mode,
+	                          unsigned int track,
+	                          sample_index_t left = 0,
+	                          sample_index_t right = SAMPLE_INDEX_MAX);
+
+	/**
+	 * Get a list of stripes that matches a given range of samples
+	 * @param track index of the track
+	 * @param left  offset of the first sample
+	 * @param right offset of the last sample
+	 * @return a list of stripes that cover the given range
+	 *         between left and right
+	 */
+	Kwave::Stripe::List stripes(unsigned int track,
+	                            sample_index_t left = 0,
+	                            sample_index_t right = SAMPLE_INDEX_MAX);
+
+	/**
+	 * Merge a list of stripes into the signal.
+	 * @param stripes list of stripes
+	 * @param track index of the track
+	 * @return true if succeeded, false if failed
+	 */
+	bool mergeStripes(const Kwave::Stripe::List &stripes,
+	                  unsigned int track);
 
 	/**
 	 * Returns the number of tracks.
@@ -140,7 +170,8 @@ namespace Kwave
 	 * @param offset index of the first sample
 	 * @param length number of samples
 	 */
-	void deleteRange(unsigned int track, sample_index_t offset,
+	void deleteRange(unsigned int track,
+	                 sample_index_t offset,
 	                 sample_index_t length);
 
 	/**
@@ -149,7 +180,8 @@ namespace Kwave
 	 * @param offset index of the first sample
 	 * @param length number of samples
 	 */
-	void insertSpace(unsigned int track, sample_index_t offset,
+	void insertSpace(unsigned int track,
+	                 sample_index_t offset,
 	                 sample_index_t length);
 
 	/**
@@ -172,6 +204,14 @@ namespace Kwave
 	 */
 	void selectTrack(unsigned int track, bool select);
 
+	/**
+	 * Returns the uuid of a track
+	 * @param track index of the track [0...tracks-1]
+	 * @return the QUuid of the track or a "null" uuid if the track
+	 *         does not exist
+	 */
+	QUuid uuidOfTrack(unsigned int track);
+
     signals:
 
 	/**
@@ -184,8 +224,9 @@ namespace Kwave
 	/**
 	 * Signals that a track has been deleted.
 	 * @param index position of the deleted track [0...tracks()-1]
+	 * @param track reference to the deleted track
 	 */
-	void sigTrackDeleted(unsigned int index);
+	void sigTrackDeleted(unsigned int index, Kwave::Track *track);
 
 	/**
 	 * Signals that the selection of one of the tracks has changed
@@ -261,24 +302,13 @@ namespace Kwave
     private:
 
 	/**
-	 * Looks up the index of a trackin the track list
+	 * Looks up the index of a track in the track list
 	 * @param track reference to the trac to be looked up
 	 * @returns index of the track [0...tracks()-1] or tracks() if not found
 	 */
 	unsigned int trackIndex(const Kwave::Track *track);
 
-//        //signal modifying functions
-//        void replaceStutter (int, int);
-//        void delayRecursive (int, int);
-//        void delay (int, int);
-//        void movingFilter (Kwave::Filter *filter, int tap,
-//                           Kwave::Curve *points,
-// 	                     int low, int high);
-//
-//        //functions creating a new Object
-//
-//        void fft (int, bool);
-//        void averageFFT (int points, window_function_t windowtype);
+    private:
 
 	/** list of tracks */
 	QList<Kwave::Track *> m_tracks;
