@@ -17,6 +17,7 @@
 
 #include "config.h"
 
+#include <new>
 #include <errno.h>
 #include <limits.h>
 #include <math.h>
@@ -53,6 +54,7 @@
 #include "libkwave/SignalManager.h"
 #include "libkwave/String.h"
 #include "libkwave/Track.h"
+#include "libkwave/Utils.h"
 #include "libkwave/Writer.h"
 #include "libkwave/undo/UndoTransactionGuard.h"
 #include "libkwave/undo/UndoAction.h"
@@ -95,8 +97,8 @@ Kwave::SignalManager::SignalManager(QWidget *parent)
     Kwave::Signal *sig = &m_signal;
     connect(sig, SIGNAL(sigTrackInserted(unsigned int, Kwave::Track *)),
             this, SLOT(slotTrackInserted(unsigned int, Kwave::Track *)));
-    connect(sig, SIGNAL(sigTrackDeleted(unsigned int)),
-            this, SLOT(slotTrackDeleted(unsigned int)));
+    connect(sig, SIGNAL(sigTrackDeleted(unsigned int, Kwave::Track *)),
+            this, SLOT(slotTrackDeleted(unsigned int, Kwave::Track *)));
     connect(sig, SIGNAL(sigTrackSelectionChanged(bool)),
             this,SIGNAL(sigTrackSelectionChanged(bool)));
     connect(sig, SIGNAL(sigSamplesDeleted(unsigned int, sample_index_t,
@@ -170,10 +172,10 @@ int Kwave::SignalManager::loadFile(const KUrl &url)
 	if (!tracks) break;
 
 	for (track = 0; track < tracks; ++track) {
-	    Kwave::Track *t = m_signal.appendTrack(length);
+	    Kwave::Track *t = m_signal.appendTrack(length, 0);
 	    Q_ASSERT(t);
 	    if (!t || (t->length() != length)) {
-		qWarning("out of memory");
+		qWarning("SignalManager::loadFile: out of memory");
 		res = -ENOMEM;
 		break;
 	    }
@@ -188,7 +190,7 @@ int Kwave::SignalManager::loadFile(const KUrl &url)
 
 	// try to calculate the resulting length, but if this is
 	// not possible, we try to use the source length instead
-	unsigned int resulting_size = info.tracks() * info.length() *
+	quint64 resulting_size = info.tracks() * info.length() *
 	                              (info.bits() >> 3);
 	bool use_src_size = (!resulting_size);
 	if (use_src_size) resulting_size = src.size();
@@ -242,7 +244,7 @@ int Kwave::SignalManager::loadFile(const KUrl &url)
 	// enter the filename/mimetype and size into the file info
 	QFileInfo fi(src);
 	info.set(Kwave::INF_FILENAME, fi.absoluteFilePath());
-	info.set(Kwave::INF_FILESIZE, static_cast<unsigned int>(src.size()));
+	info.set(Kwave::INF_FILESIZE, src.size());
 	if (!info.contains(Kwave::INF_MIMETYPE))
 	    info.set(Kwave::INF_MIMETYPE, mimetype);
 
@@ -406,7 +408,7 @@ int Kwave::SignalManager::save(const KUrl &url, bool selection)
 
 	// prepare and show the progress dialog
 	Kwave::FileProgress *dialog = new Kwave::FileProgress(m_parent_widget,
-	    filename, file_info.tracks() * file_info.length() *
+	    filename, file_info.length() * file_info.tracks() *
 	    (file_info.bits() >> 3),
 	    file_info.length(), file_info.rate(), file_info.bits(),
 	    file_info.tracks());
@@ -507,7 +509,7 @@ void Kwave::SignalManager::newSignal(sample_index_t samples, double rate,
     m_empty = false;
 
     // add all empty tracks
-    while (tracks--) m_signal.appendTrack(samples);
+    while (tracks--) m_signal.appendTrack(samples, 0);
 
     // remember the last length
     m_last_length = samples;
@@ -578,11 +580,10 @@ QString Kwave::SignalManager::signalName()
 //***************************************************************************
 const QList<unsigned int> Kwave::SignalManager::selectedTracks()
 {
-    unsigned int track;
     QList<unsigned int> list;
     const unsigned int tracks = this->tracks();
 
-    for (track = 0; track < tracks; track++) {
+    for (unsigned int track = 0; track < tracks; track++) {
 	if (!m_signal.trackSelected(track)) continue;
 	list.append(track);
     }
@@ -596,13 +597,6 @@ const QList<unsigned int> Kwave::SignalManager::allTracks()
     return m_signal.allTracks();
 }
 
-//***************************************************************************
-Kwave::Writer *Kwave::SignalManager::openWriter(unsigned int track,
-	Kwave::InsertMode mode, sample_index_t left, sample_index_t right)
-{
-    return m_signal.openWriter(track, mode, left, right);
-}
-
 //***************************************************************************
 int Kwave::SignalManager::executeCommand(const QString &command)
 {
@@ -618,6 +612,12 @@ int Kwave::SignalManager::executeCommand(const QString &command)
 	undo();
     CASE_COMMAND("redo")
 	redo();
+    CASE_COMMAND("undo_all")
+	while (m_undo_enabled && !m_undo_buffer.isEmpty())
+	    undo();
+    CASE_COMMAND("redo_all")
+	while (!m_redo_buffer.isEmpty())
+	    redo();
 
     // --- copy & paste + clipboard ---
     CASE_COMMAND("copy")
@@ -636,13 +636,13 @@ int Kwave::SignalManager::executeCommand(const QString &command)
 	Kwave::ClipBoard &clip = Kwave::ClipBoard::instance();
 	if (clip.isEmpty()) return 0;
 	if (!selectedTracks().size()) return 0;
-	sample_index_t offset = parser.toUInt();
+	sample_index_t ofs = parser.toSampleIndex();
 
 	Kwave::UndoTransactionGuard undo(*this,
 	                                 i18n("Insert Clipboard at position"));
 
-	selectRange(offset, 0);
-	clip.paste(m_parent_widget, *this, offset, 0);
+	selectRange(ofs, 0);
+	clip.paste(m_parent_widget, *this, ofs, 0);
 
     CASE_COMMAND("paste")
 	Kwave::ClipBoard &clip = Kwave::ClipBoard::instance();
@@ -815,13 +815,13 @@ int Kwave::SignalManager::executeCommand(const QString &command)
     CASE_COMMAND("add_track")
 	appendTrack();
     CASE_COMMAND("delete_track")
-	Kwave::Parser parser(command);
-	unsigned int track = parser.toUInt();
+	Kwave::Parser p(command);
+	unsigned int track = p.toUInt();
 	if (track >= tracks()) return -EINVAL;
 	deleteTrack(track);
     CASE_COMMAND("insert_track")
-	Kwave::Parser parser(command);
-	unsigned int track = parser.toUInt();
+	Kwave::Parser p(command);
+	unsigned int track = p.toUInt();
 	insertTrack(track);
 
     // track selection
@@ -905,7 +905,7 @@ void Kwave::SignalManager::insertTrack(unsigned int index)
 
     // undo action for the track insert
     if (m_undo_enabled && !registerUndoAction(
-	new Kwave::UndoInsertTrack(m_signal, index))) return;
+	new(std::nothrow) Kwave::UndoInsertTrack(m_signal, index))) return;
 
     // if the signal is currently empty, use the last
     // known length instead of the current one
@@ -913,7 +913,7 @@ void Kwave::SignalManager::insertTrack(unsigned int index)
 
     if (index >= count) {
 	// do an "append"
-	m_signal.appendTrack(len);
+	m_signal.appendTrack(len, 0);
     } else {
 	if (m_undo_enabled) {
 	    // undo action for the corresponding meta data change
@@ -921,14 +921,15 @@ void Kwave::SignalManager::insertTrack(unsigned int index)
 	    for (unsigned int t = index; t < count; t++) tracks.append(t);
 	    Kwave::MetaDataList list = m_meta_data.selectByTracks(tracks);
 	    if (!list.isEmpty() && !registerUndoAction(
-		new Kwave::UndoModifyMetaDataAction(list))) return;
+		new(std::nothrow) Kwave::UndoModifyMetaDataAction(list)))
+		    return;
 	}
 
 	// adjust the track bound meta data
 	m_meta_data.insertTrack(index);
 
 	// insert into the list
-	m_signal.insertTrack(index, len);
+	m_signal.insertTrack(index, len, 0);
     }
 
     // remember the last length
@@ -946,14 +947,17 @@ void Kwave::SignalManager::deleteTrack(unsigned int index)
 
     if (m_undo_enabled) {
 	// undo action for the track deletion
-	if (!registerUndoAction(new UndoDeleteTrack(m_signal, index))) return;
+	if (!registerUndoAction(new(std::nothrow)
+	    UndoDeleteTrack(m_signal, index)))
+		return;
 
 	// undo action for the corresponding meta data change
 	QList<unsigned int> tracks;
 	for (unsigned int t = index; t < count; t++) tracks.append(t);
 	Kwave::MetaDataList list = m_meta_data.selectByTracks(tracks);
 	if (!list.isEmpty() && !registerUndoAction(
-	    new Kwave::UndoModifyMetaDataAction(list))) return;
+	    new(std::nothrow) Kwave::UndoModifyMetaDataAction(list)))
+		return;
     }
 
     // adjust the track bound meta data
@@ -978,7 +982,8 @@ void Kwave::SignalManager::slotTrackInserted(unsigned int index,
 }
 
 //***************************************************************************
-void Kwave::SignalManager::slotTrackDeleted(unsigned int index)
+void Kwave::SignalManager::slotTrackDeleted(unsigned int index,
+                                             Kwave::Track *track)
 {
     setModified(true);
 
@@ -986,7 +991,7 @@ void Kwave::SignalManager::slotTrackDeleted(unsigned int index)
     file_info.setTracks(tracks());
     m_meta_data.replace(file_info);
 
-    emit sigTrackDeleted(index);
+    emit sigTrackDeleted(index, track);
     emit sigMetaDataChanged(m_meta_data);
 }
 
@@ -1052,7 +1057,7 @@ bool Kwave::SignalManager::deleteRange(sample_index_t offset,
 
     // put the selected meta data into a undo action
     if (m_undo_enabled) {
-	if (!registerUndoAction(new UndoDeleteMetaDataAction(
+	if (!registerUndoAction(new(std::nothrow) UndoDeleteMetaDataAction(
 	    m_meta_data.copy(offset, length, track_list))))
 	{
 	    abortUndoTransaction();
@@ -1061,7 +1066,7 @@ bool Kwave::SignalManager::deleteRange(sample_index_t offset,
 	m_meta_data.deleteRange(offset, length, track_list);
 
 	// store undo data for all audio data (without meta data)
-	if (!registerUndoAction(new UndoDeleteAction(
+	if (!registerUndoAction(new(std::nothrow) UndoDeleteAction(
 	    m_parent_widget, track_list, offset, length)))
 	{
 	    abortUndoTransaction();
@@ -1104,7 +1109,7 @@ bool Kwave::SignalManager::insertSpace(sample_index_t offset,
     // first store undo data for all tracks
     unsigned int track;
     if (m_undo_enabled) {
-	if (!registerUndoAction(new Kwave::UndoInsertAction(
+	if (!registerUndoAction(new(std::nothrow) Kwave::UndoInsertAction(
 	    m_parent_widget, track_list, offset, length))) return false;
     }
 
@@ -1152,6 +1157,46 @@ void Kwave::SignalManager::selectTrack(unsigned int track, bool select)
     }
 }
 
+//***************************************************************************
+QList<Kwave::Stripe::List> Kwave::SignalManager::stripes(
+    const QList<unsigned int> &track_list,
+    sample_index_t left, sample_index_t right)
+{
+    QList<Kwave::Stripe::List> stripes;
+
+    foreach (unsigned int track, track_list) {
+	Kwave::Stripe::List s = m_signal.stripes(track, left, right);
+	if (s.isEmpty()) {
+	    stripes.clear(); // something went wrong -> abort
+	    break;
+	}
+	stripes.append(s);
+    }
+
+    return stripes;
+}
+
+//***************************************************************************
+bool Kwave::SignalManager::mergeStripes(
+    const QList<Kwave::Stripe::List> &stripes,
+    const QList<unsigned int> &track_list)
+{
+    Q_ASSERT(stripes.count() == track_list.count());
+    if (stripes.count() != track_list.count())
+	return false;
+
+    QListIterator<Kwave::Stripe::List> it_s(stripes);
+    QListIterator<unsigned int>        it_t(track_list);
+    while (it_s.hasNext() && it_t.hasNext()) {
+	Kwave::Stripe::List s = it_s.next();
+	unsigned int        t = it_t.next();
+	if (!m_signal.mergeStripes(s, t))
+	    return false; // operation failed
+    }
+
+    return true;
+}
+
 //***************************************************************************
 Kwave::PlaybackController &Kwave::SignalManager::playbackController()
 {
@@ -1177,23 +1222,28 @@ void Kwave::SignalManager::startUndoTransaction(const QString &name)
 	// if a new action starts, discard all redo actions !
 	flushRedoBuffer();
 
-	m_undo_transaction = new Kwave::UndoTransaction(name);
+	m_undo_transaction = new(std::nothrow) Kwave::UndoTransaction(name);
 	Q_ASSERT(m_undo_transaction);
 	if (!m_undo_transaction) return;
 
+	// give all registered undo handlers a chance to register their own
+	// undo actions
+	if (!m_undo_manager.startUndoTransaction(m_undo_transaction)) {
+	    delete m_undo_transaction;
+	    m_undo_transaction = 0;
+	}
+
 	// if it is the start of the transaction, also create one
 	// for the selection
-	UndoAction *selection = new Kwave::UndoSelection(*this);
+	UndoAction *selection = new(std::nothrow) Kwave::UndoSelection(*this);
 	Q_ASSERT(selection);
-	if (selection) {
-	    if (selection->store(*this)) {
-		m_undo_transaction->append(selection);
-	    } else {
-		// out of memory
-		delete selection;
-		delete m_undo_transaction;
-		m_undo_transaction = 0;
-	    }
+	if (selection && selection->store(*this)) {
+	    m_undo_transaction->append(selection);
+	} else {
+	    // out of memory
+	    delete selection;
+	    delete m_undo_transaction;
+	    m_undo_transaction = 0;
 	}
     }
 }
@@ -1374,9 +1424,9 @@ bool Kwave::SignalManager::registerUndoAction(Kwave::UndoAction *action)
     }
 
     // check if the undo action is too large
-    unsigned int limit_mb     = Kwave::MemoryManager::instance().undoLimit();
-    unsigned int needed_size  = action->undoSize();
-    unsigned int needed_mb = needed_size  >> 20;
+    qint64 limit_mb     = Kwave::MemoryManager::instance().undoLimit();
+    qint64 needed_size  = action->undoSize();
+    qint64 needed_mb    = needed_size  >> 20;
     if (needed_mb > limit_mb) {
 	delete action;
 	return continueWithoutUndo();
@@ -1416,17 +1466,17 @@ bool Kwave::SignalManager::saveUndoDelete(QList<unsigned int> &track_list,
     if (track_list.isEmpty()) return true;
 
     // create a undo action for deletion
-    UndoDeleteAction *action =
-	new UndoDeleteAction(m_parent_widget, track_list, offset, length);
+    UndoDeleteAction *action = new(std::nothrow)
+	UndoDeleteAction(m_parent_widget, track_list, offset, length);
     if (!registerUndoAction(action)) return false;
 
     return true;
 }
 
 //***************************************************************************
-unsigned int Kwave::SignalManager::usedUndoRedoMemory()
+qint64 Kwave::SignalManager::usedUndoRedoMemory()
 {
-    unsigned int size = 0;
+    qint64 size = 0;
 
     foreach (Kwave::UndoTransaction *undo, m_undo_buffer)
 	if (undo) size += undo->undoSize();
@@ -1438,17 +1488,16 @@ unsigned int Kwave::SignalManager::usedUndoRedoMemory()
 }
 
 //***************************************************************************
-void Kwave::SignalManager::freeUndoMemory(unsigned int needed)
+void Kwave::SignalManager::freeUndoMemory(qint64 needed)
 {
-    unsigned int size = usedUndoRedoMemory() + needed;
-    unsigned int undo_limit =
-	Kwave::MemoryManager::instance().undoLimit() << 20;
+    qint64 size = usedUndoRedoMemory() + needed;
+    qint64 undo_limit = Kwave::MemoryManager::instance().undoLimit() << 20;
 
     // remove old undo actions if not enough free memory
     while (!m_undo_buffer.isEmpty() && (size > undo_limit)) {
 	Kwave::UndoTransaction *undo = m_undo_buffer.takeFirst();
 	if (!undo) continue;
-	unsigned int s = undo->undoSize();
+	qint64 s = undo->undoSize();
 	size = (size >= s) ? (size - s) : 0;
 	delete undo;
 
@@ -1461,7 +1510,7 @@ void Kwave::SignalManager::freeUndoMemory(unsigned int needed)
     while (!m_redo_buffer.isEmpty() && (size > undo_limit)) {
 	Kwave::UndoTransaction *redo = m_redo_buffer.takeLast();
 	if (!redo) continue;
-	unsigned int s = redo->undoSize();
+	qint64 s = redo->undoSize();
 	size = (size >= s) ? (size - s) : 0;
 	delete redo;
     }
@@ -1519,10 +1568,9 @@ void Kwave::SignalManager::undo()
     m_undo_enabled = false;
 
     // get free memory for redo
-    unsigned int undo_limit =
-	Kwave::MemoryManager::instance().undoLimit() << 20;
-    unsigned int redo_size = undo_transaction->redoSize();
-    unsigned int undo_size = undo_transaction->undoSize();
+    qint64 undo_limit = Kwave::MemoryManager::instance().undoLimit() << 20;
+    qint64 redo_size = undo_transaction->redoSize();
+    qint64 undo_size = undo_transaction->undoSize();
     Kwave::UndoTransaction *redo_transaction = 0;
     if ((redo_size > undo_size) && (redo_size - undo_size > undo_limit)) {
 	// not enough memory for redo
@@ -1533,7 +1581,7 @@ void Kwave::SignalManager::undo()
 
 	// create a new redo transaction
 	QString name = undo_transaction->description();
-	redo_transaction = new Kwave::UndoTransaction(name);
+	redo_transaction = new(std::nothrow) Kwave::UndoTransaction(name);
 	Q_ASSERT(redo_transaction);
     }
 
@@ -1595,7 +1643,8 @@ void Kwave::SignalManager::undo()
 	QList<unsigned int> tracks = selectedTracks();
 	bool tracks_modified = !(tracks == m_last_track_selection);
 	if (range_modified || tracks_modified) {
-	    UndoAction *redo_action = new Kwave::UndoSelection(*this,
+	    UndoAction *redo_action = new(std::nothrow) Kwave::UndoSelection(
+		*this,
 		m_last_track_selection,
 		m_last_selection.offset(),
 		m_last_selection.length());
@@ -1660,10 +1709,9 @@ void Kwave::SignalManager::redo()
     m_undo_enabled = false;
 
     // get free memory for undo
-    unsigned int undo_limit =
-	Kwave::MemoryManager::instance().undoLimit() << 20;
-    unsigned int undo_size = redo_transaction->undoSize();
-    unsigned int redo_size = redo_transaction->redoSize();
+    qint64 undo_limit = Kwave::MemoryManager::instance().undoLimit() << 20;
+    qint64 undo_size = redo_transaction->undoSize();
+    qint64 redo_size = redo_transaction->redoSize();
     Kwave::UndoTransaction *undo_transaction = 0;
     if ((undo_size > redo_size) && (undo_size - redo_size > undo_limit)) {
 	// not enough memory for undo
@@ -1674,7 +1722,7 @@ void Kwave::SignalManager::redo()
 
 	// create a new undo transaction
 	QString name = redo_transaction->description();
-	undo_transaction = new Kwave::UndoTransaction(name);
+	undo_transaction = new(std::nothrow) Kwave::UndoTransaction(name);
 	Q_ASSERT(undo_transaction);
     }
 
@@ -1780,7 +1828,8 @@ void Kwave::SignalManager::setFileInfo(Kwave::FileInfo &new_info,
 	Kwave::UndoTransactionGuard undo_transaction(*this,
 	                                             i18n("Modify File Info"));
 	Kwave::FileInfo old_inf(m_meta_data);
-	if (!registerUndoAction(new Kwave::UndoModifyMetaDataAction(old_inf)))
+	if (!registerUndoAction(
+	    new(std::nothrow) Kwave::UndoModifyMetaDataAction(old_inf)))
 	    return;
     }
 
@@ -1825,7 +1874,7 @@ Kwave::Label Kwave::SignalManager::addLabel(sample_index_t pos,
     // register the undo action
     if (m_undo_enabled) {
 	Kwave::UndoTransactionGuard undo(*this, i18n("Add Label"));
-	if (!registerUndoAction(new UndoAddMetaDataAction(label)))
+	if (!registerUndoAction(new(std::nothrow) UndoAddMetaDataAction(label)))
 	    return Kwave::Label();
     }
 
@@ -1846,15 +1895,16 @@ void Kwave::SignalManager::deleteLabel(int index, bool with_undo)
     Kwave::LabelList labels(m_meta_data);
 
     Q_ASSERT(index >= 0);
-    Q_ASSERT(index < static_cast<int>(labels.count()));
-    if ((index < 0) || (index >= static_cast<int>(labels.count()))) return;
+    Q_ASSERT(index < Kwave::toInt(labels.count()));
+    if ((index < 0) || (index >= Kwave::toInt(labels.count()))) return;
 
     Kwave::MetaData label(labels.at(index));
 
     // register the undo action
     if (with_undo) {
 	Kwave::UndoTransactionGuard undo(*this, i18n("Delete Label"));
-	if (!registerUndoAction(new UndoDeleteMetaDataAction(label)))
+	if (!registerUndoAction(new(std::nothrow)
+	    UndoDeleteMetaDataAction(label)))
 	    return;
     }
 
@@ -1872,8 +1922,8 @@ bool Kwave::SignalManager::modifyLabel(int index, sample_index_t pos,
 {
     Kwave::LabelList labels(m_meta_data);
     Q_ASSERT(index >= 0);
-    Q_ASSERT(index < static_cast<int>(labels.count()));
-    if ((index < 0) || (index >= static_cast<int>(labels.count())))
+    Q_ASSERT(index < Kwave::toInt(labels.count()));
+    if ((index < 0) || (index >= Kwave::toInt(labels.count())))
 	return false;
 
     Kwave::Label label = labels.at(index);
@@ -1886,7 +1936,7 @@ bool Kwave::SignalManager::modifyLabel(int index, sample_index_t pos,
     // add a undo action
     if (m_undo_enabled) {
 	Kwave::UndoModifyMetaDataAction *undo_modify =
-	    new Kwave::UndoModifyMetaDataAction(label);
+	    new(std::nothrow) Kwave::UndoModifyMetaDataAction(label);
 	if (!registerUndoAction(undo_modify))
 	    return false;
     }
