@@ -38,6 +38,7 @@
 #include "libkwave/ReaderMode.h"
 #include "libkwave/Selection.h"
 #include "libkwave/Signal.h"
+#include "libkwave/undo/UndoManager.h"
 
 class QBitmap;
 class QFile;
@@ -247,18 +248,22 @@ namespace Kwave
 	void selectTrack(unsigned int track, bool select);
 
 	/**
-	 * Opens an input stream for a track, starting at a specified sample
+	 * Opens an output stream for a track, starting at a specified sample
 	 * position.
+	 * @param mode specifies where and how to insert
 	 * @param track index of the track. If the track does not exist, this
 	 *        function will fail and return 0
-	 * @param mode specifies where and how to insert
 	 * @param left start of the input (only useful in insert and
 	 *             overwrite mode)
 	 * @param right end of the input (only useful with overwrite mode)
 	 * @see InsertMode
 	 */
-	Kwave::Writer *openWriter(unsigned int track, Kwave::InsertMode mode,
-	    sample_index_t left = 0, sample_index_t right = 0);
+	inline Kwave::Writer *openWriter(Kwave::InsertMode mode,
+	    unsigned int track,
+	    sample_index_t left = 0, sample_index_t right = 0)
+	{
+	    return m_signal.openWriter(mode, track, left, right);
+	}
 
 	/**
 	 * Opens a stream for reading samples. If the the last position
@@ -269,13 +274,40 @@ namespace Kwave
 	 * @param left first offset to be read (default = 0)
 	 * @param right last position to read (default = UINT_MAX)
 	 */
-	inline Kwave::SampleReader *openSampleReader(Kwave::ReaderMode mode,
+	inline Kwave::SampleReader *openReader(Kwave::ReaderMode mode,
 	    unsigned int track,
 	    sample_index_t left = 0, sample_index_t right = SAMPLE_INDEX_MAX)
 	{
-	    return m_signal.openSampleReader(mode, track, left, right);
+	    return m_signal.openReader(mode, track, left, right);
 	}
 
+
+	/**
+	 * Get a list of stripes that matches a given range of samples
+	 * @param track_list list with indices of tracks for selecting
+	 * @param left  offset of the first sample
+	 * @param right offset of the last sample
+	 * @return a list with lists of stripes that cover the given range
+	 *         between left and right
+	 */
+	QList<Kwave::Stripe::List> stripes(
+	    const QList<unsigned int> &track_list,
+	    sample_index_t left = 0,
+	    sample_index_t right = SAMPLE_INDEX_MAX);
+
+	/**
+	 * Merge a list of stripes into the signal.
+	 * @note this operation works without undo!
+	 * @param stripes list of stripe list (multi track)
+	 * @param track_list list with indices of tracks for selecting
+	 * @return true if succeeded, false if failed
+	 */
+	bool mergeStripes(const QList<Kwave::Stripe::List> &stripes,
+	                  const QList<unsigned int> &track_list);
+
+	/** Returns a reference to the undo manager */
+	inline Kwave::UndoManager &undoManager() { return m_undo_manager; }
+
 	/** Returns true if undo/redo is currently enabled */
 	inline bool undoEnabled() const { return m_undo_enabled; }
 
@@ -354,7 +386,7 @@ namespace Kwave
 	 * Retrieves the list of meta data objects, mutable
 	 * @return list with all MetaData objects
 	 */
-	Kwave::MetaDataList &metaData() { return m_meta_data; };
+	Kwave::MetaDataList &metaData() { return m_meta_data; }
 
 	/**
 	 * Retrieves the list of meta data objects, const
@@ -362,6 +394,16 @@ namespace Kwave
 	 */
 	const Kwave::MetaDataList &metaData() const { return m_meta_data; }
 
+	/**
+	 * Returns the uuid of a track
+	 * @param track index of the track [0...tracks-1]
+	 * @return the QUuid of the track or a "null" uuid if the track
+	 *         does not exist
+	 */
+	QUuid uuidOfTrack(unsigned int track) {
+	    return m_signal.uuidOfTrack(track);
+	}
+
     signals:
 
 	/**
@@ -381,8 +423,9 @@ namespace Kwave
 	/**
 	 * Signals that a track has been deleted.
 	 * @param index position of the deleted track [0...tracks()-1]
+	 * @param track reference to the new track
 	 */
-	void sigTrackDeleted(unsigned int index);
+	void sigTrackDeleted(unsigned int index, Kwave::Track *track);
 
 	/**
 	 * Signals that the selection of one of the tracks has changed
@@ -466,10 +509,11 @@ namespace Kwave
 	/**
 	 * Connected to the signal's sigTrackInserted.
 	 * @param index numeric index of the inserted track
+	 * @param track reference to the track that has been deleted
 	 * @see Signal::sigTrackDeleted
 	 * @internal
 	 */
-	void slotTrackDeleted(unsigned int index);
+	void slotTrackDeleted(unsigned int index, Kwave::Track *track);
 
 	/**
 	 * Connected to the signal's sigSamplesInserted.
@@ -617,7 +661,7 @@ namespace Kwave
 	/**
 	 * Returns the amount of memory currently used for undo + redo.
 	 */
-	unsigned int usedUndoRedoMemory();
+	qint64 usedUndoRedoMemory();
 
 	/**
 	 * Makes sure that enough memory for a following undo (or redo) action
@@ -625,7 +669,7 @@ namespace Kwave
 	 * still no enough, it also removes old redo transactions.
 	 * @param needed the amount of memory that should be free afterwards
 	 */
-	void freeUndoMemory(unsigned int needed);
+	void freeUndoMemory(qint64 needed);
 
 	/**
 	 * Enables changes of the modified flag.
@@ -705,6 +749,9 @@ namespace Kwave
 	/** mutex for locking undo transactions */
 	QMutex m_undo_transaction_lock;
 
+	/** Manager for undo/redo actions */
+	Kwave::UndoManager m_undo_manager;
+
 	/**
 	 * meta data of the signal
 	 * @see class MetaData
