@@ -17,22 +17,16 @@
 
 #include "config.h"
 
+#include <new>
 #include <string.h> // for some speed-ups like memmove, memcpy ...
 
 #include "libkwave/memcpy.h"
 #include "libkwave/Stripe.h"
+#include "libkwave/Utils.h"
 
 // define this for using only slow Qt array functions
 #undef STRICTLY_QT
 
-#ifndef unlikely
-#define unlikely(x) (__builtin_expect((x),0))
-#endif
-
-#ifndef likely
-#define likely(x) (__builtin_expect((x),1))
-#endif
-
 //***************************************************************************
 //***************************************************************************
 Kwave::Stripe::MappedArray::MappedArray(Stripe &stripe)
@@ -153,7 +147,30 @@ Kwave::Stripe::StripeStorage::StripeStorage(const StripeStorage &other)
 	    mem.writeTo(m_storage, 0, original, m_length * sizeof(sample_t));
 	    mem.unmap(other.m_storage);
 	} else {
-	    qWarning("StripeStorage: DEEP COPY (%u) MMAP FAILED!", m_length);
+	    qWarning("StripeStorage: DEEP COPY (%u) MMAP FAILED - "
+	             "FALLING BACK TO SLOW COPY!", m_length);
+	    // fall back to slow copy algorithm
+	    char buf[256]; // use a small buffer on the stack
+	    unsigned int remaining = m_length;
+	    unsigned int ofs       = 0;
+
+	    while (remaining) {
+		unsigned int len =  qMin(remaining, Kwave::toUint(sizeof(buf)));
+		unsigned int read = mem.readFrom(other.m_storage, ofs,
+		                                 &buf[0], len);
+		Q_ASSERT(read == len);
+		if (read != len) break;
+		unsigned int written = mem.writeTo(m_storage, ofs,
+		                                   &buf[0], len);
+		Q_ASSERT(written == len);
+		if (written != len) break;
+		remaining -= len;
+		ofs       += len;
+	    }
+	    if (remaining) {
+		qWarning("StripeStorage: DEEP COPY (%u) FAILED - DATA LOST!",
+		         m_length);
+	    }
 	}
     }
 }
@@ -206,7 +223,7 @@ Kwave::Stripe::StripeStorage::~StripeStorage()
 //***************************************************************************
 //***************************************************************************
 Kwave::Stripe::Stripe()
-    :m_lock(), m_data(new StripeStorage)
+    :m_lock(), m_data(new(std::nothrow) StripeStorage)
 {
 }
 
@@ -219,14 +236,14 @@ Kwave::Stripe::Stripe(const Stripe &other)
 
 //***************************************************************************
 Kwave::Stripe::Stripe(sample_index_t start)
-    :m_lock(), m_data(new StripeStorage)
+    :m_lock(), m_data(new(std::nothrow) StripeStorage)
 {
     if (m_data) m_data->m_start = start;
 }
 
 //***************************************************************************
 Kwave::Stripe::Stripe(sample_index_t start, const Kwave::SampleArray &samples)
-    :m_lock(), m_data(new StripeStorage)
+    :m_lock(), m_data(new(std::nothrow) StripeStorage)
 {
     if (m_data) m_data->m_start = start;
     if (samples.size()) append(samples, 0, samples.size());
@@ -236,7 +253,7 @@ Kwave::Stripe::Stripe(sample_index_t start, const Kwave::SampleArray &samples)
 Kwave::Stripe::Stripe(sample_index_t start,
                       Kwave::Stripe &stripe,
                       unsigned int offset)
-    :m_lock(), m_data(new StripeStorage)
+    :m_lock(), m_data(new(std::nothrow) StripeStorage)
 {
     if (!m_data) return;
 
@@ -469,6 +486,40 @@ void Kwave::Stripe::deleteRange(unsigned int offset, unsigned int length)
     }
 }
 
+//***************************************************************************
+bool Kwave::Stripe::combine(unsigned int offset, Kwave::Stripe &other)
+{
+    // detach the data and check for map count zero
+    if (!m_data) return false;
+    m_data.detach();
+    if (!m_data) return false; // OOM when detaching
+
+    {
+	QMutexLocker lock(&m_lock);
+	if (m_data->mapCount()) return false; // data is mapped
+
+	const unsigned int old_len      = m_data->m_length;
+	const unsigned int combined_len = offset + other.length();
+	if (old_len < combined_len) {
+	    // resize the storage if necessary
+	    if (resizeStorage(combined_len) != combined_len)
+		return false; // resizing failed, maybe OOM ?
+	}
+
+	// copy the data from the other stripe
+	MappedArray _src(other);
+	MappedArray _dst(*this);
+	const sample_t *src = _src.constData();
+	sample_t       *dst = _dst.data();
+	unsigned int    len = _src.size() * sizeof(sample_t);
+	if (!src || !dst) return false; // mmap of src or dst failed
+
+	MEMCPY(dst + offset, src, len);
+    }
+
+    return true;
+}
+
 //***************************************************************************
 void Kwave::Stripe::overwrite(unsigned int offset,
 	const Kwave::SampleArray &source,
@@ -527,7 +578,7 @@ void Kwave::Stripe::minMax(unsigned int first, unsigned int last,
 
     if (!m_data) return;
     MappedArray _samples(*this);
-    const sample_t *buffer = _samples.data();
+    const sample_t *buffer = _samples.constData();
     if (!buffer || !m_data) return;
 
     // loop over the mapped storage to get min/max
@@ -541,18 +592,18 @@ void Kwave::Stripe::minMax(unsigned int first, unsigned int last,
 
     // speedup: process a block of 8 samples at once, to allow loop unrolling
     const unsigned int block = 8;
-    while (likely(remaining >= block)) {
+    while (Q_LIKELY(remaining >= block)) {
 	for (unsigned int count = 0; count < block; count++) {
 	    register sample_t s = *(buffer++);
-	    if (unlikely(s < lo)) lo = s;
-	    if (unlikely(s > hi)) hi = s;
+	    if (Q_UNLIKELY(s < lo)) lo = s;
+	    if (Q_UNLIKELY(s > hi)) hi = s;
 	}
 	remaining -= block;
     }
-    while (likely(remaining--)) {
+    while (Q_LIKELY(remaining--)) {
 	register sample_t s = *(buffer++);
-	if (unlikely(s < lo)) lo = s;
-	if (unlikely(s > hi)) hi = s;
+	if (Q_UNLIKELY(s < lo)) lo = s;
+	if (Q_UNLIKELY(s > hi)) hi = s;
     }
     min = lo;
     max = hi;
