@@ -17,6 +17,8 @@
 
 #include "config.h"
 
+#include <new>
+
 #include <QtCore/QReadLocker>
 #include <QtCore/QWriteLocker>
 
@@ -24,6 +26,7 @@
 #include "libkwave/Stripe.h"
 #include "libkwave/Track.h"
 #include "libkwave/TrackWriter.h"
+#include "libkwave/Utils.h"
 #include "libkwave/Writer.h"
 
 /**
@@ -31,7 +34,6 @@
  * When creating a new stripe, it should have this size
  */
 #define STRIPE_LENGTH_OPTIMAL (4UL * 1024UL * 1024UL) /* 16MB */
-// #define STRIPE_LENGTH_OPTIMAL (64 * 1024UL) /* 64kB */
 
 /**
  * Maximum stripe size [samples].
@@ -53,15 +55,17 @@
 
 //***************************************************************************
 Kwave::Track::Track()
-    :m_lock(), m_lock_usage(), m_stripes(), m_selected(true)
+    :m_lock(), m_lock_usage(), m_stripes(), m_selected(true),
+     m_uuid(QUuid::createUuid())
 {
 }
 
 //***************************************************************************
-Kwave::Track::Track(sample_index_t length)
-    :m_lock(), m_lock_usage(), m_stripes(), m_selected(true)
+Kwave::Track::Track(sample_index_t length, QUuid *uuid)
+    :m_lock(), m_lock_usage(), m_stripes(), m_selected(true),
+     m_uuid((uuid) ? *uuid : QUuid::createUuid())
 {
-    if (length < 2*STRIPE_LENGTH_OPTIMAL) {
+    if (length < STRIPE_LENGTH_MAXIMUM) {
 	if (length) appendStripe(length);
     } else {
 	Stripe s(length - STRIPE_LENGTH_OPTIMAL);
@@ -86,17 +90,13 @@ Kwave::Track::~Track()
 //***************************************************************************
 void Kwave::Track::appendStripe(sample_index_t length)
 {
-//     SharedLockGuard lock(m_lock, true);
     sample_index_t start = unlockedLength();
-    sample_index_t len;
-
     do {
-	len = length;
-	if (len > STRIPE_LENGTH_MAXIMUM)
-	    len = STRIPE_LENGTH_MAXIMUM;
+	unsigned int len = Kwave::toUint(
+	    qMin<sample_index_t>(STRIPE_LENGTH_MAXIMUM, length));
 
 	Stripe s(start);
-	s.resize(length);
+	s.resize(len);
 	if (len) emit sigSamplesInserted(this, start, len);
 
 	length -= len;
@@ -128,6 +128,46 @@ Kwave::Stripe Kwave::Track::splitStripe(Kwave::Stripe &stripe,
     return s;
 }
 
+//***************************************************************************
+bool Kwave::Track::mergeStripe(Kwave::Stripe &stripe)
+{
+    sample_index_t left  = stripe.start();
+    sample_index_t right = stripe.end();
+    int index_before = -1;
+
+//     qDebug("Track::mergeStripe() [%llu - %llu]", left, right);
+//     dump();
+
+    // remove all stripes that are overlapped completely by
+    // this stripe and crop stripes that overlap partially
+    unlockedDelete(left, right - left + 1, true);
+
+//     qDebug("Track::mergeStripe() [%llu - %llu] - after delete", left, right);
+//     dump();
+
+    // find the stripe after which we have to insert
+    QListIterator<Stripe> it(m_stripes);
+    while (it.hasNext()) {
+	const Stripe &s = it.next();
+	if (left > s.end()) index_before = m_stripes.indexOf(s);
+    }
+
+    if (index_before >= 0) {
+	// insert after some existing stripe
+// 	qDebug("Kwave::Track::mergeStripe: insert after index %d",
+// 	       index_before);
+	m_stripes.insert(index_before + 1, stripe);
+    } else {
+	// the one and only or insert before all others
+// 	qDebug("Kwave::Track::mergeStripe: prepending");
+	m_stripes.prepend(stripe);
+    }
+
+//     qDebug("Track::mergeStripe() - done");
+//     dump();
+    return true;
+}
+
 //***************************************************************************
 sample_index_t Kwave::Track::length()
 {
@@ -149,23 +189,29 @@ Kwave::Writer *Kwave::Track::openWriter(Kwave::InsertMode mode,
                                         sample_index_t right)
 {
     // create the input stream
-    Kwave::Writer *stream = new Kwave::TrackWriter(*this, mode, left, right);
+    Kwave::Writer *stream =
+	new(std::nothrow) Kwave::TrackWriter(*this, mode, left, right);
     Q_ASSERT(stream);
 
+    if (stream) {
+	// at the point of time when the writer gets closed we can take
+	// care of fragmentation
+	connect(stream, SIGNAL(destroyed(QObject*)),
+	        this,   SLOT(defragment()),
+	        Qt::DirectConnection);
+    }
+
     return stream;
 }
 
 //***************************************************************************
-Kwave::SampleReader *Kwave::Track::openSampleReader(Kwave::ReaderMode mode,
-	sample_index_t left, sample_index_t right)
+Kwave::Stripe::List Kwave::Track::stripes(sample_index_t left,
+                                          sample_index_t right)
 {
     QReadLocker lock(&m_lock);
 
-    sample_index_t length = unlockedLength();
-    if (right >= length) right = (length) ? (length - 1) : 0;
-
     // collect all stripes that are in the requested range
-    QList<Stripe> stripes;
+    Kwave::Stripe::List stripes(left, right);
     foreach (const Stripe &stripe, m_stripes) {
 	if (!stripe.length()) continue;
 	sample_index_t start = stripe.start();
@@ -174,12 +220,70 @@ Kwave::SampleReader *Kwave::Track::openSampleReader(Kwave::ReaderMode mode,
 	if (end < left) continue; // not yet in range
 	if (start > right) break; // done
 
-	stripes.append(stripe);
+	if ((end <= right) && (start >= left)) {
+	    // append the stripe as it is, unmodified
+	    stripes.append(stripe);
+	    continue;
+	}
+
+	// there is only some overlap, no 100% match
+	// -> make a cropped copy
+	Stripe cropped(stripe);
+
+	// remove data after the end of the selection
+	if (end > right)
+	    cropped.resize(Kwave::toUint(cropped.length() - (end - right)));
+
+	// remove data before the start of the selection
+	if (start < left) {
+	    cropped.deleteRange(0, Kwave::toUint(left - start));
+	    cropped.setStart(left);
+	}
+
+	stripes.append(cropped);
     }
 
+    return stripes;
+}
+
+//***************************************************************************
+bool Kwave::Track::mergeStripes(const Kwave::Stripe::List &stripes)
+{
+    bool succeeded = true;
+    {
+	QWriteLocker lock(&m_lock);
+	foreach (Stripe stripe, stripes) {
+	    if (!mergeStripe(stripe)) {
+		succeeded = false;
+		break;
+	    }
+	}
+    }
+
+    // do some defragmentation, to combin the ends of the inserted stripes
+    defragment();
+
+    const sample_index_t left  = stripes.left();
+    const sample_index_t right = stripes.right();
+    emit sigSamplesModified(this, left, right - left + 1);
+    return succeeded;
+}
+
+//***************************************************************************
+Kwave::SampleReader *Kwave::Track::openReader(Kwave::ReaderMode mode,
+	sample_index_t left, sample_index_t right)
+{
+    QReadLocker lock(&m_lock);
+
+    const sample_index_t length = unlockedLength();
+    if (right >= length) right = (length) ? (length - 1) : 0;
+
+    // collect all stripes that are in the requested range
+    Kwave::Stripe::List stripe_list = stripes(left, right);
+
     // create the input stream
     Kwave::SampleReader *stream =
-	new Kwave::SampleReader(mode, stripes, left, right);
+	new(std::nothrow) Kwave::SampleReader(mode, stripe_list);
     Q_ASSERT(stream);
     return stream;
 }
@@ -198,6 +302,10 @@ void Kwave::Track::deleteRange(sample_index_t offset, sample_index_t length,
 	unlockedDelete(offset, length, make_gap);
     }
 
+    // deletion without gap might have left some fragments
+    if (!make_gap)
+	defragment();
+
     emit sigSamplesDeleted(this, offset, length);
 }
 
@@ -223,7 +331,9 @@ bool Kwave::Track::insertSpace(sample_index_t offset, sample_index_t shift)
 		if (start >= offset) break; // not "within" the stripe
 
 // 		qDebug("Kwave::Track::insertSpace => splitting [%u...%u]",start,end);
-		Stripe new_stripe = splitStripe(s, offset - start);
+		Stripe new_stripe = splitStripe(s,
+		    Kwave::toUint(offset - start)
+		);
 		if (!new_stripe.length()) return false; // OOM ?
 		it.insert(new_stripe);
 		break;
@@ -275,7 +385,7 @@ void Kwave::Track::unlockedDelete(sample_index_t offset, sample_index_t length,
 	    //        ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 	    //        already checked above
 	    // partial stripe overlap
-	    unsigned int ofs = (start < left) ? left : start;
+	    sample_index_t ofs = (start < left) ? left : start;
 	    if (end > right) end = right;
 // 	    qDebug("deleting [%u ... %u] (start=%u, ofs-start=%u, len=%u)",
 // 		ofs, end, start, ofs-start, end - ofs + 1);
@@ -288,7 +398,10 @@ void Kwave::Track::unlockedDelete(sample_index_t offset, sample_index_t length,
 		// case #3: delete from the left only
 		// case #4: delete from the right only
 // 		qDebug("    deleting within the stripe");
-		s.deleteRange(ofs - start, end - ofs + 1);
+		s.deleteRange(
+		    Kwave::toUint(ofs - start),
+		    Kwave::toUint(end - ofs + 1)
+		);
 		if (!s.length()) break; // OOM ?
 
 		// if deleted from start
@@ -303,7 +416,8 @@ void Kwave::Track::unlockedDelete(sample_index_t offset, sample_index_t length,
 		//          by splitting off a new stripe
 // 		qDebug("    splitting off to new stripe @ %u (ofs=%u)",
 // 		    right + 1, right + 1 - start);
-		Stripe new_stripe = splitStripe(s, right + 1 - start);
+		Stripe new_stripe = splitStripe(s,
+		    Kwave::toUint(right + 1 - start));
 		if (!new_stripe.length()) break; // OOM ?
 		it.next(); // right after "s"
 		it.insert(new_stripe);
@@ -311,10 +425,10 @@ void Kwave::Track::unlockedDelete(sample_index_t offset, sample_index_t length,
 		it.previous(); // before s, like
 
 		// erase to the end (reduce size)
+		const unsigned int todel = Kwave::toUint(s.end() - ofs + 1);
 // 		qDebug("ofs-start=%u, s->end()-ofs+1=%u [%u...%u] (%u)",
-// 		    ofs-start, s.end() - ofs + 1, s.start(),
-// 		    s.end(), s.length());
-		s.deleteRange(ofs - start, s.end() - ofs + 1);
+// 		    ofs-start, todel, s.start(), s.end(), s.length());
+		s.deleteRange(Kwave::toUint(ofs - start), todel);
 // 		qDebug("length now: %u [%u ... %u]", s.length(),
 // 		    s.start(), s.end());
 // 		Q_ASSERT(s.length());
@@ -390,9 +504,8 @@ bool Kwave::Track::appendAfter(Stripe *stripe,  sample_index_t offset,
 
     // append new stripes as long as there is something remaining
     while (length) {
-	sample_index_t len = length;
-	if (len > STRIPE_LENGTH_MAXIMUM)
-	    len = STRIPE_LENGTH_MAXIMUM;
+	unsigned int len = Kwave::toUint(qMin<sample_index_t>(
+	    length, STRIPE_LENGTH_MAXIMUM));
 
 // 	qDebug("Kwave::Track::appendAfter: new stripe, ofs=%u, len=%u",
 // 	       offset, len);
@@ -529,7 +642,8 @@ bool Kwave::Track::writeSamples(Kwave::InsertMode mode,
 	        // split the target stripe and insert the samples
 		// between the two new ones
 		Stripe new_stripe = splitStripe(*target_stripe,
-		    offset - target_stripe->start());
+		    Kwave::toUint(offset - target_stripe->start())
+		);
 		if (!new_stripe.length()) break;
 		m_stripes.insert(m_stripes.indexOf(*target_stripe) + 1,
 		    new_stripe);
@@ -579,6 +693,73 @@ bool Kwave::Track::writeSamples(Kwave::InsertMode mode,
     return true;
 }
 
+//***************************************************************************
+void Kwave::Track::defragment()
+{
+
+    if (!m_lock.tryLockForWrite()) {
+	// this could happen when there are two or more writers
+// 	qDebug("Track::defragment() - busy");
+	return;
+    }
+
+    if (m_stripes.count() > 1)
+    {
+// 	qDebug("Track::defragment(), state before:");
+// 	dump();
+
+	unsigned int   index  = 0;
+	Kwave::Stripe *before = 0;
+	Kwave::Stripe *stripe = 0;
+
+	// use a quick and simple algorithm:
+	// iterate over all stripes and analyze pairwise
+	QMutableListIterator<Kwave::Stripe> it(m_stripes);
+	while (it.hasNext()) {
+	    before = stripe;
+	    stripe = &(it.next());
+	    if (!before) continue; // skip the first entry
+	    index++;
+
+// 	    qDebug("Track::defragment(), checking #%u [%llu..%llu] (%u)",
+// 		    index, stripe->start(), stripe->end(), stripe->length());
+
+	    const sample_index_t before_start = before->start();
+	    const sample_index_t stripe_end   = stripe->end();
+	    const sample_index_t combined_len = stripe_end - before_start + 1;
+
+	    if (combined_len > STRIPE_LENGTH_MAXIMUM)
+		continue; // would be too large
+
+	    if ((before->length() < STRIPE_LENGTH_MINIMUM) ||
+		(stripe->length() < STRIPE_LENGTH_MINIMUM)) {
+// 		qDebug("Track::defragment(), combine #%u [%llu..%llu] & "
+// 		       "#%u [%llu..%llu] => [%llu..%llu] (%llu)",
+// 		       index - 1, before->start(), before->end(),
+// 		       index, stripe->start(), stripe->end(),
+// 		       before->start(), stripe->end(), combined_len);
+
+		// try to resize the stripe before to contain the
+		// combined length
+		const unsigned int offset = Kwave::toUint(
+		    stripe->start() - before_start);
+		if (!before->combine(offset, *stripe))
+		    continue; // not possible, maybe OOM ?
+
+		// remove the current stripe, to avoid an overlap
+		it.remove();
+		stripe = before;
+		index--;
+	    }
+	}
+
+// 	qDebug("Track::defragment(), state after:");
+// 	dump();
+    }
+
+    m_lock.unlock();
+}
+
 //***************************************************************************
 void Kwave::Track::use()
 {
@@ -595,10 +776,10 @@ void Kwave::Track::release()
 void Kwave::Track::dump()
 {
     qDebug("------------------------------------");
-    unsigned int index = 0;
-    unsigned int last_end = 0;
+    unsigned int   index    = 0;
+    sample_index_t last_end = 0;
     foreach (const Stripe &s, m_stripes) {
-	unsigned int start = s.start();
+	sample_index_t start = s.start();
 	if (index && (start <= last_end))
 	    qDebug("--- OVERLAP ---");
 	if (start > last_end+1)
