@@ -24,6 +24,7 @@
 #include <QtCore/QList>
 #include <QtCore/QObject>
 #include <QtCore/QReadWriteLock>
+#include <QtCore/QUuid>
 
 #include <kdemacros.h>
 
@@ -46,15 +47,16 @@ namespace Kwave
     public:
 	/**
 	 * Default constructor. Creates a new and empty track with
-	 * zero-length and no stripes
+	 * zero-length, no stripes and new uuid
 	 */
 	Track();
 
 	/**
-	 * Constructor. Creates an empty track with one stripe
-	 * with specified length.
+	 * Constructor. Creates an empty track with a specified length.
+	 * @param length the length in samples
+	 * @param uuid unique ID of the track, can be null
 	 */
-	Track(sample_index_t length);
+	Track(sample_index_t length, QUuid *uuid);
 
 	/**
 	 * Destructor.
@@ -75,6 +77,7 @@ namespace Kwave
 	 *             overwrite mode)
 	 * @param right end of the input (only useful with overwrite mode)
 	 * @see InsertMode
+	 * @note destruction of the writer triggers defragmentation
 	 */
 	Kwave::Writer *openWriter(Kwave::InsertMode mode,
 	                          sample_index_t left = 0,
@@ -87,16 +90,35 @@ namespace Kwave
 	 * @param left first offset to be read (default = 0)
 	 * @param right last position to read (default = UINT_MAX)
 	 */
-	Kwave::SampleReader *openSampleReader(Kwave::ReaderMode mode,
+	Kwave::SampleReader *openReader(Kwave::ReaderMode mode,
 	    sample_index_t left = 0,
             sample_index_t right = SAMPLE_INDEX_MAX);
 
+	/**
+	 * Get a list of stripes that matches a given range of samples
+	 * @param left  offset of the first sample
+	 * @param right offset of the last sample
+	 * @return a list of stripes that cover the given range
+	 *         between left and right
+	 */
+	Kwave::Stripe::List stripes(sample_index_t left,
+	                            sample_index_t right);
+
+	/**
+	 * Merge a list of stripes into the track.
+	 * @param stripes list of stripes
+	 * @return true if succeeded, false if failed
+	 * @note triggers a defragmentation
+	 */
+	bool mergeStripes(const Kwave::Stripe::List &stripes);
+
 	/**
 	 * Deletes a range of samples
 	 * @param offset index of the first sample
 	 * @param length number of samples
 	 * @param make_gap if true, make a gap into the list of stripes
 	 *                 instead of moving the stuff from right to left
+	 * @note triggers a defragmentation if make_gap is false
 	 */
 	void deleteRange(sample_index_t offset, sample_index_t length,
 	                 bool make_gap = false);
@@ -117,11 +139,17 @@ namespace Kwave
 	/** Sets the "selected" flag. */
 	void select(bool select);
 
+	/** returns the unique ID of this track instance */
+	const QUuid &uuid() const { return m_uuid; }
+
     public slots:
 
 	/** toggles the selection of the slot on/off */
 	void toggleSelection();
 
+	/** do some defragmentation of stripes */
+	void defragment();
+
     signals:
 
 	/**
@@ -223,6 +251,15 @@ namespace Kwave
 	 */
 	Stripe splitStripe(Stripe &stripe, unsigned int offset);
 
+
+	/**
+	 * Merge a single stripe into the track.
+	 *
+	 * @param stripe the stripe to merge
+	 * @return true if succeeded, false if failed
+	 */
+	bool mergeStripe(Kwave::Stripe &stripe);
+
 	/**
 	 * dump the list of stripes, for debugging
 	 * @internal for debugging only
@@ -279,6 +316,8 @@ namespace Kwave
 	/** True if the track is selected */
 	bool m_selected;
 
+	/** unique ID */
+	QUuid m_uuid;
     };
 }
 
