@@ -23,9 +23,6 @@
 #include <QtCore/QObject>
 #include <QtCore/QSemaphore>
 
-#include <threadweaver/Job.h>
-#include <threadweaver/ThreadWeaver.h>
-
 #include "libkwave/SampleArray.h"
 #include "libkwave/SampleSink.h"
 
@@ -54,7 +51,7 @@ namespace Kwave
 	virtual Kwave::SampleArray &data();
 
 	/** returns a const reference to the sample data */
-	virtual const Kwave::SampleArray &data() const;
+	virtual const Kwave::SampleArray &constData() const;
 
 	/** returns the number of samples that can be fetched with get() */
 	virtual unsigned int available() const;
@@ -103,38 +100,6 @@ namespace Kwave
 	/** emits the data received via input() */
 	void output(Kwave::SampleArray data);
 
-    private:
-
-	class BufferJob: public ThreadWeaver::Job
-	{
-	public:
-	    /** Constructor */
-	    BufferJob(Kwave::SampleBuffer *buffer);
-
-	    /** Destructor */
-	    virtual ~BufferJob();
-
-	    /** enqueue some data */
-	    void enqueue(Kwave::SampleArray data);
-
-	    /**
-	    * overloaded 'run' function that runns goOn() in the context
-	    * of the worker thread.
-	    */
-	    virtual void run();
-
-	private:
-
-	    /** reference to the Kwave::SampleBuffer */
-	    Kwave::SampleBuffer *m_buffer;
-
-	    /** sample data to emit */
-	    Kwave::SampleArray m_data;
-
-	    /** semaphore for limiting queue depth to 1 */
-	    QSemaphore m_sema;
-	};
-
     private:
 
 	/** array with sample data */
@@ -146,11 +111,8 @@ namespace Kwave
 	/** number of samples buffered, e.g. through put() */
 	unsigned int m_buffered;
 
-	/** thread weaver, for processing multi track jobs in parallel */
-	ThreadWeaver::Weaver m_weaver;
-
-	/** job for emitting the output in a seperate thread */
-	BufferJob m_job;
+	/** semaphore for limiting queue depth to 1 */
+	QSemaphore m_sema;
     };
 
 }
