@@ -22,9 +22,10 @@
 #include "config.h"
 
 #include <QtCore/QString>
+#include <QtCore/QUuid>
 
 #include "libkwave/Sample.h"
-#include "libkwave/Track.h"
+#include "libkwave/Stripe.h"
 #include "libkwave/undo/UndoAction.h"
 
 namespace Kwave
@@ -55,10 +56,10 @@ namespace Kwave
 	virtual QString description();
 
 	/** @see UndoAction::undoSize() */
-	virtual unsigned int undoSize();
+	virtual qint64 undoSize();
 
 	/** @see UndoAction::redoSize() */
-	virtual int redoSize();
+	virtual qint64 redoSize();
 
 	/** @see UndoAction::store() */
 	virtual bool store(SignalManager &manager);
@@ -78,8 +79,11 @@ namespace Kwave
 	/** Length of the track in samples */
 	sample_index_t m_length;
 
-	/** track that serves as buffer with undo data */
-	Kwave::Track m_buffer_track;
+	/** storage for all deleted stripes */
+	QList<Kwave::Stripe::List> m_stripes;
+
+	/** unique ID of the deleted track */
+	QUuid m_uuid;
 
     };
 }
