@@ -18,28 +18,28 @@
 #include "config.h"
 #include <errno.h>
 
-#include "libkwave/KwavePlugin.h"
+#include "libkwave/Plugin.h"
 #include "libkwave/PluginManager.h"
 
 #include "AboutPlugin.h"
-#include "AboutKwaveDialog.h"
+#include "AboutDialog.h"
 
-KWAVE_PLUGIN(AboutPlugin, "about", "2.1",
+KWAVE_PLUGIN(Kwave::AboutPlugin, "about", "2.3",
              I18N_NOOP("About Kwave"), "Ralf Waspe & Gilles Caulier");
 
 //***************************************************************************
-AboutPlugin::AboutPlugin(const PluginContext &c)
-    :Kwave::Plugin(c)
+Kwave::AboutPlugin::AboutPlugin(Kwave::PluginManager &plugin_manager)
+    :Kwave::Plugin(plugin_manager)
 {
 }
 
 //***************************************************************************
-int AboutPlugin::start(QStringList& params)
+int Kwave::AboutPlugin::start(QStringList& params)
 {
     Q_UNUSED(params);
 
     // create a new "about" dialog and show it
-    AboutKwaveDialog *dlg = new AboutKwaveDialog(
+    Kwave::AboutDialog *dlg = new Kwave::AboutDialog(
 	parentWidget(),
 	manager().pluginInfoList()
     );
