@@ -28,6 +28,8 @@
 #include <knuminput.h>
 #include <kpushbutton.h>
 
+#include "libkwave/Utils.h"
+
 #include "libgui/ScaleWidget.h"
 #include "libgui/FrequencyResponseWidget.h"
 
@@ -46,13 +48,13 @@ Kwave::BandPassDialog::BandPassDialog(QWidget *parent, double sample_rate)
     // set maximum frequency to sample rate / 2
     double f_max = sample_rate / 2.0;
 
-    slider->setMaximum(static_cast<int>(f_max));
-    slider_2->setMaximum(static_cast<int>(f_max / 2.0));
-    spinbox->setMaximum(static_cast<int>(f_max));
-    spinbox_2->setMaximum(static_cast<int>(f_max / 2.0));
+    slider->setMaximum(Kwave::toInt(f_max));
+    slider_2->setMaximum(Kwave::toInt(f_max / 2.0));
+    spinbox->setMaximum(Kwave::toInt(f_max));
+    spinbox_2->setMaximum(Kwave::toInt(f_max / 2.0));
 
     // initialize the frequency scale widget
-    scale_freq->setMinMax(0, static_cast<int>(f_max));
+    scale_freq->setMinMax(0, Kwave::toInt(f_max));
     scale_freq->setLogMode(false);
     scale_freq->setUnit(i18n("Hz"));
 
@@ -69,11 +71,11 @@ Kwave::BandPassDialog::BandPassDialog(QWidget *parent, double sample_rate)
     freq_response->setFilter(m_filter);
 
     // initialize the controls and the curve display
-    slider->setValue(static_cast<int>(m_frequency));
-    spinbox->setValue(static_cast<int>(m_frequency));
+    slider->setValue(Kwave::toInt(m_frequency));
+    spinbox->setValue(Kwave::toInt(m_frequency));
 
-    slider_2->setValue(static_cast<int>(m_bw));
-    spinbox_2->setValue(static_cast<int>(m_bw));
+    slider_2->setValue(Kwave::toInt(m_bw));
+    spinbox_2->setValue(Kwave::toInt(m_bw));
     updateDisplay();
 
     // changes in the slider or spinbox
@@ -116,7 +118,7 @@ Kwave::BandPassDialog::~BandPassDialog()
 //***************************************************************************
 void Kwave::BandPassDialog::freqValueChanged(int pos)
 {
-    if (static_cast<int>(m_frequency) != pos) {
+    if (Kwave::toInt(m_frequency) != pos) {
 	m_frequency = pos;
 	updateDisplay();
 
@@ -126,7 +128,7 @@ void Kwave::BandPassDialog::freqValueChanged(int pos)
 //***************************************************************************
 void Kwave::BandPassDialog::bwValueChanged(int pos)
 {
-    if (static_cast<int>(m_bw) != pos) {
+    if (Kwave::toInt(m_bw) != pos) {
     	m_bw = pos;
 	updateDisplay();
 
@@ -155,11 +157,11 @@ void Kwave::BandPassDialog::setParams(QStringList &params)
     Q_ASSERT(ok);
     if (ok) m_bw = bw;
 
-    slider->setValue(static_cast<int>(m_frequency));
-    spinbox->setValue(static_cast<int>(m_frequency));
+    slider->setValue(Kwave::toInt(m_frequency));
+    spinbox->setValue(Kwave::toInt(m_frequency));
 
-    slider_2->setValue(static_cast<int>(m_bw));
-    spinbox_2->setValue(static_cast<int>(m_bw));
+    slider_2->setValue(Kwave::toInt(m_bw));
+    spinbox_2->setValue(Kwave::toInt(m_bw));
 
     updateDisplay();
 }
