@@ -168,9 +168,9 @@ void Kwave::MP3Encoder::encodeID3Tags(const Kwave::MetaDataList &meta_data,
 	    }
 	    case ID3_PropertyMap::ENC_GENRE_TYPE:
 	    {
-		int id = Kwave::GenreType::fromID3(str_val);
-		if (id >= 0)
-		    str_val = Kwave::GenreType::name(id, false);
+		int genre = Kwave::GenreType::fromID3(str_val);
+		if (genre >= 0)
+		    str_val = Kwave::GenreType::name(genre, false);
 		// else: user defined genre type, take it as it is
 
 		field->SetEncoding(ID3TE_UTF16);
@@ -277,7 +277,7 @@ bool Kwave::MP3Encoder::encode(QWidget *widget, Kwave::MultiTrackReader &src,
 
     // get info: tracks, sample rate
     const unsigned int tracks     = src.tracks();
-    const unsigned int length     = src.last() - src.first() + 1;
+    const sample_index_t length   = src.last() - src.first() + 1;
     unsigned int       bits       = qBound(8U, ((info.bits() + 7) & ~0x7), 32U);
     const double       rate       = info.rate();
     const unsigned int out_tracks = qMax(tracks, 2U);
@@ -435,7 +435,7 @@ bool Kwave::MP3Encoder::encode(QWidget *widget, Kwave::MultiTrackReader &src,
     const unsigned int buf_len = sizeof(m_write_buffer);
     const int bytes_per_sample = bits / 8;
 
-    unsigned int rest = length;
+    sample_index_t rest = length;
     Kwave::SampleArray in_samples(tracks);
     Kwave::SampleArray out_samples(tracks);
 
@@ -444,13 +444,13 @@ bool Kwave::MP3Encoder::encode(QWidget *widget, Kwave::MultiTrackReader &src,
 	unsigned int y;
 
 	// merge the tracks into the sample buffer
-	quint8 *dst = &(m_write_buffer[0]);
+	quint8 *dst_buffer = &(m_write_buffer[0]);
 	unsigned int count = buf_len / (bytes_per_sample * tracks);
-	if (rest < count) count = rest;
+	if (rest < count) count = Kwave::toUint(rest);
 
 	unsigned int written = 0;
 	for (written = 0; written < count; written++) {
-	    sample_t *src_buf = 0;
+	    const sample_t *src_buf = 0;
 
 	    // fill input buffer with samples
 	    for (x = 0; x < tracks; x++) {
@@ -464,19 +464,19 @@ bool Kwave::MP3Encoder::encode(QWidget *widget, Kwave::MultiTrackReader &src,
 
 	    if (tracks > 2) {
 		// multiply matrix with input to get output
+		const Kwave::SampleArray &in = in_samples;
 		for (y = 0; y < out_tracks; y++) {
 		    double sum = 0;
-		    for (x = 0; x < tracks; x++) {
-			sum += static_cast<double>(in_samples[x]) * mixer[x][y];
-		    }
+		    for (x = 0; x < tracks; x++)
+			sum += static_cast<double>(in[x]) * mixer[x][y];
 		    out_samples[y] = static_cast<sample_t>(sum);
 		}
 
 		// use output of the matrix
-		src_buf = out_samples.data();
+		src_buf = out_samples.constData();
 	    } else {
 		// use input buffer directly
-		src_buf = in_samples.data();
+		src_buf = in_samples.constData();
 	    }
 
 	    // sample conversion from 24bit to raw PCM, native endian
@@ -485,29 +485,29 @@ bool Kwave::MP3Encoder::encode(QWidget *widget, Kwave::MultiTrackReader &src,
 #if Q_BYTE_ORDER == Q_BIG_ENDIAN
 		// big endian
 		if (bits >= 8)
-		    *(dst++) = (s >> 16);
+		    *(dst_buffer++) = static_cast<quint8>(s >> 16);
 		if (bits > 8)
-		    *(dst++) = (s >> 8);
+		    *(dst_buffer++) = static_cast<quint8>(s >> 8);
 		if (bits > 16)
-		    *(dst++) = (s & 0xFF);
+		    *(dst_buffer++) = static_cast<quint8>(s & 0xFF);
 		if (bits > 24)
-		    *(dst++) = 0x00;
+		    *(dst_buffer++) = 0x00;
 #else
 		// little endian
 		if (bits > 24)
-		    *(dst++) = 0x00;
+		    *(dst_buffer++) = 0x00;
 		if (bits > 16)
-		    *(dst++) = s & 0xFF;
+		    *(dst_buffer++) = static_cast<quint8>(s & 0xFF);
 		if (bits > 8)
-		    *(dst++) = (s >> 8);
+		    *(dst_buffer++) = static_cast<quint8>(s >> 8);
 		if (bits >= 8)
-		    *(dst++) = (s >> 16);
+		    *(dst_buffer++) = static_cast<quint8>(s >> 16);
 #endif
 	    }
 	}
 
 	// write out to the stdin of the external process
-	unsigned int bytes_written = m_process.write(
+	qint64 bytes_written = m_process.write(
 	    reinterpret_cast<char *>(&(m_write_buffer[0])),
 	    written * (bytes_per_sample * tracks)
 	);
