@@ -38,6 +38,7 @@
 #include "libkwave/MultiWriter.h"
 #include "libkwave/Sample.h"
 #include "libkwave/String.h"
+#include "libkwave/Utils.h"
 #include "libkwave/VirtualAudioFile.h"
 #include "libkwave/Writer.h"
 
@@ -155,6 +156,7 @@ bool Kwave::WavDecoder::open(QWidget *widget, QIODevice &src)
 
     // check if there is a RIFF chunk at all...
     Kwave::RIFFChunk *riff_chunk = parser.findChunk("/RIFF:WAVE");
+    Kwave::RIFFChunk *fact_chunk = parser.findChunk("/RIFF:WAVE/fact");
     Kwave::RIFFChunk *fmt_chunk  = parser.findChunk("/RIFF:WAVE/fmt ");
     Kwave::RIFFChunk *data_chunk = parser.findChunk("/RIFF:WAVE/data");
 
@@ -235,6 +237,25 @@ bool Kwave::WavDecoder::open(QWidget *widget, QIODevice &src)
 	return false;
     }
 
+    // WORKAROUND: if there is a fact chunk, it must not contain zero
+    //             for the moment the only workaround known is to open
+    //             such broken files in repair mode
+    if (fact_chunk) {
+	qint64 offset = fact_chunk->dataStart();
+	union {
+	    quint32 len;
+	    char    bytes[4];
+	} fact;
+	fact.len = 0;
+	src.seek(offset);
+	src.read(&(fact.bytes[0]), 4);
+	if ((fact.len == 0x00000000) || (fact.len == 0xFFFFFFFF)) {
+	    qWarning("WARNING: invalid 'fact' chunk content: 0x%08X "
+	             "-> silent repair mode", fact.len);
+	    need_repair = true;
+	}
+    }
+
     // final check for structural integrity:
     // we should have exactly one RIFF, fmt and data chunk !
     if ((parser.chunkCount("fmt ") != 1) ||
@@ -278,8 +299,7 @@ bool Kwave::WavDecoder::open(QWidget *widget, QIODevice &src)
     src.seek(fmt_offset);
 
     // get the encoded block of data from the mime source
-    CHECK(static_cast<unsigned int>(src.size()) >
-          sizeof(Kwave::wav_header_t) + 8);
+    CHECK(src.size() > static_cast<qint64>(sizeof(Kwave::wav_header_t) + 8));
 
     // get the header
     src.read(reinterpret_cast<char *>(&header), sizeof(Kwave::wav_fmt_header_t));
@@ -372,7 +392,7 @@ bool Kwave::WavDecoder::open(QWidget *widget, QIODevice &src)
 	return false;
     }
 
-    unsigned int length = afGetFrameCount(fh, AF_DEFAULT_TRACK);
+    AFframecount length = afGetFrameCount(fh, AF_DEFAULT_TRACK);
     tracks = afGetVirtualChannels(fh, AF_DEFAULT_TRACK);
 
     int sample_format;
@@ -400,12 +420,12 @@ bool Kwave::WavDecoder::open(QWidget *widget, QIODevice &src)
 
 	    // read the content into a QString
 	    Kwave::FileProperty prop = m_property_map.property(chunk->name());
-	    unsigned int offset = chunk->dataStart();
-	    unsigned int length = chunk->dataLength();
-	    QByteArray buffer(length+1, 0x00);
-	    src.seek(offset);
-	    src.read(buffer.data(), length);
-	    buffer[length] = 0;
+	    unsigned int ofs = chunk->dataStart();
+	    unsigned int len = chunk->dataLength();
+	    QByteArray buffer(len + 1, 0x00);
+	    src.seek(ofs);
+	    src.read(buffer.data(), len);
+	    buffer[len] = 0;
 	    QString value;
 	    value = QString::fromUtf8(buffer);
 	    info.set(prop, value);
@@ -450,7 +470,7 @@ bool Kwave::WavDecoder::open(QWidget *widget, QIODevice &src)
 	    src.read(reinterpret_cast<char *>(&data), 4);
 	    /* we currently support only 'data' */
 	    if (qstrncmp(reinterpret_cast<const char *>(&data), "data", 4)) {
-		qWarning("cue list entry %d refers to '%s', "\
+		qWarning("cue list entry %d refers to '%s', "
                          "which is not supported -> skipped",
 		         index, QByteArray(
 		         reinterpret_cast<const char *>(&data), 4).data());
@@ -483,7 +503,7 @@ bool Kwave::WavDecoder::open(QWidget *widget, QIODevice &src)
 		bool found = false;
 		QListIterator<Kwave::RIFFChunk *> it(adtl_chunk->subChunks());
 		while (it.hasNext()) {
-		    quint32 data, labl_index;
+		    quint32 labl_index;
 		    labl_chunk = it.next();
 		    /*
 		     * typedef struct {
@@ -506,13 +526,13 @@ bool Kwave::WavDecoder::open(QWidget *widget, QIODevice &src)
 		}
 		if (found) {
 		    Q_ASSERT(labl_chunk);
-		    unsigned int length = labl_chunk->length();
-		    if (length > 4) {
-			length -= 4;
-			name.resize(length);
+		    unsigned int len = labl_chunk->length();
+		    if (len > 4) {
+			len -= 4;
+			name.resize(len);
 			src.seek(labl_chunk->dataStart() + 4);
-			src.read(static_cast<char *>(name.data()), length);
-			if (name[name.count()-1] != '\0')
+			src.read(static_cast<char *>(name.data()), len);
+			if (name[name.count() - 1] != '\0')
 			    name += '\0';
 		    }
 		}
@@ -563,17 +583,16 @@ bool Kwave::WavDecoder::decode(QWidget */*widget*/, Kwave::MultiWriter &dst)
     QVector<Kwave::Writer *>writers;
     for (unsigned int t = 0; t < tracks; t++)
 	writers.append(dst[t]);
-    Q_ASSERT(writers.count() == static_cast<int>(dst.tracks()));
-    if (writers.count() != static_cast<int>(dst.tracks())) return false;
+    Q_ASSERT(writers.count() == Kwave::toInt(dst.tracks()));
+    if (writers.count() != Kwave::toInt(dst.tracks())) return false;
     Kwave::Writer **writer_fast = writers.data();
 
-    unsigned int frame_size = static_cast<unsigned int>(
+    unsigned int frame_size = Kwave::toUint(
 	afGetVirtualFrameSize(fh, AF_DEFAULT_TRACK, 1));
 
     // allocate a buffer for input data
-    const unsigned int buffer_frames = (8*1024);
-    qint32 *buffer = static_cast<qint32 *>(
-	malloc(buffer_frames * frame_size));
+    const unsigned int buffer_frames = (8 * 1024);
+    qint32 *buffer = static_cast<qint32 *>(malloc(buffer_frames * frame_size));
     Q_ASSERT(buffer);
     if (!buffer) return false;
 
@@ -582,7 +601,7 @@ bool Kwave::WavDecoder::decode(QWidget */*widget*/, Kwave::MultiWriter &dst)
     sample_index_t rest = Kwave::FileInfo(metaData()).length();
     while (rest) {
 	unsigned int frames = buffer_frames;
-	if (frames > rest) frames = rest;
+	if (frames > rest) frames = Kwave::toUint(rest);
 	unsigned int buffer_used = afReadFrames(fh,
 	    AF_DEFAULT_TRACK, reinterpret_cast<char *>(buffer), frames);
 
@@ -677,9 +696,9 @@ bool Kwave::WavDecoder::repairChunk(
 
     // recursively go over all sub-chunks
     Kwave::RIFFChunkList &list = chunk->subChunks();
-    foreach (Kwave::RIFFChunk *chunk, list) {
-	if (!chunk) continue;
-	if (!repairChunk(repair_list, chunk, offset))
+    foreach (Kwave::RIFFChunk *c, list) {
+	if (!c) continue;
+	if (!repairChunk(repair_list, c, offset))
 	    return false;
     }
 
