@@ -176,23 +176,25 @@ void Kwave::FileInfoDialog::setupFileInfoTab()
 	    bool found = false;
 	    int comp_found = -1;
 	    foreach (int c, comps) {
-		Kwave::Compression comp(c);
-		if (comp.preferredMimeType() == mimetype) {
+		Kwave::Compression cmp(c);
+		if ((cmp.preferredMimeType() == mimetype) &&
+		     comps.contains(c))
+		{
 		    found = true;
 		    comp_found = c;
 		    break;
 		}
 	    }
-	    if (!found || (comp_found != comp)) {
+	    if (found && (comp_found != comp)) {
 		int cn = ((comp_found == -1) && !comps.isEmpty()) ?
 		    comps.last() : comp_found;
+		if (cn < 0) cn = 0;
+
 		Kwave::Compression comp_old(comp);
 		Kwave::Compression comp_new(cn);
-		if (comp_old.toInt() != comp_new.toInt()) {
-		    qDebug("mime type/compression mismatch: "
-			"switch from '%s' to '%s'",
-			DBG(comp_old.name()), DBG(comp_new.name()));
-		}
+		qDebug("mime type/compression mismatch: "
+		       "switch from '%s' to '%s'",
+		       DBG(comp_old.name()), DBG(comp_new.name()));
 		m_info.set(Kwave::INF_COMPRESSION, comp_new.toInt());
 	    }
 	}
@@ -258,16 +260,16 @@ void Kwave::FileInfoDialog::setupFileInfoTab()
         lblLength->text().length() - 1),
         i18n("Shows the length of the file in samples\n"
              "and if possible as time."));
-    unsigned int samples = m_info.length();
+    sample_index_t samples = m_info.length();
     double rate = m_info.rate();
-    if (rate != 0) {
+    if (!qFuzzyIsNull(rate)) {
 	double ms = static_cast<double>(samples) * 1E3 / rate;
 	txtLength->setText(i18n("%1 (%2 samples)",
 	    Kwave::ms2string(ms),
-	    KGlobal::locale()->formatLong(samples)));
+	    Kwave::samples2string(samples)));
     } else {
 	txtLength->setText(i18n("%1 samples",
-	    KGlobal::locale()->formatLong(samples)));
+	    Kwave::samples2string(samples)));
     }
 
     /* sample format */
@@ -352,7 +354,7 @@ void Kwave::FileInfoDialog::setupMpegTab()
 
     /* MPEG version */
     initInfo(lblMpegVersion, cbMpegVersion,  Kwave::INF_MPEG_VERSION);
-    int ver = static_cast<int>(
+    int ver = Kwave::toInt(
         (2.0 * QVariant(m_info.get(Kwave::INF_MPEG_VERSION)).toDouble()));
     // 1, 2, 2.5 -> 2, 4, 5
     if ((ver < 1) || (ver > 5)) ver = 4; // default = version 2
