@@ -50,6 +50,7 @@
 #include "libkwave/SampleEncoderLinear.h"
 #include "libkwave/SampleFormat.h"
 #include "libkwave/String.h"
+#include "libkwave/Utils.h"
 
 #include "PlayBack-ALSA.h"
 
@@ -294,7 +295,6 @@ QList<int> Kwave::PlayBackALSA::detectSupportedFormats(const QString &device)
     // start with an empty list
     QList<int> supported_formats;
 
-    int err;
     ALSA_MALLOC_WRAPPER(snd_pcm_hw_params) p;
 
     if (!p) return supported_formats;
@@ -302,7 +302,7 @@ QList<int> Kwave::PlayBackALSA::detectSupportedFormats(const QString &device)
     snd_pcm_t *pcm = openDevice(device);
     if (!pcm) return supported_formats;
 
-    if (!snd_pcm_hw_params_any(pcm, p) < 0) {
+    if (snd_pcm_hw_params_any(pcm, p) < 0) {
 	if (pcm != m_handle) snd_pcm_close(pcm);
 	return supported_formats;
     }
@@ -314,7 +314,7 @@ QList<int> Kwave::PlayBackALSA::detectSupportedFormats(const QString &device)
     for (unsigned int i=0; i < count; i++) {
 	// test the sample format
 	snd_pcm_format_t format = _known_formats[i];
-	err = snd_pcm_hw_params_test_format(pcm, p, format);
+	int err = snd_pcm_hw_params_test_format(pcm, p, format);
 	if (err < 0) continue;
 
 	const snd_pcm_format_t *fmt = &(_known_formats[i]);
@@ -585,8 +585,7 @@ QString Kwave::PlayBackALSA::open(const QString &device, double rate,
     // initialize the list of supported formats
     m_supported_formats = detectSupportedFormats(device);
 
-    int err = openDevice(device, static_cast<unsigned int>(rate),
-	channels, bits);
+    int err = openDevice(device, Kwave::toUint(rate), channels, bits);
     if (err) {
 	QString reason;
 	switch (err) {
@@ -621,7 +620,7 @@ QString Kwave::PlayBackALSA::open(const QString &device, double rate,
     unsigned int chunk_bytes = m_chunk_size * m_bytes_per_sample;
     Q_ASSERT(chunk_bytes);
     if (!chunk_bytes) return QString();
-    unsigned int n = static_cast<unsigned int>(ceil(
+    unsigned int n = Kwave::toUint(ceil(
 	static_cast<float>(1 << m_bufbase) /
 	static_cast<float>(chunk_bytes)));
     if (n < 1) n = 1;
@@ -652,7 +651,7 @@ int Kwave::PlayBackALSA::write(const Kwave::SampleArray &samples)
 
     QByteArray raw(bytes, char(0));
     m_encoder->encode(samples, m_channels, raw);
-    MEMCPY(m_buffer.data() + m_buffer_used, raw.data(), bytes);
+    MEMCPY(m_buffer.data() + m_buffer_used, raw.constData(), bytes);
     m_buffer_used += bytes;
 
     // write buffer to device if full
@@ -674,9 +673,8 @@ int Kwave::PlayBackALSA::flush()
 	unsigned int buffer_samples = m_buffer_size / m_bytes_per_sample;
 	unsigned int timeout = (m_rate > 0) ?
 	    3 * ((1000 * buffer_samples) /
-	    static_cast<unsigned int>(m_rate)) : 1000U;
+	    Kwave::toUint(m_rate)) : 1000U;
 	quint8 *p = reinterpret_cast<quint8 *>(m_buffer.data());
-	int r;
 
 	// pad the buffer with silence if necessary
 	if (samples < m_chunk_size) {
@@ -689,9 +687,8 @@ int Kwave::PlayBackALSA::flush()
 
 	while (samples > 0) {
 	    // try to write as much as the device accepts
-	    r = snd_pcm_writei(m_handle, p, samples);
-	    if ((r == -EAGAIN) || ((r >= 0) &&
-	        (r < static_cast<int>(samples))))
+	    int r = snd_pcm_writei(m_handle, p, samples);
+	    if ((r == -EAGAIN) || ((r >= 0) && (r < Kwave::toInt(samples))))
 	    {
 		snd_pcm_wait(m_handle, timeout);
 	    } else if (r == -EPIPE) {
@@ -731,7 +728,7 @@ int Kwave::PlayBackALSA::flush()
 	    }
 	    if (r > 0) {
 		// advance in the buffer
-		Q_ASSERT(r <= static_cast<int>(samples));
+		Q_ASSERT(r <= Kwave::toInt(samples));
 		p       += r * m_bytes_per_sample;
 		samples -= r;
 	    }
@@ -830,7 +827,7 @@ void Kwave::PlayBackALSA::scanDevices()
 //  	    qDebug("  Subdevices: %i/%i\n",
 // 		snd_pcm_info_get_subdevices_avail(pcminfo), count);
 	    if (count > 1) {
-		for (idx = 0; idx < static_cast<int>(count); idx++) {
+		for (idx = 0; idx < Kwave::toInt(count); idx++) {
 		    snd_pcm_info_set_subdevice(pcminfo, idx);
 		    if ((err = snd_ctl_pcm_info(handle, pcminfo)) < 0) {
 			qWarning("ctrl digital audio playback info (%i): %s",
@@ -839,7 +836,7 @@ void Kwave::PlayBackALSA::scanDevices()
 			QString hwdev = hw_device + _(",%1").arg(idx);
 			QString subdevice_name =
 			    _(snd_pcm_info_get_subdevice_name(pcminfo));
-			QString name = QString(
+			QString full_name = QString(
 			    i18n("Card %1: ", card) + card_name +
 			    _("|sound_card||") +
 			    i18n("Device %1: ", dev) + device_name +
@@ -847,20 +844,20 @@ void Kwave::PlayBackALSA::scanDevices()
 			    i18n("Subdevice %1: ", idx) + subdevice_name +
 			    _("|sound_subdevice")
 			);
-			qDebug("# '%s' -> '%s'", DBG(hwdev), DBG(name));
-			m_device_list.insert(name, hwdev);
+			qDebug("# '%s' -> '%s'", DBG(hwdev), DBG(full_name));
+			m_device_list.insert(full_name, hwdev);
 		    }
 		}
 	    } else {
 		// no sub-devices
-		QString name = QString(
+		QString full_name = QString(
 		    i18n("Card %1: ", card) +
 		         card_name + _("|sound_card||") +
 		    i18n("Device %1: ", dev) +
 		          device_name + _("|sound_subdevice")
 		);
 // 		qDebug("# '%s' -> '%s'", DBG(hw_device), DBG(name));
-		m_device_list.insert(name, hw_device);
+		m_device_list.insert(full_name, hw_device);
 	    }
 	}
 	snd_ctl_close(handle);
