@@ -40,6 +40,7 @@
 #include "libkwave/memcpy.h"
 #include "libkwave/SampleEncoderLinear.h"
 #include "libkwave/String.h"
+#include "libkwave/Utils.h"
 
 #include "PlayBack-PulseAudio.h"
 
@@ -76,8 +77,7 @@
 //***************************************************************************
 Kwave::PlayBackPulseAudio::PlayBackPulseAudio(const Kwave::FileInfo &info)
     :Kwave::PlayBackDevice(), m_mainloop_thread(this, QVariant()),
-     m_mainloop_lock(), m_mainloop_signal(),
-     m_info(info), m_rate(0), m_channels(0),
+     m_mainloop_lock(), m_mainloop_signal(), m_info(info), m_rate(0),
      m_bytes_per_sample(0), m_buffer(0), m_buffer_size(0), m_buffer_used(0),
      m_bufbase(10), m_pa_proplist(0), m_pa_mainloop(0), m_pa_context(0),
      m_pa_stream(0), m_device_list()
@@ -447,6 +447,12 @@ QString Kwave::PlayBackPulseAudio::open(const QString &device, double rate,
 	DBG(device.split(_("|")).at(0)), rate, channels,
 	bits, bufbase);
 
+    m_rate = rate;
+
+    if (channels > 255)
+	return i18n("%1 channels are not supported, maximum is 255").arg(
+	    channels);
+
     // close the previous device
     if (m_pa_stream) close();
 
@@ -491,8 +497,8 @@ QString Kwave::PlayBackPulseAudio::open(const QString &device, double rate,
 #else
     sample_spec.format = PA_SAMPLE_S24_32LE;
 #endif
-    sample_spec.channels = channels;
-    sample_spec.rate     = static_cast<quint32>(rate);
+    sample_spec.channels = static_cast<uint8_t>(channels);
+    sample_spec.rate     = static_cast<uint32_t>(m_rate);
 
     // use the current title / filename or fixed string as stream name
     QString name;
@@ -614,15 +620,15 @@ int Kwave::PlayBackPulseAudio::write(const Kwave::SampleArray &samples)
     Q_ASSERT (m_buffer_used + bytes <= m_buffer_size);
     if (m_buffer_used + bytes > m_buffer_size) {
 	qWarning("PlayBackPulseAudio::write(): buffer overflow ?! (%u/%u)",
-	         static_cast<unsigned int>(m_buffer_used),
-	         static_cast<unsigned int>(m_buffer_size));
+	         Kwave::toUint(m_buffer_used),
+	         Kwave::toUint(m_buffer_size));
 	m_buffer_used = 0;
 	return -EIO;
     }
 
     // copy the samples
     MEMCPY(reinterpret_cast<quint8 *>(m_buffer) + m_buffer_used,
-	   samples.data(), bytes);
+	   samples.constData(), bytes);
     m_buffer_used += bytes;
 
     // write the buffer if it is full
@@ -640,7 +646,8 @@ int Kwave::PlayBackPulseAudio::flush()
 
     // calculate a reasonable time for the timeout (16 buffers)
     int samples_per_buffer = (m_buffer_size / m_bytes_per_sample);
-    int ms = (samples_per_buffer * 1000) / m_info.rate();
+    int ms = (!qFuzzyIsNull(m_rate)) ?
+	Kwave::toInt((samples_per_buffer * 1000.0) / m_rate) : 0;
     int timeout = (ms + 1) * 16;
     if (timeout < TIMEOUT_MIN_FLUSH)
 	timeout = TIMEOUT_MIN_FLUSH;
@@ -730,7 +737,8 @@ int Kwave::PlayBackPulseAudio::close()
 
 	// calculate a reasonable time for the timeout (16 buffers)
 	int samples_per_buffer = (m_buffer_size / m_bytes_per_sample);
-	int ms = (samples_per_buffer * 1000) / m_info.rate();
+	int ms = (!qFuzzyIsNull(m_rate)) ?
+	    Kwave::toInt((samples_per_buffer * 1000.0) / m_rate) : 0;
 	int timeout = (ms + 1) * 4;
 	if (timeout < TIMEOUT_MIN_DRAIN)
 	    timeout = TIMEOUT_MIN_DRAIN;
@@ -809,10 +817,10 @@ void Kwave::PlayBackPulseAudio::scanDevices()
 
 	// if the name is not unique, add the internal sink name
 	int unique = true;
-	foreach (QString s, m_device_list.keys()) {
-	    if (s == sink) continue;
-	    if ((m_device_list[s].m_description == description) &&
-		(m_device_list[s].m_driver      == driver))
+	foreach (QString sn, m_device_list.keys()) {
+	    if (sn == sink) continue;
+	    if ((m_device_list[sn].m_description == description) &&
+		(m_device_list[sn].m_driver      == driver))
 	    {
 		unique = false;
 		break;
