@@ -25,6 +25,7 @@
 
 #include "libkwave/Compression.h"
 #include "libkwave/String.h"
+#include "libkwave/Utils.h"
 
 #include "Record-ALSA.h"
 
@@ -142,9 +143,6 @@ static const snd_pcm_format_t _known_formats[] =
 #endif
 };
 
-/** sleep seconds, used for recording */
-static const unsigned int g_sleep_min = 0;
-
 //***************************************************************************
 /** find out the SampleFormat of an ALSA format */
 static Kwave::SampleFormat sample_format_of(snd_pcm_format_t fmt)
@@ -227,17 +225,16 @@ void Kwave::RecordALSA::detectSupportedFormats()
     Q_ASSERT(m_handle);
     if (!m_handle || !m_hw_params) return;
 
-    int err;
-    if (!snd_pcm_hw_params_any(m_handle, m_hw_params) < 0) return;
+    if (snd_pcm_hw_params_any(m_handle, m_hw_params) < 0) return;
 
     // try all known formats
 //     qDebug("--- list of supported formats --- ");
     const unsigned int count =
 	sizeof(_known_formats) / sizeof(_known_formats[0]);
-    for (unsigned int i=0; i < count; i++) {
+    for (unsigned int i = 0; i < count; i++) {
 	// test the sample format
 	snd_pcm_format_t format = _known_formats[i];
-	err = snd_pcm_hw_params_test_format(m_handle, m_hw_params, format);
+	int err = snd_pcm_hw_params_test_format(m_handle, m_hw_params, format);
 	if (err < 0) continue;
 
 	const snd_pcm_format_t *fmt = &(_known_formats[i]);
@@ -401,8 +398,7 @@ int Kwave::RecordALSA::initialize()
 	return -EINVAL;
     }
 
-    unsigned int rrate = m_rate > 0 ?
-	static_cast<unsigned int>(rint(m_rate)) : 0;
+    unsigned int rrate = (m_rate > 0) ? Kwave::toUint(rint(m_rate)) : 0;
     err = snd_pcm_hw_params_set_rate_near(m_handle, m_hw_params, &rrate, 0);
     if (err < 0) {
 	qWarning("Cannot set sample rate: %s", snd_strerror(err));
@@ -412,7 +408,7 @@ int Kwave::RecordALSA::initialize()
 //     qDebug("   real rate = %u", rrate);
     if (m_rate * 1.05 < rrate || m_rate * 0.95 > rrate) {
 	qWarning("rate is not accurate (requested = %iHz, got = %iHz)",
-	         static_cast<int>(m_rate), static_cast<int>(rrate));
+	         Kwave::toInt(m_rate), Kwave::toInt(rrate));
     }
     m_rate = rrate;
 
@@ -601,13 +597,13 @@ int Kwave::RecordALSA::read(QByteArray &buffer, unsigned int offset)
 
     // try to read as much as the device accepts
     Q_ASSERT(samples);
-    Q_ASSERT(offset + samples <= static_cast<unsigned int>(buffer.size()));
+    Q_ASSERT(offset + samples <= Kwave::toUint(buffer.size()));
     int r = snd_pcm_readi(m_handle, buffer.data() + offset, samples);
 
     // handle all negative result codes
     if (r == -EAGAIN) {
 	unsigned int timeout = (m_rate > 0) ?
-	    (((1000 * samples) / 4) / static_cast<unsigned int>(m_rate)) : 10U;
+	    (((1000 * samples) / 4) / Kwave::toUint(m_rate)) : 10U;
 	snd_pcm_wait(m_handle, timeout);
 	return -EAGAIN;
     } else if (r == -EPIPE) {
@@ -647,8 +643,8 @@ int Kwave::RecordALSA::read(QByteArray &buffer, unsigned int offset)
     // no error, successfully read something:
     // advance in the buffer
 //     qDebug("<<< after read, r=%d", r);
-    Q_ASSERT(r <= static_cast<int>(samples));
-    if (r > static_cast<int>(samples)) r = samples;
+    Q_ASSERT(r <= Kwave::toInt(samples));
+    if (r > Kwave::toInt(samples)) r = samples;
 
     return (r * m_bytes_per_sample);
 }
@@ -714,11 +710,10 @@ int Kwave::RecordALSA::tracks()
 QList<double> Kwave::RecordALSA::detectSampleRates()
 {
     QList<double> list;
-    int err;
 
     if (!m_handle || !m_hw_params) return list;
 
-    if (!snd_pcm_hw_params_any(m_handle, m_hw_params) < 0) return list;
+    if (snd_pcm_hw_params_any(m_handle, m_hw_params) < 0) return list;
 
     static const unsigned int known_rates[] = {
 	  1000, // (just for testing)
@@ -762,11 +757,14 @@ QList<double> Kwave::RecordALSA::detectSampleRates()
     for (unsigned int i = 0; i < ELEMENTS_OF(known_rates); i++) {
 	unsigned int rate = known_rates[i];
 
-	err = snd_pcm_hw_params_test_rate(m_handle, m_hw_params, rate, 0);
+	int err = snd_pcm_hw_params_test_rate(m_handle, m_hw_params, rate, 0);
 	if (err < 0) continue;
 
 	// do not produce duplicates
-	if (list.contains(rate)) continue;
+	bool is_duplicate = false;
+	foreach (const double &r, list)
+	    if (qFuzzyCompare(rate, r)) { is_duplicate = true; break; }
+	if (is_duplicate) continue;
 
 // 	qDebug("found rate %u Hz", rate);
 	list.append(rate);
@@ -778,7 +776,7 @@ QList<double> Kwave::RecordALSA::detectSampleRates()
 //***************************************************************************
 int Kwave::RecordALSA::setSampleRate(double &new_rate)
 {
-    if (new_rate != m_rate) m_initialized = false;
+    if (!qFuzzyCompare(new_rate, m_rate)) m_initialized = false;
     m_rate = new_rate;
     return 0;
 }
@@ -907,7 +905,7 @@ QList<Kwave::SampleFormat> Kwave::RecordALSA::detectSampleFormats()
 	// only accept bits/sample if compression types
 	// and bits per sample match
 	if (compression_of(*fmt) != m_compression) continue;
-	if (snd_pcm_format_width(*fmt) != static_cast<int>(m_bits_per_sample))
+	if (snd_pcm_format_width(*fmt) != Kwave::toInt(m_bits_per_sample))
 	    continue;
 
 	// do not produce duplicates
@@ -1033,7 +1031,7 @@ void Kwave::RecordALSA::scanDevices()
 //  	    qDebug("  Subdevices: %i/%i\n",
 // 		snd_pcm_info_get_subdevices_avail(pcminfo), count);
 	    if (count > 1) {
-		for (idx = 0; idx < static_cast<int>(count); idx++) {
+		for (idx = 0; idx < Kwave::toInt(count); idx++) {
 		    snd_pcm_info_set_subdevice(pcminfo, idx);
 		    if ((err = snd_ctl_pcm_info(handle, pcminfo)) < 0) {
 			qWarning("ctrl digital audio playback info (%i): %s",
@@ -1042,27 +1040,27 @@ void Kwave::RecordALSA::scanDevices()
 			QString hwdev = hw_device + _(",%1").arg(idx);
 			QString subdevice_name = _(
 			    snd_pcm_info_get_subdevice_name(pcminfo));
-			QString name =
+			QString full_name =
 			    i18n("Card %1: ", card_name) +
 			    _("|sound_card||") +
 			    i18n("Device %1: ", device_name) +
 			    _("|sound_device||") +
 			    i18n("Subdevice %1: ", subdevice_name) +
 			    _("|sound_subdevice");
-			qDebug("# '%s' -> '%s'", DBG(hwdev), DBG(name));
-			m_device_list.insert(name, hwdev);
+			qDebug("# '%s' -> '%s'", DBG(hwdev), DBG(full_name));
+			m_device_list.insert(full_name, hwdev);
 		    }
 		}
 	    } else {
 		// no sub-devices
-		QString name = QString(
+		QString full_name = QString(
 		    i18n("Card %1: ", card_name) +
 		    _("|sound_card||") +
 		    i18n("Device %1: ", device_name) +
 		    _("|sound_subdevice")
 		)/*.arg(card).arg(dev)*/;
 // 		qDebug("# '%s' -> '%s'", hw_device.data(), name.data());
-		m_device_list.insert(name, hw_device);
+		m_device_list.insert(full_name, hw_device);
 	    }
 	}
 
