@@ -169,16 +169,9 @@ Kwave::RecordDialog::RecordDialog(QWidget *parent, QStringList &params,
     slSourceBufferSize->setValue(m_params.buffer_size);
     sourceBufferSizeChanged(m_params.buffer_size);
 
-    // various displays: level meter, oscilloscope, FFT, Overview
-    chkDisplayLevelMeter->setChecked(m_params.display_level_meter);
-    displayLevelMeterChecked(m_params.display_level_meter);
-    chkDisplayOscilloscope->setChecked(m_params.display_oscilloscope);
-    chkDisplayFFT->setChecked(m_params.display_fft);
-    chkDisplayOverview->setChecked(m_params.display_overview);
-
     // after this point all controls have their initial values
 
-    /* --- connect some missing lowlevel gui functionality --- */
+    /* --- connect some missing low level GUI functionality --- */
 
     connect(cbMethod, SIGNAL(activated(int)),
             this, SLOT(methodSelected(int)));
@@ -206,10 +199,6 @@ Kwave::RecordDialog::RecordDialog(QWidget *parent, QStringList &params,
     connect(cbDevice, SIGNAL(activated(const QString &)),
             this, SLOT(setDevice(const QString &)));
 
-    // visualizations
-    connect(chkDisplayLevelMeter, SIGNAL(toggled(bool)),
-            this, SLOT(displayLevelMeterChecked(bool)));
-
     // setup controls
     connect(chkRecordTime, SIGNAL(toggled(bool)),
             this, SLOT(recordTimeChecked(bool)));
@@ -304,6 +293,12 @@ Kwave::RecordDialog::RecordDialog(QWidget *parent, QStringList &params,
     tabRecord->setCurrentIndex(0);
     if (page) delete page;
 
+    // add the "Done" button manually, otherwise it would have "Cancel" semantic
+    KGuiItem item = KStandardGuiItem::ok();
+    item.setText(i18n("&Done"));
+    buttonBox->addButton(item, QDialogButtonBox::AcceptRole,
+	                 this, SLOT(accept()));
+
     // set the focus onto the "Record" button
     btRecord->setFocus();
 }
@@ -724,7 +719,7 @@ void Kwave::RecordDialog::setTracks(unsigned int tracks)
 void Kwave::RecordDialog::tracksChanged(int tracks)
 {
     if (tracks < 1) return; // no device
-    if (tracks == static_cast<int>(m_params.tracks)) return;
+    if (tracks == Kwave::toInt(m_params.tracks)) return;
 
     m_params.tracks = tracks;
     emit sigTracksChanged(tracks);
@@ -777,7 +772,7 @@ void Kwave::RecordDialog::sampleRateChanged(const QString &rate)
 {
     if (!rate.length()) return; // no rate selected, combo box clear
     double sample_rate = string2rate(rate);
-    if (sample_rate == m_params.sample_rate) return;
+    if (qFuzzyCompare(sample_rate, m_params.sample_rate)) return;
 
     m_params.sample_rate = sample_rate;
     emit sampleRateChanged(sample_rate);
@@ -1184,13 +1179,13 @@ void Kwave::RecordDialog::updateBufferState(unsigned int count,
 	case Kwave::REC_DONE: {
 	    if (m_samples_recorded > 1) {
 		double rate = m_params.sample_rate;
-		double ms = (rate) ?
+		double ms = (rate > 0) ?
 		    ((static_cast<double>(m_samples_recorded) / rate) * 1E3)
 		    : 0;
 		txt = _(" ") +
 		    i18n("Length: %1", Kwave::ms2string(ms)) +
 		    _(" ") + i18n("(%1 samples)",
-		    KGlobal::locale()->formatLong(m_samples_recorded));
+		    Kwave::samples2string(m_samples_recorded));
 	    } else txt = _("");
 	    break;
 	}
@@ -1258,25 +1253,6 @@ void Kwave::RecordDialog::triggerChanged(int trigger)
     m_params.record_trigger = trigger;
 }
 
-//***************************************************************************
-void Kwave::RecordDialog::displayLevelMeterChecked(bool enabled)
-{
-    m_params.display_level_meter = enabled;
-
-    Q_ASSERT(level_meter);
-    Q_ASSERT(lbl_level_meter);
-    if (!level_meter || !lbl_level_meter) return;
-    if (enabled) {
-	level_meter->show();
-	lbl_level_meter->show();
-    } else {
-	level_meter->setTracks(0);
-	level_meter->reset();
-	level_meter->hide();
-	lbl_level_meter->hide();
-    }
-}
-
 //***************************************************************************
 void Kwave::RecordDialog::updateBufferProgressBar()
 {
@@ -1305,7 +1281,7 @@ void Kwave::RecordDialog::updateEffects(unsigned int track,
 {
     if (!buffer.size()) return;
 
-    if (m_params.display_level_meter && level_meter) {
+    if (level_meter) {
 	level_meter->setTracks(m_params.tracks);
 	level_meter->setSampleRate(m_params.sample_rate);
 	level_meter->updateTrack(track, buffer);
@@ -1314,7 +1290,7 @@ void Kwave::RecordDialog::updateEffects(unsigned int track,
 }
 
 //***************************************************************************
-void Kwave::RecordDialog::setRecordedSamples(unsigned int samples_recorded)
+void Kwave::RecordDialog::setRecordedSamples(sample_index_t samples_recorded)
 {
     // if (!m_params.record_time_limited) return; // not of interest
     m_samples_recorded = samples_recorded;
