@@ -34,11 +34,12 @@ namespace Kwave
      * (sorted, preferred first)
      */
     typedef enum {
-	RECORD_NONE = 0, /**< none selected */
-	RECORD_JACK,     /**< Jack sound daemon */
-	RECORD_ALSA,     /**< ALSA native */
-	RECORD_OSS,      /**< OSS native or ALSA OSS emulation */
-	RECORD_INVALID   /**< (keep this the last entry, EOL delimiter) */
+	RECORD_NONE = 0,   /**< none selected */
+	RECORD_JACK,       /**< Jack sound daemon */
+	RECORD_ALSA,       /**< ALSA native */
+	RECORD_OSS,        /**< OSS native or ALSA OSS emulation */
+	RECORD_PULSEAUDIO, /**< PulseAudio sound daemon */
+	RECORD_INVALID     /**< (keep this the last entry, EOL delimiter) */
     } record_method_t;
 
     /** post-increment operator for the record method */
@@ -104,11 +105,6 @@ namespace Kwave
 
 	unsigned int buffer_count;	/**< number of buffers */
 	unsigned int buffer_size;	/**< power of the record buffer size */
-
-	bool display_level_meter;	/**< enable level meter display */
-	bool display_oscilloscope;	/**< enable oscilloscope display */
-	bool display_fft;		/**< enable fft display */
-	bool display_overview;		/**< enable overview display */
     };
 }
 
