@@ -20,6 +20,7 @@
 #include <errno.h>
 #include <math.h>
 #include <stdlib.h>
+#include <new>
 
 #include <QtCore/QtGlobal>
 #include <QtGui/QCursor>
@@ -43,6 +44,7 @@
 #include "libkwave/SampleFormat.h"
 #include "libkwave/SignalManager.h"
 #include "libkwave/String.h"
+#include "libkwave/Utils.h"
 #include "libkwave/Writer.h"
 
 #include "RecordDevice.h"
@@ -52,8 +54,9 @@
 #include "SampleDecoderLinear.h"
 #include "Record-ALSA.h"
 #include "Record-OSS.h"
+#include "Record-PulseAudio.h"
 
-KWAVE_PLUGIN(Kwave::RecordPlugin, "record", "2.3",
+KWAVE_PLUGIN(Kwave::RecordPlugin, "record", "2.4",
              I18N_NOOP("Record"), "Thomas Eschenbacher");
 
 //***************************************************************************
@@ -126,8 +129,8 @@ QStringList *Kwave::RecordPlugin::setup(QStringList &previous_params)
 	    SLOT(changeSampleFormat(Kwave::SampleFormat)));
     connect(m_dialog, SIGNAL(sigBuffersChanged()),
             this,     SLOT(buffersChanged()));
-    connect(this,     SIGNAL(sigRecordedSamples(unsigned int)),
-            m_dialog, SLOT(setRecordedSamples(unsigned int)));
+    connect(this,     SIGNAL(sigRecordedSamples(sample_index_t)),
+            m_dialog, SLOT(setRecordedSamples(sample_index_t)));
 
     connect(m_dialog,      SIGNAL(sigTriggerChanged(bool)),
             &m_controller, SLOT(enableTrigger(bool)));
@@ -174,7 +177,7 @@ QStringList *Kwave::RecordPlugin::setup(QStringList &previous_params)
 
     QStringList *list = new QStringList();
     Q_ASSERT(list);
-    if (list && m_dialog->exec()) {
+    if (list && (m_dialog->exec() == QDialog::Accepted)) {
 	// user has pressed "OK"
 	*list = m_dialog->params().toList();
     } else {
@@ -195,18 +198,16 @@ QStringList *Kwave::RecordPlugin::setup(QStringList &previous_params)
     if (m_decoder) delete m_decoder;
     m_decoder = 0;
 
-    if (m_dialog) {
-	Kwave::FileInfo info(signalManager().metaData());
-	info.setLength(signalLength());
-	info.setTracks(m_dialog->params().tracks);
-	signalManager().setFileInfo(info, false);
-	delete m_dialog;
-	m_dialog = 0;
-    }
+    delete m_dialog;
+    m_dialog = 0;
 
     // flush away all prerecording buffers
     m_prerecording_queue.clear();
 
+    // enable undo again if we recorded something
+    if (!signalManager().isEmpty())
+	signalManager().enableUndo();
+
     return list;
 }
 
@@ -267,6 +268,14 @@ void Kwave::RecordPlugin::setMethod(Kwave::record_method_t method)
 		    Q_ASSERT(m_device);
 		    break;
 #endif /* HAVE_ALSA_SUPPORT */
+
+#ifdef HAVE_PULSEAUDIO_SUPPORT
+		case Kwave::RECORD_PULSEAUDIO:
+		    m_device = new Kwave::RecordPulseAudio();
+		    Q_ASSERT(m_device);
+		    break;
+#endif /* HAVE_PULSEAUDIO_SUPPORT */
+
 		default:
 		    qDebug("unsupported recording method (%d)",
 			static_cast<int>(method));
@@ -506,7 +515,7 @@ void Kwave::RecordPlugin::changeSampleRate(double new_rate)
     if (!m_dialog) return;
 
     InhibitRecordGuard _lock(*this); // don't record while settings change
-//     qDebug("RecordPlugin::changeSampleRate(%u)", static_cast<int>(new_rate));
+//     qDebug("RecordPlugin::changeSampleRate(%u)", Kwave::toInt(new_rate));
 
     if (!m_device || m_device_name.isNull()) {
 	// no device -> dummy/shortcut
@@ -517,8 +526,11 @@ void Kwave::RecordPlugin::changeSampleRate(double new_rate)
 
     // check the supported sample rates
     QList<double> supported_rates = m_device->detectSampleRates();
+    bool is_supported = false;
+    foreach (const double &r, supported_rates)
+	if (qFuzzyCompare(new_rate, r)) { is_supported = true; break; }
     double rate = new_rate;
-    if (!supported_rates.isEmpty() && !supported_rates.contains(rate)) {
+    if (!is_supported) {
 	// find the nearest sample rate
 	double nearest = supported_rates.last();
 	foreach (double r, supported_rates) {
@@ -529,9 +541,9 @@ void Kwave::RecordPlugin::changeSampleRate(double new_rate)
 
 	const QString sr1(m_dialog->rate2string(new_rate));
 	const QString sr2(m_dialog->rate2string(rate));
-	if ((static_cast<int>(new_rate) > 0) &&
-	    (static_cast<int>(rate) > 0) &&
-	    (static_cast<int>(new_rate) != static_cast<int>(rate)))
+	if ((Kwave::toInt(new_rate) > 0) &&
+	    (Kwave::toInt(rate) > 0) &&
+	    (Kwave::toInt(new_rate) != Kwave::toInt(rate)))
 	    notice(i18n("%1 Hz is not supported, "\
 		        "using %2 Hz", sr1, sr2));
     }
@@ -546,9 +558,9 @@ void Kwave::RecordPlugin::changeSampleRate(double new_rate)
 
 	const QString sr1(m_dialog->rate2string(new_rate));
 	const QString sr2(m_dialog->rate2string(rate));
-	if ((static_cast<int>(new_rate) > 0) &&
-	    (static_cast<int>(rate) > 0) &&
-	    (static_cast<int>(new_rate) != static_cast<int>(rate)))
+	if ((Kwave::toInt(new_rate) > 0) &&
+	    (Kwave::toInt(rate) > 0) &&
+	    (Kwave::toInt(new_rate) != Kwave::toInt(rate)))
 	    notice(i18n("%1 Hz failed, using %2 Hz", sr1, sr2));
     }
     m_dialog->setSampleRate(rate);
@@ -637,15 +649,15 @@ void Kwave::RecordPlugin::changeBitsPerSample(unsigned int new_bits)
 	// find the nearest resolution
 	int nearest = supported_bits.last();
 	foreach (unsigned int b, supported_bits) {
-	    if (qAbs(static_cast<int>(b) - nearest) <= qAbs(bits - nearest))
-	        nearest = static_cast<int>(b);
+	    if (qAbs(Kwave::toInt(b) - nearest) <= qAbs(bits - nearest))
+	        nearest = Kwave::toInt(b);
 	}
 	bits = nearest;
 
-	if ((static_cast<int>(new_bits) > 0) && (bits > 0)) notice(
+	if ((Kwave::toInt(new_bits) > 0) && (bits > 0)) notice(
 	    i18n("%1 bits per sample is not supported, "\
 	         "using %2 bits per sample",
-		 static_cast<int>(new_bits), bits));
+		 Kwave::toInt(new_bits), bits));
     }
     m_dialog->setSupportedBits(supported_bits);
 
@@ -655,10 +667,10 @@ void Kwave::RecordPlugin::changeBitsPerSample(unsigned int new_bits)
 	// revert to the current device setting if failed
 	bits = m_device->bitsPerSample();
 	if (bits < 0) bits = 0;
-	if ((new_bits> 0) && (bits > 0)) notice(
+	if ((new_bits > 0) && (bits > 0)) notice(
 	    i18n("%1 bits per sample failed, "
 		 "using %2 bits per sample",
-		 static_cast<int>(new_bits), bits));
+		 Kwave::toInt(new_bits), bits));
     }
     m_dialog->setBitsPerSample(bits);
 
@@ -723,7 +735,7 @@ void Kwave::RecordPlugin::changeSampleFormat(Kwave::SampleFormat new_format)
 void Kwave::RecordPlugin::buffersChanged()
 {
     InhibitRecordGuard _lock(*this); // don't record while settings change
-    // this implicitely activates the new settings
+    // this implicitly activates the new settings
 }
 
 //***************************************************************************
@@ -868,13 +880,13 @@ void Kwave::RecordPlugin::setupRecordThread()
     m_prerecording_queue.clear();
     if (params.pre_record_enabled) {
 	// prepare a queue for each track
-	const unsigned int prerecording_samples = static_cast<unsigned int>(
+	const unsigned int prerecording_samples = Kwave::toUint(
 	    rint(params.pre_record_time * params.sample_rate));
 	m_prerecording_queue.resize(params.tracks);
 	for (int i=0; i < m_prerecording_queue.size(); i++)
 	    m_prerecording_queue[i].setSize(prerecording_samples);
 
-	if (m_prerecording_queue.size() != static_cast<int>(params.tracks)) {
+	if (m_prerecording_queue.size() != Kwave::toInt(params.tracks)) {
 	    m_prerecording_queue.clear();
 	    Kwave::MessageBox::sorry(m_dialog, i18n("Out of memory"));
 	    return;
@@ -918,30 +930,29 @@ void Kwave::RecordPlugin::startRecording()
 	 * -> start over with a new signal and new settings
 	 */
 	if ((!m_writers) ||
-	    (m_writers->tracks() != tracks) ||
-	    (Kwave::FileInfo(signalManager().metaData()).rate() != rate))
+	    (m_writers->tracks() != tracks) || !qFuzzyCompare(
+	     Kwave::FileInfo(signalManager().metaData()).rate(), rate))
 	{
 	    // create a new and empty signal
 
 	    emitCommand(QString(_("newsignal(%1,%2,%3,%4)")).arg(
 		samples).arg(rate).arg(bits).arg(tracks));
 	    Kwave::SignalManager &mgr = signalManager();
-	    if ((mgr.rate() != rate) || (mgr.bits() != bits) ||
+	    if (!qFuzzyCompare(mgr.rate(), rate) || (mgr.bits() != bits) ||
 	        (mgr.tracks() != tracks))
 	    {
 		emitCommand(_("close"));
 		return;
 	    }
 
-	    // we do not need UNDO here
+	    // we do not need undo while recording, this would only waste undo
+	    // buffers with modified/inserted data
 	    signalManager().disableUndo();
 
 	    // create a sink for our audio data
 	    if (m_writers) delete m_writers;
-	    m_writers = new Kwave::MultiTrackWriter(signalManager(),
-	                                            Kwave::Append);
-	    Q_ASSERT(m_writers);
-	    Q_ASSERT((m_writers) && (m_writers->tracks() == tracks));
+	    m_writers = new(std::nothrow) Kwave::MultiTrackWriter(
+	        signalManager(), Kwave::Append);
 	    if ((!m_writers) || (m_writers->tracks() != tracks)) {
 		Kwave::MessageBox::sorry(m_dialog, i18n("Out of memory"));
 		return;
@@ -990,21 +1001,21 @@ void Kwave::RecordPlugin::recordStopped(int reason)
     if (reason >= 0) return; // nothing to do
 
     // recording was aborted
-    QString description;
+    QString err_msg;
     switch (reason) {
 	case -ENOBUFS:
-	    description = i18n("Buffer overrun. Please increase the "\
-	                       "number and/or size of the record buffers.");
+	    err_msg = i18n("Buffer overrun. Please increase the "\
+	                    "number and/or size of the record buffers.");
 	    break;
 	case -EBUSY:
-	    description = i18n("The recording device seems to be busy.");
+	    err_msg = i18n("The recording device seems to be busy.");
 	    break;
 	default:
-	    description = i18n("Reading from the recording device failed. "\
-	                       "Error number = %1 (%2)", -reason,
-			       QString::fromLocal8Bit(strerror(-reason)));
+	    err_msg = i18n("Reading from the recording device failed. "\
+	                   "Error number = %1 (%2)", -reason,
+	                    QString::fromLocal8Bit(strerror(-reason)));
     }
-    Kwave::MessageBox::error(m_dialog, description);
+    Kwave::MessageBox::error(m_dialog, err_msg);
 
     if (m_writers) m_writers->flush();
     qDebug("RecordPlugin::recordStopped(): last=%lu",
@@ -1013,6 +1024,16 @@ void Kwave::RecordPlugin::recordStopped(int reason)
 
     // flush away all prerecording buffers
     m_prerecording_queue.clear();
+
+    // update the file info if we recorded something
+    // NOTE: this implicitly sets the "modified" flag of the signal
+    if (m_writers && m_writers->last()) {
+	Kwave::FileInfo info(signalManager().metaData());
+	info.setLength(signalLength());
+	info.setTracks(m_dialog->params().tracks);
+	signalManager().setFileInfo(info, false);
+    }
+
 }
 
 //***************************************************************************
@@ -1108,7 +1129,7 @@ void Kwave::RecordPlugin::split(QByteArray &raw_data, QByteArray &dest,
 	    case 1: {
 		// 1...8 bits per sample, use 8 bit pointers
 		const quint8 *src =
-		    reinterpret_cast<quint8 *>(raw_data.data());
+		    reinterpret_cast<const quint8 *>(raw_data.constData());
 		quint8 *dst =
 		    reinterpret_cast<quint8 *>(dest.data());
 		src += track;
@@ -1122,7 +1143,7 @@ void Kwave::RecordPlugin::split(QByteArray &raw_data, QByteArray &dest,
 	    case 2: {
 		// 9...16 bits per sample, use 16 bit pointers
 		const quint16 *src =
-		    reinterpret_cast<const quint16 *>(raw_data.data());
+		    reinterpret_cast<const quint16 *>(raw_data.constData());
 		quint16 *dst =
 		    reinterpret_cast<quint16 *>(dest.data());
 		src += track;
@@ -1136,7 +1157,7 @@ void Kwave::RecordPlugin::split(QByteArray &raw_data, QByteArray &dest,
 	    case 3: {
 		// 17...24 bits per sample, use 8 bit pointers, three times
 		const quint8 *src =
-		    reinterpret_cast<quint8 *>(raw_data.data());
+		    reinterpret_cast<const quint8 *>(raw_data.constData());
 		quint8 *dst =
 		    reinterpret_cast<quint8 *>(dest.data());
 		src += track * 3;
@@ -1151,7 +1172,7 @@ void Kwave::RecordPlugin::split(QByteArray &raw_data, QByteArray &dest,
 	    case 4: {
 		// 24...32 bits per sample, use 32 bit pointers
 		const quint32 *src =
-		    reinterpret_cast<quint32 *>(raw_data.data());
+		    reinterpret_cast<const quint32 *>(raw_data.constData());
 		quint32 *dst =
 		    reinterpret_cast<quint32 *>(dest.data());
 		src += track;
@@ -1165,7 +1186,7 @@ void Kwave::RecordPlugin::split(QByteArray &raw_data, QByteArray &dest,
 	    case 8: {
 		// 64 bits per sample, use 64 bit pointers
 		const quint64 *src =
-		    reinterpret_cast<const quint64 *>(raw_data.data());
+		    reinterpret_cast<const quint64 *>(raw_data.constData());
 		quint64 *dst =
 		    reinterpret_cast<quint64 *>(dest.data());
 		src += track;
@@ -1177,9 +1198,9 @@ void Kwave::RecordPlugin::split(QByteArray &raw_data, QByteArray &dest,
 		break;
 	    }
 	    default: {
-		// default: bytewise operation
+		// default: byte wise operation
 		const quint8 *src =
-		    reinterpret_cast<const quint8 *>(raw_data.data());
+		    reinterpret_cast<const quint8 *>(raw_data.constData());
 		quint8 *dst =
 		    reinterpret_cast<quint8 *>(dest.data());
 		src += (track * bytes_per_sample);
@@ -1199,7 +1220,7 @@ void Kwave::RecordPlugin::split(QByteArray &raw_data, QByteArray &dest,
 
 //***************************************************************************
 bool Kwave::RecordPlugin::checkTrigger(unsigned int track,
-                                       Kwave::SampleArray &buffer)
+                                       const Kwave::SampleArray &buffer)
 {
     Q_ASSERT(m_dialog);
     if (!m_dialog) return false;
@@ -1216,7 +1237,7 @@ bool Kwave::RecordPlugin::checkTrigger(unsigned int track,
     // check the input parameters
     if (!buffer.size()) return false;
     if (!m_writers) return false;
-    if (m_trigger_value.size() != static_cast<int>(m_writers->tracks()))
+    if (m_trigger_value.size() != Kwave::toInt(m_writers->tracks()))
 	return false;
 
     // pass the buffer through a rectifier and a lowpass with
@@ -1246,23 +1267,23 @@ bool Kwave::RecordPlugin::checkTrigger(unsigned int track,
      */
 
     // rise coefficient: ~20Hz
-    const float f_rise = 20.0;
+    const float f_rise = 20.0f;
     float Fg = f_rise / rate;
-    float n = 1.0 / tan(M_PI * Fg);
-    const float a0_r = 1.0 / (1.0 + n);
-    const float b1_r = (1.0 - n) / (1.0 + n);
+    float n = 1.0f / tanf(float(M_PI) * Fg);
+    const float a0_r = 1.0f / (1.0f + n);
+    const float b1_r = (1.0f - n) / (1.0f + n);
 
     // fall coefficient: ~1.0Hz
-    const float f_fall = 1.0;
+    const float f_fall = 1.0f;
     Fg = f_fall / rate;
-    n = 1.0 / tan(M_PI * Fg);
-    const float a0_f = 1.0 / (1.0 + n);
-    const float b1_f = (1.0 - n) / (1.0 + n);
+    n = 1.0f / tanf(float(M_PI) * Fg);
+    const float a0_f = 1.0f / (1.0f + n);
+    const float b1_f = (1.0f - n) / (1.0f + n);
 
     float y = m_trigger_value[track];
     float last_x = y;
-    for (unsigned int t=0; t < buffer.size(); ++t) {
-	float x = fabs(sample2float(buffer[t])); /* rectifier */
+    for (unsigned int t = 0; t < buffer.size(); ++t) {
+	float x = fabsf(sample2float(buffer[t])); /* rectifier */
 
 	if (x > y) { /* diode */
 	    // rise if amplitude is above average (serial R)
@@ -1291,9 +1312,9 @@ void Kwave::RecordPlugin::enqueuePrerecording(unsigned int track,
                                               const Kwave::SampleArray &decoded)
 {
     Q_ASSERT(m_dialog);
-    Q_ASSERT(static_cast<int>(track) < m_prerecording_queue.size());
+    Q_ASSERT(Kwave::toInt(track) < m_prerecording_queue.size());
     if (!m_dialog) return;
-    if (static_cast<int>(track) >= m_prerecording_queue.size()) return;
+    if (Kwave::toInt(track) >= m_prerecording_queue.size()) return;
 
     // append the array with decoded sample to the prerecording buffer
     m_prerecording_queue[track].put(decoded);
@@ -1380,15 +1401,16 @@ void Kwave::RecordPlugin::processBuffer()
 
     // check for reached recording time limit if enabled
     if (params.record_time_limited && m_writers) {
-	const unsigned int last = m_writers->last();
-	const unsigned int already_recorded = (last) ? (last + 1) : 0;
-	const unsigned int limit = static_cast<const unsigned int>(rint(
+	const sample_index_t last = m_writers->last();
+	const sample_index_t already_recorded = (last) ? (last + 1) : 0;
+	const sample_index_t limit = static_cast<const sample_index_t>(rint(
 	    params.record_time * params.sample_rate));
 	if (already_recorded + samples >= limit) {
 	    // reached end of recording time, we are full
 	    if (m_state == Kwave::REC_RECORDING) {
-		samples = (limit > already_recorded) ?
-		          (limit - already_recorded) : 0;
+		samples = Kwave::toUint(
+		    (limit > already_recorded) ?
+		    (limit - already_recorded) : 0);
 		buffer.resize(samples * tracks * bytes_per_sample);
 	    }
 	    recording_done = true;
@@ -1397,8 +1419,8 @@ void Kwave::RecordPlugin::processBuffer()
 
     QByteArray buf;
     buf.resize(bytes_per_sample * samples);
-    Q_ASSERT(buf.size() == static_cast<int>(bytes_per_sample * samples));
-    if (buf.size() != static_cast<int>(bytes_per_sample * samples)) return;
+    Q_ASSERT(buf.size() == Kwave::toInt(bytes_per_sample * samples));
+    if (buf.size() != Kwave::toInt(bytes_per_sample * samples)) return;
 
     Kwave::SampleArray decoded(samples);
     Q_ASSERT(decoded.size() == samples);
@@ -1463,8 +1485,7 @@ void Kwave::RecordPlugin::processBuffer()
 		break;
 	    case Kwave::REC_RECORDING: {
 		// put the decoded track data into the buffer
-		Q_ASSERT(m_writers);
-		if (!m_writers) break;
+		if (!m_writers) break; // (could happen due to queued signal)
 		Q_ASSERT(tracks == m_writers->tracks());
 		if (!tracks || (tracks != m_writers->tracks())) break;
 
@@ -1479,7 +1500,7 @@ void Kwave::RecordPlugin::processBuffer()
     }
 
     // update the number of recorded samples
-    if (m_writers) emit sigRecordedSamples(m_writers->last()+1);
+    if (m_writers) emit sigRecordedSamples(m_writers->last() + 1);
 
     // if this was the last received buffer, change state
     if (recording_done &&
