@@ -67,7 +67,7 @@ namespace Kwave
     signals:
 
 	/** emitted to promote the recording progress to the dialog */
-	void sigRecordedSamples(unsigned int samples_recorded);
+	void sigRecordedSamples(sample_index_t samples_recorded);
 
     protected:
 
@@ -83,13 +83,13 @@ namespace Kwave
 		:m_recorder(recorder)
 	    {
 		m_recorder.enterInhibit();
-	    };
+	    }
 
 	    /** Destructor, re-enables recording */
 	    virtual ~InhibitRecordGuard()
 	    {
 		m_recorder.leaveInhibit();
-	    };
+	    }
 
 	private:
 	    Kwave::RecordPlugin &m_recorder;
@@ -115,7 +115,7 @@ namespace Kwave
 
 	/**
 	 * command for starting the recording, completion is
-	 * signalled with sigStarted()
+	 * signaled with sigStarted()
 	 */
 	void startRecording();
 
@@ -161,7 +161,7 @@ namespace Kwave
 	/** restart recorder with new buffer settings */
 	void buffersChanged();
 
-	/** the prerecording checkbos has changed */
+	/** the prerecording checkbox has changed */
 	void prerecordingChanged(bool enable);
 
     private:
@@ -188,7 +188,7 @@ namespace Kwave
 	 * @param buffer array with Kwave sample data
 	 * @return true if trigger reached or no trigger set
 	 */
-	bool checkTrigger(unsigned int track, Kwave::SampleArray &buffer);
+	bool checkTrigger(unsigned int track, const Kwave::SampleArray &buffer);
 
 	/**
 	 * Split off one track from a raw buffer with multiple tracks into
@@ -271,7 +271,7 @@ namespace Kwave
 	unsigned int m_inhibit_count;
 
 	/** buffer for trigger values */
-	QVector<double> m_trigger_value;
+	QVector<float> m_trigger_value;
 
     };
 }
