@@ -23,14 +23,15 @@
 
 #include "libkwave/Sample.h"
 #include "libkwave/SampleFormat.h"
+#include "libkwave/Utils.h"
 
 #include "SampleDecoderLinear.h"
 
 //***************************************************************************
-void decode_NULL(const quint8 *src, sample_t *dst, unsigned int count)
+static void decode_NULL(const quint8 *src, sample_t *dst, unsigned int count)
 {
     while (count--) {
-        printf("%02X ", static_cast<int>(*src));
+        printf("%02X ", Kwave::toInt(*src));
         *(dst++) = count % (1 << (SAMPLE_BITS-1));
     }
 }
@@ -168,7 +169,7 @@ void Kwave::SampleDecoderLinear::decode(QByteArray &raw_data,
     if (!m_decoder) return;
 
     unsigned int samples = raw_data.size() / m_bytes_per_sample;
-    const quint8 *src  = reinterpret_cast<const quint8 *>(raw_data.data());
+    const quint8 *src  = reinterpret_cast<const quint8 *>(raw_data.constData());
     sample_t *dst = decoded.data();
 
     m_decoder(src, dst, samples);
