@@ -63,6 +63,33 @@ namespace Kwave
 	 */
 	virtual void updateProgress(qreal progress);
 
+    private:
+
+	/**
+	 * common slice paramters, which can be passed to all worker
+	 * threads
+	 */
+	typedef struct {
+	    sample_index_t m_first;      /**< index of the first sample */
+	    sample_index_t m_last;       /**< index of the last sample  */
+	    unsigned int   m_block_size; /**< block size [samples]      */
+	} SliceParams;
+
+	/**
+	 * reverse a slice of samples
+	 * @param track index of the track
+	 * @param src_a reader for reading from the start (forward)
+	 * @param src_b reader fro reading from the end (backwards)
+	 * @param params common slice parameters (first/last/block size)
+	 */
+	void reverseSlice(unsigned int track,
+	                  Kwave::SampleReader *src_a,
+	                  Kwave::SampleReader *src_b,
+	                  const Kwave::ReversePlugin::SliceParams &params);
+
+	/** reverses the content of an array of samples */
+	void reverse(Kwave::SampleArray &buffer);
+
     };
 }
 
