@@ -19,9 +19,6 @@
 #include <errno.h>
 
 #include <klocale.h> // for the i18n macro
-#include <threadweaver/Job.h>
-#include <threadweaver/ThreadWeaver.h>
-#include <threadweaver/DebuggingAids.h>
 
 #include <QtCore/QList>
 #include <QtCore/QListIterator>
@@ -34,6 +31,7 @@
 #include "libkwave/MultiTrackWriter.h"
 #include "libkwave/PluginManager.h"
 #include "libkwave/SignalManager.h"
+#include "libkwave/Utils.h"
 #include "libkwave/Writer.h"
 #include "libkwave/modules/RateConverter.h"
 #include "libkwave/undo/UndoTransactionGuard.h"
@@ -96,7 +94,7 @@ void Kwave::SampleRatePlugin::run(QStringList params)
 	return;
 
     double old_rate = Kwave::FileInfo(signalManager().metaData()).rate();
-    if ((old_rate <= 0) || (old_rate == m_new_rate)) return;
+    if ((old_rate <= 0) || qFuzzyCompare(old_rate, m_new_rate)) return;
 
     Kwave::UndoTransactionGuard undo_guard(*this, i18n("Change sample rate"));
 
@@ -112,7 +110,7 @@ void Kwave::SampleRatePlugin::run(QStringList params)
     } else {
 	length = selection(&tracks, &first, &last, true);
 	if ((length == signalLength()) &&
-	    (tracks.count() == static_cast<int>(mgr.tracks())))
+	    (tracks.count() == Kwave::toInt(mgr.tracks())))
 	{
 	    // manually selected the whole signal
 	    m_whole_signal = true;
@@ -126,7 +124,7 @@ void Kwave::SampleRatePlugin::run(QStringList params)
 
     // calculate the new length
     double ratio = m_new_rate / old_rate;
-    sample_index_t new_length = length * ratio;
+    sample_index_t new_length = static_cast<sample_index_t>(length * ratio);
     if ((new_length == length) || !new_length) return;
 
     // if the new length is bigger than the current length,
