@@ -126,7 +126,7 @@ QStringList *Kwave::SaveBlocksPlugin::setup(QStringList &previous_params)
 	);
     } else {
 	// user pressed "Cancel"
-	if (list) delete list;
+	delete list;
 	list = 0;
     }
 
