@@ -20,23 +20,40 @@
 
 #include "config.h"
 
+#include <QtCore/QByteArray>
+#include <QtCore/QBitArray>
+#include <QtCore/QFuture>
 #include <QtCore/QList>
+#include <QtCore/QMutex>
+#include <QtCore/QQueue>
+#include <QtCore/QReadWriteLock>
 #include <QtCore/QString>
+#include <QtCore/QTimer>
+#include <QtCore/QUuid>
 
+#include "libkwave/FixedPool.h"
 #include "libkwave/Plugin.h"
 #include "libkwave/WindowFunction.h"
 
 class QImage;
 class QStringList;
 
+/** maximum number of FFT points */
+#define MAX_FFT_POINTS 32767
+
+/** maximum number of concurrent FFT jobs */
+#define MAX_FFT_JOBS 256
+
+/** maximum number of slices (width of the image) */
+#define MAX_SLICES 32767
 
 namespace Kwave
 {
     class MultiTrackReader;
     class OverViewCache;
     class PluginContext;
+    class SelectionTracker;
     class SonagramWindow;
-    class StripeInfoPrivate;
 
     /**
      * plugin that shows a sonagram window
@@ -61,19 +78,45 @@ namespace Kwave
 	virtual int start(QStringList &params);
 
 	/**
-	 * Runns once until all stripes of the sonagram are
+	 * Runns once until all slices of the sonagram are
 	 * calculated.
 	 * @param params list of strings with parameters
 	 * @see Kwave::Plugin::run()
 	 */
 	virtual void run(QStringList params);
 
+    private:
+
+	/** internal data of a slice */
+	typedef struct {
+	    /** index of the slice */
+	    unsigned int m_index;
+
+	    /** array with input samples */
+	    double m_input[MAX_FFT_POINTS];
+
+	    /** FFT output data */
+	    fftw_complex m_output[MAX_FFT_POINTS];
+
+	    /** rendered FFT result data */
+	    char m_result[MAX_FFT_POINTS];
+	} Slice;
+
     signals:
 
-	void stripeAvailable(Kwave::StripeInfoPrivate *stripe_info);
+	/**
+	 * emitted when a new slice has been calculated in calculateSlice()
+	 * @param slice the slice data container, including result
+	 */
+	void sliceAvailable(Kwave::SonagramPlugin::Slice *slice);
 
     private slots:
 
+	/**
+	 * validate the sonagram by calling makeAllValid in a background thread
+	 */
+	void validate();
+
 	/**
 	 * Connected to the SonagramWindow's "destroyed()" signal.
 	 * @see class SonagramWindow
@@ -82,17 +125,42 @@ namespace Kwave
 
 	/**
 	 * Internally used to synchronously insert the data of one
-	 * sonagram stripe int the current image and refresh the
+	 * sonagram slice int the current image and refresh the
 	 * display.
-	 * DO NOT CALL DIRECTLY!
+	 * @note DO NOT CALL DIRECTLY!
+	 * @param slice a slice data container, including result
 	 */
-	void insertStripe(Kwave::StripeInfoPrivate *stripe_info);
+	void insertSlice(Kwave::SonagramPlugin::Slice *slice);
 
 	/**
 	 * Updates the overview image under the sonagram
 	 */
 	void refreshOverview();
 
+	/**
+	 * Connected to the selection tracker's sigTrackInserted.
+	 * @param track_id unique ID of the track
+	 * @see SelectionTracker::sigTrackInserted
+	 */
+	void slotTrackInserted(const QUuid &track_id);
+
+	/**
+	 * Connected to the selection tracker's sigTrackInserted.
+	 * @param track_id unique ID of the track
+	 * @see SelectionTracker::sigTrackDeleted
+	 */
+	void slotTrackDeleted(const QUuid &track_id);
+
+	/**
+	 * Connected to the selection tracker's sigInvalidated.
+	 * @param track_id UUID of the track or null for "all tracks"
+	 * @param first index of the first invalidated sample
+	 * @param last index of the last invalidated sample
+	 */
+	void slotInvalidated(const QUuid *track_id,
+	                     sample_index_t first,
+	                     sample_index_t last);
+
     protected:
 
 	/**
@@ -105,45 +173,46 @@ namespace Kwave
 
     private:
 
+	/**
+	 * will be run in a background thread to make all stripes
+	 * valid.
+	 */
+	void makeAllValid();
+
+	/**
+	 * Requests an update of the sonagram or portions of it
+	 */
+	void requestValidation();
+
+	/**
+	 * do the FFT calculation on a slice
+	 * @param slice structure with the input data and output buffer
+	 */
+	void calculateSlice(Kwave::SonagramPlugin::Slice *slice);
+
 	/**
 	 * Creates a new image for the current processing.
 	 * If an old image exists, it will be deleted first, a new image
 	 * will not be created if either width or height is zero.
 	 * The image will get 8 bits depth and use a color or a greyscale
 	 * palette.
-	 * @param width number of horizontal pixels (stripes = signal
+	 * @param width number of horizontal pixels (slices = signal
 	 *        length / fft points, rounded up) [1..32767]
 	 * @param height number of vertical pixels (= fft points / 2) [1..32767]
 	 */
 	void createNewImage(const unsigned int width,
 	    const unsigned int height);
 
-	/**
-	 * Calculates the fft for one stripe. If the input data range runns
-	 * over the end of the signal or selection, zeroes will be used instead.
-	 * @param source a MultiTrackReader with the samples
-	 * @param points number of fft points
-	 * @param output reference to an array to receive the output
-	 */
-	void calculateStripe(Kwave::MultiTrackReader &source, const int points,
-	    QByteArray &output);
-
     private:
 
 	/** the main view of the plugin, a SonagramWindow */
 	Kwave::SonagramWindow *m_sonagram_window;
 
-	/** list of selected channels */
-	QList<unsigned int> m_selected_channels;
+	/** selection tracker */
+	Kwave::SelectionTracker *m_selection;
 
-	/** first sample of the selection, inclusive */
-	sample_index_t m_first_sample;
-
-	/** last sample of the selection, inclusive */
-	sample_index_t m_last_sample;
-
-	/** number of stripes (= width of the image in pixels) */
-	unsigned int m_stripes;
+	/** number of slices (= width of the image in pixels) */
+	unsigned int m_slices;
 
 	/** number of fft points */
 	unsigned int m_fft_points;
@@ -166,6 +235,23 @@ namespace Kwave
 	/** cache with the current signal overview */
 	Kwave::OverViewCache *m_overview_cache;
 
+	/** pool of slices */
+	Kwave::FixedPool<MAX_FFT_JOBS, Slice> m_slice_pool;
+
+	/** bit field with "is valid" a flag for each stripe */
+	QBitArray m_valid;
+
+	/** lock used for tracking running background jobs */
+	QReadWriteLock m_pending_jobs;
+
+	/** lock to protect the job list (m_valid) */
+	QMutex m_lock_job_list;
+
+	/** the currently running background job */
+	QFuture<void> m_future;
+
+	/** timer for refreshing the sonagram */
+	QTimer m_repaint_timer;
     };
 }
 
