@@ -19,6 +19,7 @@
 
 #include <math.h>
 #include <limits.h>
+#include <new>
 
 #include <QtGui/QBitmap>
 #include <QtGui/QImage>
@@ -87,14 +88,14 @@ Kwave::SonagramWindow::SonagramWindow(const QString &name)
 {
     KIconLoader icon_loader;
 
-    for (int i = 0; i < 256; m_histogram[i++] = 0) {}
+    for (int i = 0; i < 256; ++i) { m_histogram[i] = 0; }
 
-    QWidget *mainwidget = new QWidget(this);
+    QWidget *mainwidget = new(std::nothrow) QWidget(this);
     Q_ASSERT(mainwidget);
     if (!mainwidget) return;
     setCentralWidget(mainwidget);
 
-    QGridLayout *top_layout = new QGridLayout(mainwidget/*, 3, 2*/);
+    QGridLayout *top_layout = new(std::nothrow) QGridLayout(mainwidget/*, 3, 2*/);
     Q_ASSERT(top_layout);
     if (!top_layout) return;
 
@@ -135,31 +136,34 @@ Kwave::SonagramWindow::SonagramWindow(const QString &name)
     status->insertItem(i18n("Frequency: ------ Hz"), 2);
     status->insertItem(i18n("Amplitude: --- %"), 3);
 
-    m_view = new Kwave::ImageView(mainwidget);
+    m_view = new(std::nothrow) Kwave::ImageView(mainwidget);
     Q_ASSERT(m_view);
     if (!m_view) return;
     top_layout->addWidget(m_view, 0, 1);
     QPalette palette;
-    palette.setBrush(m_view->backgroundRole(), QBrush(background));
+    palette.setBrush(m_view->backgroundRole(), QBrush(QImage(background)));
+    m_view->setAutoFillBackground(true);
     m_view->setPalette(palette);
 
-    m_xscale = new Kwave::ScaleWidget(mainwidget, 0, 100, i18n("ms"));
+    m_xscale = new(std::nothrow)
+	Kwave::ScaleWidget(mainwidget, 0, 100, i18n("ms"));
     Q_ASSERT(m_xscale);
     if (!m_xscale) return;
     m_xscale->setFixedHeight(m_xscale->sizeHint().height());
     top_layout->addWidget(m_xscale, 1, 1);
 
-    m_yscale = new Kwave::ScaleWidget(mainwidget, 0, 100, i18n("Hz"));
+    m_yscale = new(std::nothrow)
+	Kwave::ScaleWidget(mainwidget, 0, 100, i18n("Hz"));
     Q_ASSERT(m_yscale);
     if (!m_yscale) return ;
     m_yscale->setFixedWidth(m_yscale->sizeHint().width());
     m_yscale->setMinimumHeight(9*6*5);
     top_layout->addWidget(m_yscale, 0, 0);
 
-    m_overview = new Kwave::ImageView(mainwidget);
+    m_overview = new(std::nothrow) Kwave::ImageView(mainwidget);
     Q_ASSERT(m_overview);
     if (!m_overview) return;
-    m_overview->setFixedHeight(30);
+    m_overview->setFixedHeight(SONAGRAM_OVERVIEW_HEIGHT);
     top_layout->addWidget(m_overview, 2, 1);
 
     connect(m_view, SIGNAL(sigCursorPos(const QPoint)),
@@ -260,12 +264,12 @@ void Kwave::SonagramWindow::setImage(QImage image)
     m_image = image;
 
     // re-initialize histogram over all pixels
-    for (int i=0; i < 256; i++)
+    for (unsigned int i = 0; i < 256; i++)
 	m_histogram[i] = 0;
     if (!m_image.isNull()) {
 	for (int x = 0; x < m_image.width(); x++) {
 	    for (int y = 0; y < m_image.height(); y++) {
-		unsigned char p = m_image.pixelIndex(x, y);
+		quint8 p = static_cast<quint8>(m_image.pixelIndex(x, y));
 		m_histogram[p]++;
 	    }
 	}
@@ -277,42 +281,41 @@ void Kwave::SonagramWindow::setImage(QImage image)
 //****************************************************************************
 void Kwave::SonagramWindow::setOverView(const QImage &overview)
 {
-    m_overview->setImage(overview);
+    if (m_overview) m_overview->setImage(overview);
 }
 
 //****************************************************************************
-void Kwave::SonagramWindow::insertStripe(const unsigned int stripe_nr,
-	const QByteArray &stripe)
+void Kwave::SonagramWindow::insertSlice(const unsigned int slice_nr,
+                                        const QByteArray &slice)
 {
     Q_ASSERT(m_view);
-    Q_ASSERT(!m_image.isNull());
     if (!m_view) return;
     if (m_image.isNull()) return;
 
     unsigned int image_width  = m_image.width();
     unsigned int image_height = m_image.height();
 
-    // stripe is out of range ?
-    if (stripe_nr >= image_width) return;
+    // slice is out of range ?
+    if (slice_nr >= image_width) return;
 
     unsigned int y;
-    unsigned int size = stripe.size();
+    unsigned int size = slice.size();
     for (y = 0; y < size; y++) {
-	unsigned char p;
+	quint8 p;
 
 	// remove the current pixel from the histogram
-	p = m_image.pixelIndex(stripe_nr, y);
+	p = static_cast<quint8>(m_image.pixelIndex(slice_nr, y));
 	m_histogram[p]--;
 
 	// set the new pixel value
-	p = static_cast<int>(stripe[(size - 1) - y]);
-	m_image.setPixel(stripe_nr, y, p);
+	p = slice[(size - 1) - y];
+	m_image.setPixel(slice_nr, y, p);
 
 	// insert the new pixel into the histogram
 	m_histogram[p]++;
     }
     while (y < image_height) { // fill the rest with blank
-	m_image.setPixel(stripe_nr, y++, 0xFE);
+	m_image.setPixel(slice_nr, y++, 0xFE);
 	m_histogram[0xFE]++;
     }
 
@@ -329,46 +332,42 @@ void Kwave::SonagramWindow::adjustBrightness()
 
     // get the sum of pixels != 0
     unsigned long int sum = 0;
-    for (int i=0; i < 254; i++) {
+    for (unsigned int i = 1; i <= 254; i++)
 	sum += m_histogram[i];
-    }
-    // cut off all parts below the cutoff ratio (e.g. 0.1%)
-    unsigned int cutoff = static_cast<int>(sum * COLOR_CUTOFF_RATIO);
 
-    // get the first used color
-    unsigned int first=0;
-    while ((first < 253) && (m_histogram[first] <= cutoff)) {
-	first++;
-    }
+    // cut off all parts below the cutoff ratio (e.g. 0.1%)
+    unsigned int cutoff = Kwave::toUint(sum * COLOR_CUTOFF_RATIO);
+
+    // get the last used color from the histogram
+    int last = 254;
+    while ((last >= 0) && (m_histogram[last] <= cutoff))
+	last--;
 
     QColor c;
-    for (unsigned int i=0; i < 255; i++) {
+    for (int i = 0; i < 255; i++) {
 	int v;
 
-	if (i <= first) {
-	    v = 0;
+	if (i >= last) {
+	    v = 254;
 	} else {
-	    // map [first...254] to [0...254]
-	    v = (i-first)*254/(254-first);
+	    // map [0...last] to [254...0]
+	    v = ((last - i) * 254) / last;
 	}
 
 	if (m_color_mode == 1) {
 	    // rainbow effect
-	    c.setHsv( (v*255)/256, 255, 255 );
+	    c.setHsv( (v * 255) / 255, 255, 255, 255);
 	} else {
 	    // greyscale palette
-	    c.setRgb(v, v, v);
+	    c.setRgb(v, v, v, 255);
 	}
 
-	c.setAlpha(255);
-	m_image.setColor(i, c.rgb());
-// 	qDebug("color[%3d] = 0x%08X",i, c.rgb());
+	m_image.setColor(i, c.rgba());
+// 	qDebug("color[%3d] = 0x%08X",i, c.rgba());
     }
 
     // use color 0xFF for transparency !
-    c.setRgb(0,0,0);
-    c.setAlpha(0);
-    m_image.setColor(255, c.rgb());
+    m_image.setColor(0xFF, QColor(0, 0, 0, 0).rgba());
 }
 
 //****************************************************************************
@@ -444,11 +443,12 @@ void Kwave::SonagramWindow::toSignal()
 }
 
 //***************************************************************************
-void Kwave::SonagramWindow::translatePixels2TF(const QPoint p, double *ms, double *f)
+void Kwave::SonagramWindow::translatePixels2TF(const QPoint p,
+                                               double *ms, double *f)
 {
     if (ms) {
 	// get the time coordinate [0...(N_samples-1)* (1/f_sample) ]
-	if (m_rate != 0) {
+	if (!qFuzzyIsNull(m_rate)) {
 	    *ms = static_cast<double>(p.x()) *
 	          static_cast<double>(m_points) * 1000.0 / m_rate;
 	} else {
@@ -461,7 +461,7 @@ void Kwave::SonagramWindow::translatePixels2TF(const QPoint p, double *ms, doubl
 	double py = (m_points >= 2) ? (m_points / 2) - 1 : 0;
 	double y = py - p.y();
 	if (y < 0) y = 0;
-	*f = y / py * (m_rate/2.0);
+	*f = y / py * (m_rate / 2.0);
     }
 }
 
@@ -471,10 +471,10 @@ void Kwave::SonagramWindow::updateScaleWidgets()
     double ms;
     double f;
 
-    translatePixels2TF(QPoint(m_image.width()-1, 0), &ms, &f);
+    translatePixels2TF(QPoint(m_image.width() - 1, 0), &ms, &f);
 
-    m_xscale->setMinMax(0, static_cast<int>(rint(ms)));
-    m_yscale->setMinMax(0, static_cast<int>(rint(f)));
+    m_xscale->setMinMax(0, Kwave::toInt(rint(ms)));
+    m_yscale->setMinMax(0, Kwave::toInt(rint(f)));
 }
 
 //***************************************************************************
@@ -509,11 +509,11 @@ void Kwave::SonagramWindow::cursorPosChanged(const QPoint pos)
     KStatusBar *status = statusBar();
     Q_ASSERT(status);
     Q_ASSERT(m_points);
-    Q_ASSERT(m_rate != 0);
+    Q_ASSERT(!qFuzzyIsNull(m_rate));
     if (!status) return;
     if (m_image.isNull()) return;
     if (!m_points) return;
-    if (m_rate == 0) return;
+    if (qFuzzyIsNull(m_rate)) return;
 
     double ms;
     double f;
@@ -524,17 +524,16 @@ void Kwave::SonagramWindow::cursorPosChanged(const QPoint pos)
     status->changeItem(i18n("Time: %1", Kwave::ms2string(ms)), 1);
 
     // item 2: frequency in Hz
-    QString text = i18n("Frequency: %1 Hz", static_cast<int>(f));
+    QString text = i18n("Frequency: %1 Hz", Kwave::toInt(f));
     status->changeItem(text, 2);
 
     // item 3: amplitude in %
     if (m_image.valid(pos.x(), pos.y())) {
-	a = (254.0 - m_image.pixelIndex(pos.x(), pos.y())) *
-	    (100.0 / 254.0);
+	a = m_image.pixelIndex(pos.x(), pos.y()) * (100.0 / 254.0);
     } else {
 	a = 0.0;
     }
-    text = i18n("Amplitude: %1%", static_cast<int>(a));
+    text = i18n("Amplitude: %1%", Kwave::toInt(a));
     status->changeItem(text, 3);
 }
 
